@@ -5,6 +5,7 @@
 
 #include "build.h"
 #include "install.h"
+#include "debug.h"
 
 static int checkSpec(Header h)
 {
@@ -22,8 +23,8 @@ static int checkSpec(Header h)
     if (rpmdbOpen(rootdir, &db, mode, 0644)) {
 	const char *dn;
 	dn = rpmGetPath( (rootdir ? rootdir : ""), "%{_dbpath}", NULL);
-	rpmMessage(RPMMESS_ERROR, _("cannot open %s/packages.rpm\n"), dn);
-	xfree(dn);
+	rpmError(RPMERR_OPEN, _("cannot open rpm database in %s\n"), dn);
+	free((void *)dn);
 	exit(EXIT_FAILURE);
     }
     ts = rpmtransCreateSet(db, rootdir);
@@ -61,7 +62,8 @@ static int isSpecFile(const char *specfile)
 
     fd = Fopen(specfile, "r.ufdio");
     if (fd == NULL || Ferror(fd)) {
-	fprintf(stderr, _("Unable to open spec file %s: %s\n"), specfile, Fstrerror(fd));
+	rpmError(RPMERR_OPEN, _("Unable to open spec file %s: %s\n"),
+		specfile, Fstrerror(fd));
 	return 0;
     }
     count = Fread(buf, sizeof(buf[0]), sizeof(buf), fd);
@@ -86,12 +88,10 @@ static int isSpecFile(const char *specfile)
 }
 
 static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
-	const char *passPhrase, int fromTarball, char *cookie,
-	int force, int nodeps)
+	const char *passPhrase, char *cookie)
 {
     int buildAmount = ba->buildAmount;
     const char *buildRootURL = NULL;
-    int test = ba->noBuild;
     const char * specFile;
     const char * specURL;
     int specut;
@@ -99,17 +99,19 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
     Spec spec = NULL;
     int rc;
 
+#ifndef	DYING
     rpmSetTables(RPM_MACHTABLE_BUILDARCH, RPM_MACHTABLE_BUILDOS);
+#endif
 
     if (ba->buildRootOverride)
 	buildRootURL = rpmGenPath(NULL, ba->buildRootOverride, NULL);
 
-    if (fromTarball) {
+    if (ba->buildMode == 't') {
 	FILE *fp;
 	const char *specDir;
 	const char * tmpSpecFile;
 	char * cmd, *s;
-	int res;
+	rpmCompressedMagic res = COMPRESSED_OTHER;
 	static const char *zcmds[] = { "cat", "gunzip", "bunzip2", "cat" };
 
 	specDir = rpmGetPath("%{_specdir}", NULL);
@@ -125,10 +127,9 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	sprintf(cmd, "%s < %s | tar xOvf - Specfile 2>&1 > %s",
 			zcmds[res & 0x3], arg, tmpSpecFile);
 	if (!(fp = popen(cmd, "r"))) {
-	    fprintf(stderr, _("Failed to open tar pipe: %s\n"), 
-			strerror(errno));
-	    xfree(specDir);
-	    xfree(tmpSpecFile);
+	    rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
+	    free((void *)specDir);
+	    free((void *)tmpSpecFile);
 	    return 1;
 	}
 	if ((!fgets(buf, sizeof(buf) - 1, fp)) || !strchr(buf, '/')) {
@@ -138,18 +139,18 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	    sprintf(cmd, "%s < %s | tar xOvf - \\*.spec 2>&1 > %s",
 		    zcmds[res & 0x3], arg, tmpSpecFile);
 	    if (!(fp = popen(cmd, "r"))) {
-		fprintf(stderr, _("Failed to open tar pipe: %s\n"), 
-			strerror(errno));
-		xfree(specDir);
-		xfree(tmpSpecFile);
+		rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
+		free((void *)specDir);
+		free((void *)tmpSpecFile);
 		return 1;
 	    }
 	    if (!fgets(buf, sizeof(buf) - 1, fp)) {
 		/* Give up */
-		fprintf(stderr, _("Failed to read spec file from %s\n"), arg);
+		rpmError(RPMERR_READ, _("Failed to read spec file from %s\n"),
+			arg);
 		unlink(tmpSpecFile);
-		xfree(specDir);
-		xfree(tmpSpecFile);
+		free((void *)specDir);
+		free((void *)tmpSpecFile);
 	    	return 1;
 	    }
 	}
@@ -170,15 +171,16 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	specURL = s = alloca(strlen(specDir) + strlen(cmd) + 5);
 	sprintf(s, "%s/%s", specDir, cmd);
 	res = rename(tmpSpecFile, s);
-	xfree(specDir);
-	xfree(tmpSpecFile);
+	free((void *)specDir);
 	
 	if (res) {
-	    fprintf(stderr, _("Failed to rename %s to %s: %s\n"),
-		    tmpSpecFile, s, strerror(errno));
+	    rpmError(RPMERR_RENAME, _("Failed to rename %s to %s: %m\n"),
+			tmpSpecFile, s);
 	    unlink(tmpSpecFile);
+	    free((void *)tmpSpecFile);
 	    return 1;
 	}
+	free((void *)tmpSpecFile);
 
 	/* Make the directory which contains the tarball the source 
 	   directory for this run */
@@ -210,17 +212,22 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 
     if (specut != URL_IS_DASH) {
 	struct stat st;
-	Stat(specURL, &st);
+	if (Stat(specURL, &st) < 0) {
+	    rpmError(RPMERR_STAT, _("failed to stat %s: %m\n"), specURL);
+	    rc = 1;
+	    goto exit;
+	}
 	if (! S_ISREG(st.st_mode)) {
-	    fprintf(stderr, _("File is not a regular file: %s\n"), specURL);
+	    rpmError(RPMERR_NOTREG, _("File %s is not a regular file.\n"),
+		specURL);
 	    rc = 1;
 	    goto exit;
 	}
 
 	/* Try to verify that the file is actually a specfile */
 	if (!isSpecFile(specURL)) {
-	    fprintf(stderr, _("File %s does not appear to be a specfile.\n"),
-		specURL);
+	    rpmError(RPMERR_BADSPEC,
+		_("File %s does not appear to be a specfile.\n"), specURL);
 	    rc = 1;
 	    goto exit;
 	}
@@ -230,7 +237,7 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 #define	_anyarch(_f)	\
 (((_f)&(RPMBUILD_PREP|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
     if (parseSpec(&spec, specURL, ba->rootdir, buildRootURL, 0, passPhrase,
-		cookie, _anyarch(buildAmount), force)) {
+		cookie, _anyarch(buildAmount), ba->force)) {
 	rc = 1;
 	goto exit;
     }
@@ -240,31 +247,29 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
     initSourceHeader(spec);
 
     /* Check build prerequisites */
-    if (!nodeps && checkSpec(spec->sourceHeader)) {
+    if (!ba->noDeps && checkSpec(spec->sourceHeader)) {
 	rc = 1;
 	goto exit;
     }
 
-    if (buildSpec(spec, buildAmount, test)) {
+    if (buildSpec(spec, buildAmount, ba->noBuild)) {
 	rc = 1;
 	goto exit;
     }
     
-    if (fromTarball) Unlink(specURL);
+    if (ba->buildMode == 't') Unlink(specURL);
     rc = 0;
 
 exit:
     if (spec)
 	freeSpec(spec);
     if (buildRootURL)
-	xfree(buildRootURL);
+	free((void *)buildRootURL);
     return rc;
 }
 
-/** */
 int build(const char * arg, struct rpmBuildArguments * ba,
-	const char * passPhrase, int fromTarball, char * cookie,
-	const char * rcfile, int force, int nodeps)
+	const char * passPhrase, char * cookie, const char * rcfile)
 {
     char *t, *te;
     int rc = 0;
@@ -273,8 +278,7 @@ int build(const char * arg, struct rpmBuildArguments * ba,
     int cleanFlags = ba->buildAmount & buildCleanMask;
 
     if (targets == NULL) {
-	rc =  buildForTarget(arg, ba, passPhrase, fromTarball, cookie,
-		force, nodeps);
+	rc =  buildForTarget(arg, ba, passPhrase, cookie);
 	goto exit;
     }
 
@@ -300,8 +304,7 @@ int build(const char * arg, struct rpmBuildArguments * ba,
 	/* Read in configuration for target. */
 	rpmFreeMacros(NULL);
 	rpmReadConfigFiles(rcfile, target);
-	rc = buildForTarget(arg, ba, passPhrase, fromTarball, cookie,
-		force, nodeps);
+	rc = buildForTarget(arg, ba, passPhrase, cookie);
 	if (rc)
 	    break;
     }
@@ -312,92 +315,3 @@ exit:
     rpmReadConfigFiles(rcfile, NULL);
     return rc;
 }
-
-#define	POPT_USECATALOG		1000
-#define	POPT_NOLANG		1001
-#define	POPT_RMSOURCE		1002
-#define	POPT_RMBUILD		1003
-#define	POPT_BUILDROOT		1004
-#define	POPT_BUILDARCH		1005
-#define	POPT_BUILDOS		1006
-#define	POPT_TARGETPLATFORM	1007
-#define	POPT_NOBUILD		1008
-#define	POPT_SHORTCIRCUIT	1009
-#define	POPT_RMSPEC		1010
-
-extern int noLang;
-static int noBuild = 0;
-static int useCatalog = 0;
-
-static void buildArgCallback( /*@unused@*/ poptContext con,
-	/*@unused@*/ enum poptCallbackReason reason,
-	const struct poptOption * opt, const char * arg, const void * data)
-{
-    struct rpmBuildArguments * rba = (struct rpmBuildArguments *) data;
-
-    switch (opt->val) {
-    case POPT_USECATALOG: rba->useCatalog = 1; break;
-    case POPT_NOBUILD: rba->noBuild = 1; break;
-    case POPT_NOLANG: rba->noLang = 1; break;
-    case POPT_SHORTCIRCUIT: rba->shortCircuit = 1; break;
-    case POPT_RMSOURCE: rba->buildAmount |= RPMBUILD_RMSOURCE; break;
-    case POPT_RMSPEC: rba->buildAmount |= RPMBUILD_RMSPEC; break;
-    case POPT_RMBUILD: rba->buildAmount |= RPMBUILD_RMBUILD; break;
-    case POPT_BUILDROOT:
-	if (rba->buildRootOverride) {
-	    fprintf(stderr, _("buildroot already specified"));
-	    exit(EXIT_FAILURE);
-	    /*@notreached@*/
-	}
-	rba->buildRootOverride = xstrdup(arg);
-	break;
-    case POPT_BUILDARCH:
-	fprintf(stderr, _("--buildarch has been obsoleted.  Use the --target option instead.\n")); 
-	exit(EXIT_FAILURE);
-	/*@notreached@*/ break;
-    case POPT_BUILDOS:
-	fprintf(stderr, _("--buildos has been obsoleted.  Use the --target option instead.\n")); 
-	exit(EXIT_FAILURE);
-	/*@notreached@*/ break;
-    case POPT_TARGETPLATFORM:
-	if (rba->targets) {
-	    int len = strlen(rba->targets) + 1 + strlen(arg) + 1;
-	    rba->targets = xrealloc(rba->targets, len);
-	    strcat(rba->targets, ",");
-	} else {
-	    rba->targets = xmalloc(strlen(arg) + 1);
-	    rba->targets[0] = '\0';
-	}
-	strcat(rba->targets, arg);
-	break;
-    }
-}
-
-/** */
-struct poptOption rpmBuildPoptTable[] = {
-	{ NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA,
-		buildArgCallback, 0, NULL, NULL },
-	{ "buildarch", '\0', POPT_ARG_STRING, 0,  POPT_BUILDARCH,
-		N_("override build architecture"), "ARCH" },
-	{ "buildos", '\0', POPT_ARG_STRING, 0,  POPT_BUILDOS,
-		N_("override build operating system"), "OS" },
-	{ "buildroot", '\0', POPT_ARG_STRING, 0,  POPT_BUILDROOT,
-		N_("override build root"), "DIRECTORY" },
-	{ "clean", '\0', 0, 0, POPT_RMBUILD,
-		N_("remove build tree when done"), NULL},
-	{ "nobuild", '\0', 0, &noBuild,  POPT_NOBUILD,
-		N_("do not execute any stages of the build"), NULL },
-	{ "nolang", '\0', 0, &noLang, POPT_NOLANG,
-		N_("do not accept I18N msgstr's from specfile"), NULL},
-	{ "rmsource", '\0', 0, 0, POPT_RMSOURCE,
-		N_("remove sources when done"), NULL},
-	{ "rmspec", '\0', 0, 0, POPT_RMSPEC,
-		N_("remove specfile when done"), NULL},
-	{ "short-circuit", '\0', 0, 0,  POPT_SHORTCIRCUIT,
-		N_("skip straight to specified stage (only for c,i)"), NULL },
-	{ "target", '\0', POPT_ARG_STRING, 0,  POPT_TARGETPLATFORM,
-		N_("override target platform"), "CPU-VENDOR-OS" },
-	{ "usecatalog", '\0', 0, &useCatalog, POPT_USECATALOG,
-		N_("lookup I18N strings in specfile catalog"), NULL},
-	{ 0, 0, 0, 0, 0,	NULL, NULL }
-};
