@@ -16,7 +16,7 @@
  * installs of a system w/o actually mounting filesystems.
  */
 struct fprintCacheEntry_s {
-    char * dirName;			/*!< path to existing directory */
+    const char * dirName;		/*!< path to existing directory */
     dev_t dev;				/*!< stat(2) device number */
     ino_t ino;				/*!< stat(2) inode number */
     int isFake;				/*!< (currently unused) */
@@ -68,7 +68,7 @@ extern "C" {
  * @param sizeHint	number of elements expected
  * @return pointer to initialized fingerprint cache
  */
-/*@only@*/ fingerPrintCache fpCacheCreate(int sizeHint);
+/*@only@*/ fingerPrintCache fpCacheCreate(int sizeHint)	/*@*/;
 
 /**
  * Destroy finger print cache.
@@ -85,24 +85,24 @@ void		fpCacheFree(/*@only@*/ fingerPrintCache cache);
  * @return pointer to the finger print associated with a file path.
  */
 fingerPrint	fpLookup(fingerPrintCache cache, const char * dirName, 
-			const char * baseName, int scareMemory);
+			const char * baseName, int scareMemory)	/*@*/;
 
 /**
  * Return hash value for a finger print.
  * Hash based on dev and inode only!
- * @param key   pointer to finger print entry
+ * @param key		pointer to finger print entry
  * @return hash value
  */
-unsigned int fpHashFunction(const void * key);
+unsigned int fpHashFunction(const void * key)	/*@*/;
 
 /**
  * Compare two finger print entries.
- * exactly equivalent to FP_EQUAL.
+ * exactly equivalent to FP_EQUAL macro.
  * @param key1		finger print 1
  * @param key2		finger print 2
  * @return result of comparing key1 and key2
  */
-int fpEqual(const void * key1, const void * key2);
+int fpEqual(const void * key1, const void * key2)	/*@*/;
 
 /**
  * Return finger prints of an array of file paths.
@@ -116,7 +116,8 @@ int fpEqual(const void * key1, const void * key2);
  */
 void fpLookupList(fingerPrintCache cache, const char ** dirNames, 
 		  const char ** baseNames, const int * dirIndexes, 
-		  int fileCount, fingerPrint * fpList);
+		  int fileCount, fingerPrint * fpList)
+			/*@modifies cache, *fpList @*/;
 
 /**
  * Return finger prints of all file names in header.
@@ -125,7 +126,8 @@ void fpLookupList(fingerPrintCache cache, const char ** dirNames,
  * @param h		package header
  * @retval fpList	pointer to array of finger prints
  */
-void fpLookupHeader(fingerPrintCache cache, Header h, fingerPrint * fpList);
+void fpLookupHeader(fingerPrintCache cache, Header h, fingerPrint * fpList)
+	/*@modifies h, cache, *fpList @*/;
 
 #ifdef __cplusplus
 }
