@@ -1,11 +1,16 @@
-#include "system.h"
+/**
+ * \file lib/fs.c
+ */
 
+#include "system.h"
 #include <rpmlib.h>
-#include <rpmmacro.h>
+#include <rpmmacro.h>	/* XXX for rpmGetPath */
+#include "debug.h"
 
 struct fsinfo {
-    /*@only@*/ const char * mntPoint;
-    dev_t dev;
+/*@only@*/ const char * mntPoint;	/*!< path to mount point. */
+    dev_t dev;				/*!< devno for mount point. */
+    int rdonly;				/*!< is mount point read only? */
 };
 
 /*@only@*/ /*@null@*/ static struct fsinfo * filesystems = NULL;
@@ -17,12 +22,14 @@ void freeFilesystems(void)
     if (filesystems) {
 	int i;
 	for (i = 0; i < numFilesystems; i++)
-	    xfree(filesystems[i].mntPoint);
+	    free((void *)filesystems[i].mntPoint);
 	free(filesystems);
 	filesystems = NULL;
     }
     if (fsnames) {
+#if 0	/* XXX leak/segfault on exit of "rpm -qp --qf '%{#fsnames}' pkg" */
 	free(fsnames);
+#endif
 	fsnames = NULL;
     }
     numFilesystems = 0;
@@ -41,19 +48,25 @@ void freeFilesystems(void)
  */
 int mntctl(int command, int size, char *buffer);
 
+/**
+ * Get information for mounted file systems.
+ * @todo determine rdonly for non-linux file systems.
+ * @return		0 on success, 1 on error
+ */
 static int getFilesystemList(void)
 {
     int size;
     void * buf;
     struct vmount * vm;
     struct stat sb;
+    int rdonly = 0;
     int num;
     int fsnameLength;
     int i;
 
     num = mntctl(MCTL_QUERY, sizeof(size), (char *) &size);
     if (num < 0) {
-	rpmError(RPMERR_MTAB, _("mntctl() failed to return fugger size: %s"), 
+	rpmError(RPMERR_MTAB, _("mntctl() failed to return size: %s\n"), 
 		 strerror(errno));
 	return 1;
     }
@@ -68,7 +81,7 @@ static int getFilesystemList(void)
     buf = alloca(size);
     num = mntctl(MCTL_QUERY, size, buf);
     if ( num <= 0 ) {
-        rpmError(RPMERR_MTAB, "mntctl() failed to return mount points: %s", 
+        rpmError(RPMERR_MTAB, _("mntctl() failed to return mount points: %s\n"), 
 		 strerror(errno));
 	return 1;
     }
@@ -88,7 +101,7 @@ static int getFilesystemList(void)
 	filesystems[i].mntPoint = fsnames[i] = fsn;
 	
 	if (stat(filesystems[i].mntPoint, &sb)) {
-	    rpmError(RPMERR_STAT, _("failed to stat %s: %s"), fsnames[i],
+	    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), fsnames[i],
 			strerror(errno));
 
 	    freeFilesystems();
@@ -96,6 +109,7 @@ static int getFilesystemList(void)
 	}
 	
 	filesystems[i].dev = sb.st_dev;
+	filesystems[i].rdonly = rdonly;
 
 	/* goto the next vmount structure: */
 	vm = (struct vmount *)((char *)vm + vm->vmt_length);
@@ -109,12 +123,18 @@ static int getFilesystemList(void)
 
 #else	/* HAVE_MNTCTL */
 
+/**
+ * Get information for mounted file systems.
+ * @todo determine rdonly for non-linux file systems.
+ * @return		0 on success, 1 on error
+ */
 static int getFilesystemList(void)
 {
     int numAlloced = 10;
     struct stat sb;
     int i;
-    char * mntdir;
+    const char * mntdir;
+    int rdonly = 0;
 #   if GETMNTENT_ONE || GETMNTENT_TWO
     our_mntent item;
     FILE * mtab;
@@ -129,7 +149,7 @@ static int getFilesystemList(void)
 #   if GETMNTENT_ONE || GETMNTENT_TWO
 	mtab = fopen(MOUNTED, "r");
 	if (!mtab) {
-	    rpmError(RPMERR_MTAB, _("failed to open %s: %s"), MOUNTED, 
+	    rpmError(RPMERR_MTAB, _("failed to open %s: %s\n"), MOUNTED, 
 		     strerror(errno));
 	    return 1;
 	}
@@ -145,8 +165,12 @@ static int getFilesystemList(void)
 	    /* this is Linux */
 	    our_mntent * itemptr = getmntent(mtab);
 	    if (!itemptr) break;
-	    item = *itemptr;
+	    item = *itemptr;	/* structure assignment */
 	    mntdir = item.our_mntdir;
+#if defined(MNTOPT_RO)
+	    if (hasmntopt(itemptr, MNTOPT_RO) != NULL)
+		rdonly = 1;
+#endif
 #	elif GETMNTENT_TWO
 	    /* Solaris, maybe others */
 	    if (getmntent(mtab, &item)) break;
@@ -157,7 +181,7 @@ static int getFilesystemList(void)
 #	endif
 
 	if (stat(mntdir, &sb)) {
-	    rpmError(RPMERR_STAT, "failed to stat %s: %s", mntdir,
+	    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), mntdir,
 			strerror(errno));
 
 	    freeFilesystems();
@@ -173,6 +197,7 @@ static int getFilesystemList(void)
 
 	filesystems[numFilesystems-1].dev = sb.st_dev;
 	filesystems[numFilesystems-1].mntPoint = xstrdup(mntdir);
+	filesystems[numFilesystems-1].rdonly = rdonly;
     }
 
 #   if GETMNTENT_ONE || GETMNTENT_TWO
@@ -183,6 +208,7 @@ static int getFilesystemList(void)
 
     filesystems[numFilesystems].dev = 0;
     filesystems[numFilesystems].mntPoint = NULL;
+    filesystems[numFilesystems].rdonly = 0;
 
     fsnames = xcalloc((numFilesystems + 1), sizeof(*fsnames));
     for (i = 0; i < numFilesystems; i++)
@@ -206,7 +232,7 @@ int rpmGetFilesystemList(const char *** listptr, int * num)
 }
 
 int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles,
-			  uint_32 ** usagesPtr, /*@unused@*/int flags)
+			  uint_32 ** usagesPtr, /*@unused@*/ int flags)
 {
     int_32 * usages;
     int i, len, j;
@@ -258,9 +284,9 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	    chptr = dirName + strlen(dirName) - 1;
 	    while (stat(dirName, &sb)) {
 		if (errno != ENOENT) {
-		    rpmError(RPMERR_STAT, _("failed to stat %s: %s"), buf,
+		    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), buf,
 				strerror(errno));
-		    xfree(sourceDir);
+		    free((void *)sourceDir);
 		    free(usages);
 		    return 1;
 		}
@@ -280,8 +306,8 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 
 		if (j == numFilesystems) {
 		    rpmError(RPMERR_BADDEV, 
-				_("file %s is on an unknown device"), buf);
-		    xfree(sourceDir);
+				_("file %s is on an unknown device\n"), buf);
+		    free((void *)sourceDir);
 		    free(usages);
 		    return 1;
 		}
@@ -295,7 +321,7 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	usages[lastfs] += fssizes[i];
     }
 
-    if (sourceDir) xfree(sourceDir);
+    if (sourceDir) free((void *)sourceDir);
 
     *usagesPtr = usages;
 
