@@ -1,3 +1,7 @@
+/**
+ * \file lib/misc.c
+ */
+
 #include "system.h"
 
 static int _debug = 0;
@@ -7,6 +11,10 @@ static int _debug = 0;
 #include <rpmmacro.h>	/* XXX for rpmGetPath */
 
 #include "misc.h"
+#include "debug.h"
+
+/*@access Header@*/		/* XXX compared with NULL */
+/*@access FD_t@*/		/* XXX compared with NULL */
 
 char * RPMVERSION = VERSION;	/* just to put a marker in librpm.a */
 
@@ -175,17 +183,6 @@ int rpmvercmp(const char * a, const char * b)
     if (!*one) return -1; else return 1;
 }
 
-void stripTrailingSlashes(char * str)
-{
-    char * chptr;
-
-    chptr = str + strlen(str) - 1;
-    while (*chptr == '/' && chptr >= str) {
-	*chptr = '\0';
-	chptr--;
-    }
-}
-
 int doputenv(const char *str)
 {
     char * a;
@@ -218,167 +215,75 @@ int dosetenv(const char *name, const char *value, int overwrite)
     return putenv(a);
 }
 
-/* unameToUid(), uidTouname() and the group variants are really poorly
-   implemented. They really ought to use hash tables. I just made the
-   guess that most files would be owned by root or the same person/group
-   who owned the last file. Those two values are cached, everything else
-   is looked up via getpw() and getgr() functions.  If this performs
-   too poorly I'll have to implement it properly :-( */
-
-int unameToUid(const char * thisUname, uid_t * uid)
+static int rpmMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
 {
-    /*@only@*/ static char * lastUname = NULL;
-    static int lastUnameLen = 0;
-    static int lastUnameAlloced;
-    static uid_t lastUid;
-    struct passwd * pwent;
-    int thisUnameLen;
+    char * d, * de;
+    int created = 0;
+    int rc;
 
-    if (!thisUname) {
-	lastUnameLen = 0;
+    if (path == NULL)
 	return -1;
-    } else if (!strcmp(thisUname, "root")) {
-	*uid = 0;
-	return 0;
-    }
+    d = alloca(strlen(path)+2);
+    de = stpcpy(d, path);
+    de[1] = '\0';
+    for (de = d; *de; de++) {
+	struct stat st;
+	char savec;
 
-    thisUnameLen = strlen(thisUname);
-    if (!lastUname || thisUnameLen != lastUnameLen ||
-	strcmp(thisUname, lastUname)) {
-	if (lastUnameAlloced < thisUnameLen + 1) {
-	    lastUnameAlloced = thisUnameLen + 10;
-	    lastUname = xrealloc(lastUname, lastUnameAlloced);	/* XXX memory leak */
+	while (*de && *de != '/') de++;
+	savec = de[1];
+	de[1] = '\0';
+
+	rc = stat(d, &st);
+	if (rc) {
+	    switch(errno) {
+	    default:
+		return errno;
+		/*@notreached@*/ break;
+	    case ENOENT:
+		break;
+	    }
+	    rc = mkdir(d, mode);
+	    if (rc)
+		return errno;
+	    created = 1;
+	    if (!(uid == (uid_t) -1 && gid == (gid_t) -1)) {
+		rc = chown(d, uid, gid);
+		if (rc)
+		    return errno;
+	    }
+	} else if (!S_ISDIR(st.st_mode)) {
+	    return ENOTDIR;
 	}
-	strcpy(lastUname, thisUname);
-
-	pwent = getpwnam(thisUname);
-	if (!pwent) {
-	    endpwent();
-	    pwent = getpwnam(thisUname);
-	    if (!pwent) return -1;
-	}
-
-	lastUid = pwent->pw_uid;
-    }
-
-    *uid = lastUid;
-
-    return 0;
-}
-
-int gnameToGid(const char * thisGname, gid_t * gid)
-{
-    /*@only@*/ static char * lastGname = NULL;
-    static int lastGnameLen = 0;
-    static int lastGnameAlloced;
-    static uid_t lastGid;
-    int thisGnameLen;
-    struct group * grent;
-
-    if (!thisGname) {
-	lastGnameLen = 0;
-	return -1;
-    } else if (!strcmp(thisGname, "root")) {
-	*gid = 0;
-	return 0;
-    }
-
-    thisGnameLen = strlen(thisGname);
-    if (!lastGname || thisGnameLen != lastGnameLen ||
-	strcmp(thisGname, lastGname)) {
-	if (lastGnameAlloced < thisGnameLen + 1) {
-	    lastGnameAlloced = thisGnameLen + 10;
-	    lastGname = xrealloc(lastGname, lastGnameAlloced);	/* XXX memory leak */
-	}
-	strcpy(lastGname, thisGname);
-
-	grent = getgrnam(thisGname);
-	if (!grent) {
-	    endgrent();
-	    grent = getgrnam(thisGname);
-	    if (!grent) return -1;
-	}
-	lastGid = grent->gr_gid;
-    }
-
-    *gid = lastGid;
-
-    return 0;
-}
-
-char * uidToUname(uid_t uid)
-{
-    static int lastUid = -1;
-    /*@only@*/ static char * lastUname = NULL;
-    static int lastUnameLen = 0;
-    struct passwd * pwent;
-    int len;
-
-    if (uid == (uid_t) -1) {
-	lastUid = -1;
-	return NULL;
-    } else if (!uid) {
-	return "root";
-    } else if (uid == lastUid) {
-	return lastUname;
-    } else {
-	pwent = getpwuid(uid);
-	if (!pwent) return NULL;
-
-	lastUid = uid;
-	len = strlen(pwent->pw_name);
-	if (lastUnameLen < len + 1) {
-	    lastUnameLen = len + 20;
-	    lastUname = xrealloc(lastUname, lastUnameLen);
-	}
-	strcpy(lastUname, pwent->pw_name);
-
-	return lastUname;
-    }
-}
-
-char * gidToGname(gid_t gid)
-{
-    static int lastGid = -1;
-    /*@only@*/ static char * lastGname = NULL;
-    static int lastGnameLen = 0;
-    struct group * grent;
-    int len;
-
-    if (gid == (gid_t) -1) {
-	lastGid = -1;
-	return NULL;
-    } else if (!gid) {
-	return "root";
-    } else if (gid == lastGid) {
-	return lastGname;
-    } else {
-	grent = getgrgid(gid);
-	if (!grent) return NULL;
-
-	lastGid = gid;
-	len = strlen(grent->gr_name);
-	if (lastGnameLen < len + 1) {
-	    lastGnameLen = len + 20;
-	    lastGname = xrealloc(lastGname, lastGnameLen);
-	}
-	strcpy(lastGname, grent->gr_name);
-
-	return lastGname;
+	de[1] = savec;
     }
+    rc = 0;
+    if (created)
+	rpmMessage(RPMMESS_WARNING, "created %%_tmppath directory %s\n", path);
+    return rc;
 }
 
 int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 {
+    const char * tpmacro = "%{?_tmppath:%{_tmppath}}%{!?_tmppath:/var/tmp}";
     const char * tempfn = NULL;
     const char * tfn = NULL;
+    static int _initialized = 0;
     int temput;
     FD_t fd = NULL;
     int ran;
 
     if (!prefix) prefix = "";
 
-    /* XXX should probably use mktemp here */
+    /* Create the temp directory if it doesn't already exist. */
+    if (!_initialized) {
+	_initialized = 1;
+	tempfn = rpmGenPath(prefix, tpmacro, NULL);
+	if (rpmMkpath(tempfn, 0755, (uid_t) -1, (gid_t) -1))
+	    goto errxit;
+    }
+
+    /* XXX should probably use mkstemp here */
     srand(time(NULL));
     ran = rand() % 100000;
 
@@ -388,12 +293,12 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 	char tfnbuf[64];
 #ifndef	NOTYET
 	sprintf(tfnbuf, "rpm-tmp.%d", ran++);
-	if (tempfn)	xfree(tempfn);
-	tempfn = rpmGenPath(prefix, "%{_tmppath}/", tfnbuf);
+	if (tempfn)	free((void *)tempfn);
+	tempfn = rpmGenPath(prefix, tpmacro, tfnbuf);
 #else
 	strcpy(tfnbuf, "rpm-tmp.XXXXXX");
-	if (tempfn)	xfree(tempfn);
-	tempfn = rpmGenPath(prefix, "%{_tmppath}/", mktemp(tfnbuf));
+	if (tempfn)	free((void *)tempfn);
+	tempfn = rpmGenPath(prefix, tpmacro, mktemp(tfnbuf));
 #endif
 
 	temput = urlPath(tempfn, &tfn);
@@ -420,18 +325,18 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
 	if (!stat(tfn, &sb) && S_ISLNK(sb.st_mode)) {
-	    rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), tfn);
+	    rpmError(RPMERR_SCRIPT, _("error creating temporary file %s\n"), tfn);
 	    goto errxit;
 	}
 
 	if (sb.st_nlink != 1) {
-	    rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), tfn);
+	    rpmError(RPMERR_SCRIPT, _("error creating temporary file %s\n"), tfn);
 	    goto errxit;
 	}
 
 	if (fstat(Fileno(fd), &sb2) == 0) {
 	    if (sb2.st_ino != sb.st_ino || sb2.st_dev != sb.st_dev) {
-		rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), tfn);
+		rpmError(RPMERR_SCRIPT, _("error creating temporary file %s\n"), tfn);
 		goto errxit;
 	    }
 	}
@@ -443,7 +348,7 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
     if (fnptr)
 	*fnptr = tempfn;
     else if (tempfn) {
-	xfree(tempfn);
+	free((void *)tempfn);
 	tempfn = NULL;
     }
     *fdptr = fd;
@@ -451,7 +356,7 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
     return 0;
 
 errxit:
-    if (tempfn) xfree(tempfn);
+    if (tempfn) free((void *)tempfn);
     if (fd) Fclose(fd);
     return 1;
 }
@@ -471,7 +376,7 @@ char * currentDirectory(void)
     return currDir;
 }
 
-int _noDirTokens = 1;
+int _noDirTokens = 0;
 
 static int dncmp(const void * a, const void * b)
 {
@@ -496,6 +401,11 @@ void compressFilelist(Header h)
      * a bit faster.
      */
 
+    if (headerIsEntry(h, RPMTAG_DIRNAMES)) {
+	headerRemoveEntry(h, RPMTAG_OLDFILENAMES);
+	return;		/* Already converted. */
+    }
+
     if (!headerGetEntry(h, RPMTAG_OLDFILENAMES, NULL,
 			(void **) &fileNames, &count))
 	return;		/* no file list */
@@ -538,14 +448,14 @@ void compressFilelist(Header h)
     }
 
 exit:
-    headerAddEntry(h, RPMTAG_DIRNAMES, RPM_STRING_ARRAY_TYPE,
-			dirNames, dirIndex + 1);
     headerAddEntry(h, RPMTAG_DIRINDEXES, RPM_INT32_TYPE,
 			dirIndexes, count);
     headerAddEntry(h, RPMTAG_BASENAMES, RPM_STRING_ARRAY_TYPE,
 			baseNames, count);
+    headerAddEntry(h, RPMTAG_DIRNAMES, RPM_STRING_ARRAY_TYPE,
+			dirNames, dirIndex + 1);
 
-    xfree(fileNames);
+    free((void *)fileNames);
 
     headerRemoveEntry(h, RPMTAG_OLDFILENAMES);
 }
@@ -587,8 +497,8 @@ static void doBuildFileList(Header h, /*@out@*/ const char *** fileListPtr,
 	data = stpcpy( stpcpy(data, dirNames[dirIndexes[i]]), baseNames[i]);
 	*data++ = '\0';
     }
-    xfree(baseNames);
-    xfree(dirNames);
+    free((void *)baseNames);
+    free((void *)dirNames);
 
     *fileListPtr = fileNames;
     *fileCountPtr = count;
@@ -599,19 +509,18 @@ void expandFilelist(Header h)
     const char ** fileNames = NULL;
     int count = 0;
 
-    doBuildFileList(h, &fileNames, &count, RPMTAG_BASENAMES,
+    if (!headerIsEntry(h, RPMTAG_OLDFILENAMES)) {
+	doBuildFileList(h, &fileNames, &count, RPMTAG_BASENAMES,
 			RPMTAG_DIRNAMES, RPMTAG_DIRINDEXES);
-
-    if (fileNames == NULL || count <= 0)
-	return;
-
-    headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
+	if (fileNames == NULL || count <= 0)
+	    return;
+	headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
 			fileNames, count);
+	free((void *)fileNames);
+    }
 
-    xfree(fileNames);
-
-    headerRemoveEntry(h, RPMTAG_BASENAMES);
     headerRemoveEntry(h, RPMTAG_DIRNAMES);
+    headerRemoveEntry(h, RPMTAG_BASENAMES);
     headerRemoveEntry(h, RPMTAG_DIRINDEXES);
 }
 
@@ -689,7 +598,7 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
 	    if (argc == 0)
 		argv = xmalloc((argc+2) * sizeof(*argv));
 	    else
-		argv = xrealloc(argv, (argc+2) * sizeof(*argv));
+		argv = xrealloc(argv, (argc+1) * sizeof(*argv));
 if (_debug)
 fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, av[j]);
 	    argv[argc++] = xstrdup(av[j]);
@@ -728,7 +637,7 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, av[j]);
 	globRoot += nb;
 	*globRoot = '\0';
 if (_debug)
-fprintf(stderr, "*** GLOB maxb %d diskURL %d %*s globURL %p %s\n", maxb, nb, nb, av[j], globURL, globURL);
+fprintf(stderr, "*** GLOB maxb %d diskURL %d %*s globURL %p %s\n", (int)maxb, (int)nb, (int)nb, av[j], globURL, globURL);
 	
 	if (argc == 0)
 	    argv = xmalloc((gl.gl_pathc+1) * sizeof(*argv));
@@ -744,22 +653,26 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, globURL);
 	    argv[argc++] = xstrdup(globURL);
 	}
 	Globfree(&gl);
-	xfree(globURL);
+	free((void *)globURL);
     }
-    argv[argc] = NULL;
-    if (argvPtr)
-	*argvPtr = argv;
-    if (argcPtr)
-	*argcPtr = argc;
-    rc = 0;
+    if (argv != NULL && argc > 0) {
+	argv[argc] = NULL;
+	if (argvPtr)
+	    *argvPtr = argv;
+	if (argcPtr)
+	    *argcPtr = argc;
+	rc = 0;
+    } else
+	rc = 1;
+
 
 exit:
     if (av)
-	xfree(av);
+	free((void *)av);
     if ((rc || argvPtr == NULL) && argv) {
 	for (i = 0; i < argc; i++)
-	    xfree(argv[i]);
-	xfree(argv);
+	    free((void *)argv[i]);
+	free((void *)argv);
 	argv = NULL;
     }
     return rc;
@@ -781,6 +694,7 @@ int rpmHeaderGetEntry(Header h, int_32 tag, int_32 *type,
 	if (count > 0) {
 	    *p = fl;
 	    if (c)	*c = count;
+	    if (type)	*type = RPM_STRING_ARRAY_TYPE;
 	    return 1;
 	}
 	if (c)	*c = 0;
@@ -820,7 +734,7 @@ int rpmHeaderGetEntry(Header h, int_32 tag, int_32 *type,
 /*
  * XXX Yet Another dressed entry to unify signature/header tag retrieval.
  */
-int rpmPackageGetEntry(void *leadp, Header sigs, Header h,
+int rpmPackageGetEntry( /*@unused@*/ void *leadp, Header sigs, Header h,
 	int_32 tag, int_32 *type, void **p, int_32 *c)
 {
     int_32 sigtag;
@@ -839,11 +753,92 @@ int rpmPackageGetEntry(void *leadp, Header sigs, Header h,
 	/*@notreached@*/ break;
     }
 
+    if (headerIsEntry(h, tag))
+	return rpmHeaderGetEntry(h, tag, type, p, c);
+
     if (sigs == NULL) {
 	if (c)	*c = 0;
 	return 0;
     }
 
     return headerGetEntry(sigs, sigtag, type, p, c);
+}
 
+/*
+ * Up to rpm 3.0.4, packages implicitly provided their own name-version-release.
+ * Retrofit an explicit "Provides: name = epoch:version-release.
+ */
+void providePackageNVR(Header h)
+{
+    const char *name, *version, *release;
+    int_32 * epoch;
+    const char *pEVR;
+    char *p;
+    int_32 pFlags = RPMSENSE_EQUAL;
+    const char ** provides = NULL;
+    const char ** providesEVR = NULL;
+    int_32 * provideFlags = NULL;
+    int providesCount;
+    int i;
+    int bingo = 1;
+
+    /* Generate provides for this package name-version-release. */
+    headerNVR(h, &name, &version, &release);
+    pEVR = p = alloca(21 + strlen(version) + 1 + strlen(release) + 1);
+    *p = '\0';
+    if (headerGetEntry(h, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL)) {
+	sprintf(p, "%d:", *epoch);
+	while (*p)
+	    p++;
+    }
+    (void) stpcpy( stpcpy( stpcpy(p, version) , "-") , release);
+
+    /*
+     * Rpm prior to 3.0.3 does not have versioned provides.
+     * If no provides at all are available, we can just add.
+     */
+    if (!headerGetEntry(h, RPMTAG_PROVIDENAME, NULL,
+		(void **) &provides, &providesCount)) {
+	goto exit;
+    }
+
+    /*
+     * Otherwise, fill in entries on legacy packages.
+     */
+    if (!headerGetEntry(h, RPMTAG_PROVIDEVERSION, NULL,
+		(void **) &providesEVR, NULL)) {
+	for (i = 0; i < providesCount; i++) {
+	    char * vdummy = "";
+	    int_32 fdummy = RPMSENSE_ANY;
+	    headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+			&vdummy, 1);
+	    headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+			&fdummy, 1);
+	}
+	goto exit;
+    }
+
+    headerGetEntry(h, RPMTAG_PROVIDEFLAGS, NULL,
+	(void **) &provideFlags, NULL);
+
+    for (i = 0; i < providesCount; i++) {
+	if (!(provideFlags[i] == RPMSENSE_EQUAL &&
+	    !strcmp(name, provides[i]) && !strcmp(pEVR, providesEVR[i])))
+	    continue;
+	bingo = 0;
+	break;
+    }
+
+exit:
+    if (provides) free((void *)provides);
+    if (providesEVR) free((void *)providesEVR);
+
+    if (bingo) {
+	headerAddOrAppendEntry(h, RPMTAG_PROVIDENAME, RPM_STRING_ARRAY_TYPE,
+		&name, 1);
+	headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+		&pFlags, 1);
+	headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+		&pEVR, 1);
+    }
 }
