@@ -1,4 +1,7 @@
-/* checksig.c: verify the signature of an RPM */
+/** \ingroup rpmcli
+ * \file lib/rpmchecksig.c
+ * Verify the signature of a package.
+ */
 
 #include "system.h"
 
@@ -7,6 +10,10 @@
 #include "rpmlead.h"
 #include "signature.h"
 #include "misc.h"	/* XXX for makeTempFile() */
+#include "debug.h"
+
+/*@access Header@*/		/* XXX compared with NULL */
+/*@access FD_t@*/		/* XXX compared with NULL */
 
 static int manageFile(FD_t *fdp, const char **fnp, int flags, int rc)
 {
@@ -28,7 +35,7 @@ static int manageFile(FD_t *fdp, const char **fnp, int flags, int rc)
     if (*fdp == NULL && fnp && *fnp) {
 	fd = Fopen(*fnp, ((flags & O_WRONLY) ? "w.ufdio" : "r.ufdio"));
 	if (fd == NULL || Ferror(fd)) {
-	    fprintf(stderr, _("%s: open failed: %s\n"), *fnp,
+	    rpmError(RPMERR_OPEN, _("%s: open failed: %s\n"), *fnp,
 		Fstrerror(fd));
 	    return 1;
 	}
@@ -39,7 +46,7 @@ static int manageFile(FD_t *fdp, const char **fnp, int flags, int rc)
     /* open a temp file */
     if (*fdp == NULL && (fnp == NULL || *fnp == NULL)) {
 	if (makeTempFile(NULL, (fnp ? &fn : NULL), &fd)) {
-	    fprintf(stderr, _("makeTempFile failed\n"));
+	    rpmError(RPMERR_MAKETEMP, _("makeTempFile failed\n"));
 	    return 1;
 	}
 	if (fnp)
@@ -71,13 +78,13 @@ static int copyFile(FD_t *sfdp, const char **sfnp,
 
     while ((count = Fread(buffer, sizeof(buffer[0]), sizeof(buffer), *sfdp)) > 0) {
 	if (Fwrite(buffer, sizeof(buffer[0]), count, *tfdp) < 0) {
-	    fprintf(stderr, _("%s: Fwrite failed: %s\n"), *tfnp,
+	    rpmError(RPMERR_FWRITE, _("%s: Fwrite failed: %s\n"), *tfnp,
 		Fstrerror(*tfdp));
 	    goto exit;
 	}
     }
     if (count < 0) {
-	fprintf(stderr, _("%s: Fread failed: %s\n"), *sfnp, Fstrerror(*sfdp));
+	rpmError(RPMERR_FREAD, _("%s: Fread failed: %s\n"), *sfnp, Fstrerror(*sfdp));
 	goto exit;
     }
 
@@ -89,12 +96,12 @@ exit:
     return rc;
 }
 
-int rpmReSign(int add, char *passPhrase, const char **argv)
+int rpmReSign(rpmResignFlags add, char *passPhrase, const char **argv)
 {
     FD_t fd = NULL;
     FD_t ofd = NULL;
-    struct rpmlead lead;
-    unsigned short sigtype;
+    struct rpmlead lead, *l = &lead;
+    int sigtype;
     const char *rpm, *trpm;
     const char *sigtarget = NULL;
     char tmprpm[1024+1];
@@ -109,29 +116,29 @@ int rpmReSign(int add, char *passPhrase, const char **argv)
 	if (manageFile(&fd, &rpm, O_RDONLY, 0))
 	    goto exit;
 
-	if (readLead(fd, &lead)) {
-	    fprintf(stderr, _("%s: readLead failed\n"), rpm);
+	if (readLead(fd, l)) {
+	    rpmError(RPMERR_READLEAD, _("%s: readLead failed\n"), rpm);
 	    goto exit;
 	}
-	switch (lead.major) {
+	switch (l->major) {
 	case 1:
-	    fprintf(stderr, _("%s: Can't sign v1.0 RPM\n"), rpm);
+	    rpmError(RPMERR_BADSIGTYPE, _("%s: Can't sign v1.0 RPM\n"), rpm);
 	    goto exit;
 	    /*@notreached@*/ break;
 	case 2:
-	    fprintf(stderr, _("%s: Can't re-sign v2.0 RPM\n"), rpm);
+	    rpmError(RPMERR_BADSIGTYPE, _("%s: Can't re-sign v2.0 RPM\n"), rpm);
 	    goto exit;
 	    /*@notreached@*/ break;
 	default:
 	    break;
 	}
 
-	if (rpmReadSignature(fd, &sig, lead.signature_type)) {
-	    fprintf(stderr, _("%s: rpmReadSignature failed\n"), rpm);
+	if (rpmReadSignature(fd, &sig, l->signature_type)) {
+	    rpmError(RPMERR_SIGGEN, _("%s: rpmReadSignature failed\n"), rpm);
 	    goto exit;
 	}
 	if (sig == NULL) {
-	    fprintf(stderr, _("%s: No signature available\n"), rpm);
+	    rpmError(RPMERR_SIGGEN, _("%s: No signature available\n"), rpm);
 	    goto exit;
 	}
 
@@ -143,7 +150,7 @@ int rpmReSign(int add, char *passPhrase, const char **argv)
 	/* ASSERT: fd == NULL && ofd == NULL */
 
 	/* Generate the new signatures */
-	if (add != ADD_SIGNATURE) {
+	if (add != RESIGN_ADD_SIGNATURE) {
 	    rpmFreeSignature(sig);
 	    sig = rpmNewSignature();
 	    rpmAddSignature(sig, sigtarget, RPMSIGTAG_SIZE, passPhrase);
@@ -162,15 +169,15 @@ int rpmReSign(int add, char *passPhrase, const char **argv)
 	if (manageFile(&ofd, &trpm, O_WRONLY|O_CREAT|O_TRUNC, 0))
 	    goto exit;
 
-	lead.signature_type = RPMSIG_HEADERSIG;
-	if (writeLead(ofd, &lead)) {
-	    fprintf(stderr, _("%s: writeLead failed: %s\n"), trpm,
+	l->signature_type = RPMSIG_HEADERSIG;
+	if (writeLead(ofd, l)) {
+	    rpmError(RPMERR_WRITELEAD, _("%s: writeLead failed: %s\n"), trpm,
 		Fstrerror(ofd));
 	    goto exit;
 	}
 
 	if (rpmWriteSignature(ofd, sig)) {
-	    fprintf(stderr, _("%s: rpmWriteSignature failed: %s\n"), trpm,
+	    rpmError(RPMERR_SIGGEN, _("%s: rpmWriteSignature failed: %s\n"), trpm,
 		Fstrerror(ofd));
 	    goto exit;
 	}
@@ -184,7 +191,7 @@ int rpmReSign(int add, char *passPhrase, const char **argv)
 
 	/* Clean up intermediate target */
 	unlink(sigtarget);
-	xfree(sigtarget);	sigtarget = NULL;
+	free((void *)sigtarget);	sigtarget = NULL;
 
 	/* Move final target into place. */
 	unlink(rpm);
@@ -203,7 +210,7 @@ exit:
     }
     if (sigtarget) {
 	unlink(sigtarget);
-	xfree(sigtarget);
+	free((void *)sigtarget);
 	sigtarget = NULL;
     }
     if (tmprpm[0] != '\0') {
@@ -214,12 +221,12 @@ exit:
     return rc;
 }
 
-int rpmCheckSig(int flags, const char **argv)
+int rpmCheckSig(rpmCheckSigFlags flags, const char **argv)
 {
     FD_t fd = NULL;
     FD_t ofd = NULL;
     int res2, res3;
-    struct rpmlead lead;
+    struct rpmlead lead, *l = &lead;
     const char *rpm = NULL;
     char result[1024];
     const char * sigtarget = NULL;
@@ -227,9 +234,9 @@ int rpmCheckSig(int flags, const char **argv)
     unsigned char missingKeys[7164];
     unsigned char untrustedKeys[7164];
     Header sig;
-    HeaderIterator sigIter;
+    HeaderIterator hi;
     int_32 tag, type, count;
-    void *ptr;
+    const void * ptr;
     int res = 0;
 
     while ((rpm = *argv++) != NULL) {
@@ -240,26 +247,26 @@ int rpmCheckSig(int flags, const char **argv)
 	}
 
 	if (readLead(fd, &lead)) {
-	    fprintf(stderr, _("%s: readLead failed\n"), rpm);
+	    rpmError(RPMERR_READLEAD, _("%s: readLead failed\n"), rpm);
 	    res++;
 	    goto bottom;
 	}
-	switch (lead.major) {
+	switch (l->major) {
 	case 1:
-	    fprintf(stderr, _("%s: No signature available (v1.0 RPM)\n"), rpm);
+	    rpmError(RPMERR_BADSIGTYPE, _("%s: No signature available (v1.0 RPM)\n"), rpm);
 	    res++;
 	    goto bottom;
 	    /*@notreached@*/ break;
 	default:
 	    break;
 	}
-	if (rpmReadSignature(fd, &sig, lead.signature_type)) {
-	    fprintf(stderr, _("%s: rpmReadSignature failed\n"), rpm);
+	if (rpmReadSignature(fd, &sig, l->signature_type)) {
+	    rpmError(RPMERR_SIGGEN, _("%s: rpmReadSignature failed\n"), rpm);
 	    res++;
 	    goto bottom;
 	}
 	if (sig == NULL) {
-	    fprintf(stderr, _("%s: No signature available\n"), rpm);
+	    rpmError(RPMERR_SIGGEN, _("%s: No signature available\n"), rpm);
 	    res++;
 	    goto bottom;
 	}
@@ -277,10 +284,9 @@ int rpmCheckSig(int flags, const char **argv)
 	untrustedKeys[0] = '\0';
 	sprintf(buffer, "%s:%c", rpm, (rpmIsVerbose() ? '\n' : ' ') );
 
-	for (sigIter = headerInitIterator(sig);
-	    headerNextIterator(sigIter, &tag, &type, &ptr, &count);
-	    ptr = ((type == RPM_STRING_ARRAY_TYPE || type == RPM_I18NSTRING_TYPE)
-		? xfree(ptr), NULL : NULL))
+	for (hi = headerInitIterator(sig);
+	    headerNextIterator(hi, &tag, &type, &ptr, &count);
+	    ptr = headerFreeData(ptr, type))
 	{
 	    switch (tag) {
 	    case RPMSIGTAG_PGP5:	/* XXX legacy */
@@ -399,16 +405,16 @@ int rpmCheckSig(int flags, const char **argv)
 		}
 	    }
 	}
-	headerFreeIterator(sigIter);
+	headerFreeIterator(hi);
 	res += res2;
 	unlink(sigtarget);
-	xfree(sigtarget);	sigtarget = NULL;
+	free((void *)sigtarget);	sigtarget = NULL;
 
 	if (res2) {
 	    if (rpmIsVerbose()) {
-		fprintf(stderr, "%s", (char *)buffer);
+		rpmError(RPMERR_SIGVFY, "%s", (char *)buffer);
 	    } else {
-		fprintf(stderr, "%s%s%s%s%s%s%s%s\n", (char *)buffer,
+		rpmError(RPMERR_SIGVFY, "%s%s%s%s%s%s%s%s\n", (char *)buffer,
 			_("NOT OK"),
 			(missingKeys[0] != '\0') ? _(" (MISSING KEYS:") : "",
 			(char *)missingKeys,
@@ -420,9 +426,9 @@ int rpmCheckSig(int flags, const char **argv)
 	    }
 	} else {
 	    if (rpmIsVerbose()) {
-		fprintf(stdout, "%s", (char *)buffer);
+		rpmError(RPMERR_SIGVFY, "%s", (char *)buffer);
 	    } else {
-		fprintf(stdout, "%s%s%s%s%s%s%s%s\n", (char *)buffer,
+		rpmError(RPMERR_SIGVFY, "%s%s%s%s%s%s%s%s\n", (char *)buffer,
 			_("OK"),
 			(missingKeys[0] != '\0') ? _(" (MISSING KEYS:") : "",
 			(char *)missingKeys,
@@ -438,7 +444,7 @@ int rpmCheckSig(int flags, const char **argv)
 	if (ofd)	manageFile(&ofd, NULL, 0, 0);
 	if (sigtarget) {
 	    unlink(sigtarget);
-	    xfree(sigtarget);	sigtarget = NULL;
+	    free((void *)sigtarget);	sigtarget = NULL;
 	}
     }
 
