@@ -1,21 +1,21 @@
+/** \ingroup lead
+ * \file lib/rpmlead.c
+ */
+
 #include "system.h"
 
 #if HAVE_MACHINE_TYPES_H
 # include <machine/types.h>
 #endif
 
-#ifdef	__LCLINT__
-#define	ntohl(_x)	(_x)
-#define	ntohs(_x)	(_x)
-#define	htonl(_x)	(_x)
-#define	htons(_x)	(_x)
-#else
+#if !defined(__LCLINT__)
 #include <netinet/in.h>
 #endif	/* __LCLINT__ */
 
 #include <rpmlib.h>
 
 #include "rpmlead.h"
+#include "debug.h"
 
 /* The lead needs to be 8 byte aligned */
 
@@ -45,7 +45,7 @@ int writeLead(FD_t fd, struct rpmlead *lead)
 int readLead(FD_t fd, struct rpmlead *lead)
 {
     if (timedRead(fd, (char *)lead, sizeof(*lead)) != sizeof(*lead)) {
-	rpmError(RPMERR_READERROR, _("read failed: %s (%d)"), Fstrerror(fd), 
+	rpmError(RPMERR_READ, _("read failed: %s (%d)\n"), Fstrerror(fd), 
 	      errno);
 	return 1;
     }
@@ -59,4 +59,3 @@ int readLead(FD_t fd, struct rpmlead *lead)
 
     return 0;
 }
-
