@@ -12,13 +12,18 @@
 #include <rpmmacro.h>
 
 #include "misc.h"
+#include "debug.h"
 
 static const char *defrcfiles = LIBRPMRC_FILENAME ":/etc/rpmrc:~/.rpmrc";
 
+const char * macrofiles = MACROFILES;
+
+typedef /*@owned@*/ const char * cptr_t;
+
 struct machCacheEntry {
     const char * name;
     int count;
-    const char ** equivs;
+    cptr_t * equivs;
     int visited;
 };
 
@@ -52,19 +57,20 @@ struct rpmOption {
 };
 
 struct defaultEntry {
-    char *name;
-    char *defName;
+/*@owned@*/ const char * name;
+/*@owned@*/ const char * defName;
 };
 
 struct canonEntry {
-    char *name;
-    char *short_name;
+/*@owned@*/ const char * name;
+/*@owned@*/ const char * short_name;
     short num;
 };
 
 /* tags are 'key'canon, 'key'translate, 'key'compat
-
-   for giggles, 'key'_canon, 'key'_compat, and 'key'_canon will also work */
+ *
+ * for giggles, 'key'_canon, 'key'_compat, and 'key'_canon will also work
+ */
 struct tableType {
     const char * const key;
     const int hasCanon;
@@ -100,25 +106,25 @@ static int optionTableSize = sizeof(optionTable) / sizeof(*optionTable);
 #define OS	0
 #define ARCH	1
 
-static char * current[2];
+static cptr_t current[2];
 static int currTables[2] = { RPM_MACHTABLE_INSTOS, RPM_MACHTABLE_INSTARCH };
 static struct rpmvarValue values[RPMVAR_NUM];
 static int defaultsInitialized = 0;
 
 /* prototypes */
-static int doReadRC(FD_t fd, const char * urlfn);
+static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn);
 static void rpmSetVarArch(int var, const char * val, const char * arch);
 static void rebuildCompatTables(int type, const char *name);
 
 static int optionCompare(const void * a, const void * b) {
-    return strcasecmp(((struct rpmOption *) a)->name,
+    return xstrcasecmp(((struct rpmOption *) a)->name,
 		      ((struct rpmOption *) b)->name);
 }
 
 static void rpmRebuildTargetVars(const char **target, const char ** canontarget);
 
-static struct machCacheEntry * machCacheFindEntry(struct machCache * cache,
-						  const char * key)
+static /*@observer@*/ struct machCacheEntry *
+machCacheFindEntry(struct machCache * cache, const char * key)
 {
     int i;
 
@@ -141,10 +147,10 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
     chptr = name;
     while (*chptr && *chptr != ':') chptr++;
     if (!*chptr) {
-	rpmError(RPMERR_RPMRC, _("missing second ':' at %s:%d"), fn, linenum);
+	rpmError(RPMERR_RPMRC, _("missing second ':' at %s:%d\n"), fn, linenum);
 	return 1;
     } else if (chptr == name) {
-	rpmError(RPMERR_RPMRC, _("missing architecture name at %s:%d"), fn,
+	rpmError(RPMERR_RPMRC, _("missing architecture name at %s:%d\n"), fn,
 			     linenum);
 	return 1;
     }
@@ -161,8 +167,8 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
 	entry = machCacheFindEntry(cache, name);
 	if (entry) {
 	    for (i = 0; i < entry->count; i++)
-		xfree(entry->equivs[i]);
-	    xfree(entry->equivs);
+		free((void *)entry->equivs[i]);
+	    free((void *)entry->equivs);
 	    entry->equivs = NULL;
 	    entry->count = 0;
 	}
@@ -196,21 +202,13 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
     return 0;
 }
 
-static struct machEquivInfo * machEquivSearch(
-		struct machEquivTable * table, const char * name)
+static /*@observer@*/ struct machEquivInfo *
+	machEquivSearch(const struct machEquivTable * table, const char * name)
 {
     int i;
 
-/*
- * XXX The strcasecmp below is necessary so the old (rpm < 2.90) style
- * XXX os-from-uname (e.g. "Linux") is compatible with the new
- * XXX os-from-platform (e.g "linux" from "sparc-*-linux").
- * XXX A copy of this string is embedded in headers and is
- * XXX used by rpmInstallPackage->{os,arch}Okay->rpmMachineScore->.
- * XXX to verify correct arch/os from headers.
- */
     for (i = 0; i < table->count; i++)
-	if (!strcasecmp(table->list[i].name, name))
+	if (!xstrcasecmp(table->list[i].name, name))
 	    return table->list + i;
 
     return NULL;
@@ -266,11 +264,11 @@ static void machFindEquivs(struct machCache * cache,
 	cache->cache[i].visited = 0;
 
     while (table->count > 0) {
-	xfree(table->list[--table->count].name);
+	free((void *)table->list[--table->count].name);
 	table->list[table->count].name = NULL;
     }
     table->count = 0;
-    if (table->list) xfree(table->list);
+    if (table->list) free((void *)table->list);
     table->list = NULL;
 
     /*
@@ -304,18 +302,19 @@ static int addCanon(struct canonEntry ** table, int * tableLen, char * line,
     tshort_name = strtok(NULL, " \t");
     s = strtok(NULL, " \t");
     if (! (tname && tshort_name && s)) {
-	rpmError(RPMERR_RPMRC, _("Incomplete data line at %s:%d"), fn, lineNum);
+	rpmError(RPMERR_RPMRC, _("Incomplete data line at %s:%d\n"),
+		fn, lineNum);
 	return RPMERR_RPMRC;
     }
     if (strtok(NULL, " \t")) {
-	rpmError(RPMERR_RPMRC, _("Too many args in data line at %s:%d"),
+	rpmError(RPMERR_RPMRC, _("Too many args in data line at %s:%d\n"),
 	      fn, lineNum);
 	return RPMERR_RPMRC;
     }
 
     tnum = strtoul(s, &s1, 10);
     if ((*s1) || (s1 == s) || (tnum == ULONG_MAX)) {
-	rpmError(RPMERR_RPMRC, _("Bad arch/os number: %s (%s:%d)"), s,
+	rpmError(RPMERR_RPMRC, _("Bad arch/os number: %s (%s:%d)\n"), s,
 	      fn, lineNum);
 	return(RPMERR_RPMRC);
     }
@@ -350,12 +349,12 @@ static int addDefault(struct defaultEntry **table, int *tableLen, char *line,
     t->name = strtok(line, ": \t");
     t->defName = strtok(NULL, " \t");
     if (! (t->name && t->defName)) {
-	rpmError(RPMERR_RPMRC, _("Incomplete default line at %s:%d"),
+	rpmError(RPMERR_RPMRC, _("Incomplete default line at %s:%d\n"),
 		 fn, lineNum);
 	return RPMERR_RPMRC;
     }
     if (strtok(NULL, " \t")) {
-	rpmError(RPMERR_RPMRC, _("Too many args in default line at %s:%d"),
+	rpmError(RPMERR_RPMRC, _("Too many args in default line at %s:%d\n"),
 	      fn, lineNum);
 	return RPMERR_RPMRC;
     }
@@ -366,9 +365,9 @@ static int addDefault(struct defaultEntry **table, int *tableLen, char *line,
     return 0;
 }
 
-static struct canonEntry *lookupInCanonTable(char *name,
-					     struct canonEntry *table,
-					     int tableLen) {
+static /*@null@*/ const struct canonEntry *lookupInCanonTable(const char *name,
+	const struct canonEntry *table, int tableLen)
+{
     while (tableLen) {
 	tableLen--;
 	if (!strcmp(name, table[tableLen].name)) {
@@ -379,8 +378,9 @@ static struct canonEntry *lookupInCanonTable(char *name,
     return NULL;
 }
 
-static const char *lookupInDefaultTable(const char *name, struct defaultEntry *table,
-				  int tableLen) {
+static /*@observer@*/ const char * lookupInDefaultTable(const char *name,
+		const struct defaultEntry *table, int tableLen)
+{
     while (tableLen) {
 	tableLen--;
 	if (!strcmp(name, table[tableLen].name)) {
@@ -407,8 +407,8 @@ int rpmReadConfigFiles(const char * file, const char * target)
     {	const char *cpu = rpmExpand("%{_target_cpu}", NULL);
 	const char *os = rpmExpand("%{_target_os}", NULL);
 	rpmSetMachine(cpu, os);
-	xfree(cpu);
-	xfree(os);
+	free((void *)cpu);
+	free((void *)os);
     }
 
     return 0;
@@ -443,7 +443,7 @@ static void setPathDefault(int var, const char *macroname, const char *subdir)
 	strcat(fn, subdir);
 
 	rpmSetVar(var, fn);
-	if (topdir)	xfree(topdir);
+	if (topdir)	free((void *)topdir);
     }
 
     if (macroname != NULL) {
@@ -539,19 +539,26 @@ int rpmReadRC(const char * rcfiles)
 	/* Expand ~/ to $HOME/ */
 	fn[0] = '\0';
 	if (r[0] == '~' && r[1] == '/') {
-	    char *home = getenv("HOME");
+	    const char * home = getenv("HOME");
 	    if (home == NULL) {
 	    /* XXX Only /usr/lib/rpm/rpmrc must exist in default rcfiles list */
 		if (rcfiles == defrcfiles && myrcfiles != r)
 		    continue;
-		rpmError(RPMERR_RPMRC, _("Cannot expand %s"), r);
+		rpmError(RPMERR_RPMRC, _("Cannot expand %s\n"), r);
+		rc = 1;
+		break;
+	    }
+	    if (strlen(home) > (sizeof(fn) - strlen(r))) {
+		rpmError(RPMERR_RPMRC, _("Cannot read %s, HOME is too large.\n"),
+				r);
 		rc = 1;
 		break;
 	    }
 	    strcpy(fn, home);
 	    r++;
 	}
-	strcat(fn, r);
+	strncat(fn, r, sizeof(fn) - (strlen(fn) + 1));
+	fn[sizeof(fn)-1] = '\0';
 
 	/* Read another rcfile */
 	fd = Fopen(fn, "r.fpio");
@@ -559,7 +566,7 @@ int rpmReadRC(const char * rcfiles)
 	    /* XXX Only /usr/lib/rpm/rpmrc must exist in default rcfiles list */
 	    if (rcfiles == defrcfiles && myrcfiles != r)
 		continue;
-	    rpmError(RPMERR_RPMRC, _("Unable to open %s for reading: %s."),
+	    rpmError(RPMERR_RPMRC, _("Unable to open %s for reading: %s.\n"),
 		 fn, Fstrerror(fd));
 	    rc = 1;
 	    break;
@@ -578,7 +585,7 @@ int rpmReadRC(const char * rcfiles)
 	if ((macrofiles = rpmGetVar(RPMVAR_MACROFILES)) != NULL) {
 	    macrofiles = strdup(macrofiles);
 	    rpmInitMacros(NULL, macrofiles);
-	    xfree(macrofiles);
+	    free((void *)macrofiles);
 	}
     }
 
@@ -604,7 +611,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
     next[0] = '\0';
     rc = Fread(next, sizeof(*next), nb, fd);
     if (Ferror(fd) || (size > 0 && rc != nb)) {	/* XXX Feof(fd) */
-	rpmError(RPMERR_RPMRC, _("Failed to read %s: %s."), urlfn,
+	rpmError(RPMERR_RPMRC, _("Failed to read %s: %s.\n"), urlfn,
 		 Fstrerror(fd));
 	rc = 1;
     } else
@@ -641,7 +648,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 	}
 
 	if (*se != ':') {
-	    rpmError(RPMERR_RPMRC, _("missing ':' (found 0x%02x) at %s:%d"),
+	    rpmError(RPMERR_RPMRC, _("missing ':' (found 0x%02x) at %s:%d\n"),
 		     (0xff & *se), urlfn, linenum);
 	    return 1;
 	}
@@ -658,7 +665,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 
 	    arch = val = fn = NULL;
 	    if (*se == '\0') {
-		rpmError(RPMERR_RPMRC, _("missing argument for %s at %s:%d"),
+		rpmError(RPMERR_RPMRC, _("missing argument for %s at %s:%d\n"),
 		      option->name, urlfn, linenum);
 		return 1;
 	    }
@@ -675,31 +682,31 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 
 		fn = rpmGetPath(s, NULL);
 		if (fn == NULL || *fn == '\0') {
-		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\""),
+		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\"\n"),
 			option->name, urlfn, linenum, s);
-		    if (fn) xfree(fn);
+		    if (fn) free((void *)fn);
 		    return 1;
 		    /*@notreached@*/
 		}
 
 		fdinc = Fopen(fn, "r.fpio");
 		if (fdinc == NULL || Ferror(fdinc)) {
-		    rpmError(RPMERR_RPMRC, _("cannot open %s at %s:%d: %s"),
+		    rpmError(RPMERR_RPMRC, _("cannot open %s at %s:%d: %s\n"),
 			fn, urlfn, linenum, Fstrerror(fdinc));
 		    rc = 1;
 		} else {
 		    rc = doReadRC(fdinc, fn);
 		}
-		if (fn) xfree(fn);
+		if (fn) free((void *)fn);
 		if (rc) return rc;
 		continue;	/* XXX don't save include value as var/macro */
 	      }	/*@notreached@*/ break;
 	    case RPMVAR_MACROFILES:
 		fn = rpmGetPath(se, NULL);
 		if (fn == NULL || *fn == '\0') {
-		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\""),
+		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\"\n"),
 			option->name, urlfn, linenum, fn);
-		    if (fn) xfree(fn);
+		    if (fn) free((void *)fn);
 		    return 1;
 		}
 		se = (char *)fn;
@@ -724,7 +731,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		while (*se && !isspace(*se)) se++;
 		if (*se == '\0') {
 		    rpmError(RPMERR_RPMRC,
-				_("missing architecture for %s at %s:%d"),
+				_("missing architecture for %s at %s:%d\n"),
 			  	option->name, urlfn, linenum);
 		    return 1;
 		}
@@ -732,7 +739,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		while (*se && isspace(*se)) se++;
 		if (*se == '\0') {
 		    rpmError(RPMERR_RPMRC,
-				_("missing argument for %s at %s:%d"),
+				_("missing argument for %s at %s:%d\n"),
 			  	option->name, urlfn, linenum);
 		    return 1;
 		}
@@ -752,7 +759,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		free(name);
 	    }
 	    rpmSetVarArch(option->var, val, arch);
-	    if (fn) xfree(fn);
+	    if (fn) free((void *)fn);
 
 	} else {	/* For arch/os compatibilty tables ... */
 	    int gotit;
@@ -791,7 +798,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 	    }
 
 	    if (!gotit) {
-		rpmError(RPMERR_RPMRC, _("bad option '%s' at %s:%d"),
+		rpmError(RPMERR_RPMRC, _("bad option '%s' at %s:%d\n"),
 			    s, urlfn, linenum);
 	    }
 	}
@@ -800,11 +807,126 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
     return 0;
 }
 
-static void defaultMachine(const char ** arch, const char ** os) {
+#	if defined(__linux__) && defined(__i386__)
+#include <setjmp.h>
+#include <signal.h>
+
+/*
+ * Generic CPUID function
+ */
+static inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
+{
+#ifdef PIC
+	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
+		: "=a"(*eax), "=g"(*ebx), "=&c"(*ecx), "=&d"(*edx)
+		: "a" (op));
+#else
+	__asm__("cpuid"
+		: "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx)
+		: "a" (op));
+#endif
+
+}
+
+/*
+ * CPUID functions returning a single datum
+ */
+static inline unsigned int cpuid_eax(unsigned int op)
+{
+	unsigned int val;
+
+#ifdef PIC
+	__asm__("pushl %%ebx; cpuid; popl %%ebx"
+		: "=a" (val) : "a" (op) : "ecx", "edx");
+#else
+	__asm__("cpuid"
+		: "=a" (val) : "a" (op) : "ebx", "ecx", "edx");
+#endif
+	return val;
+}
+
+static inline unsigned int cpuid_ebx(unsigned int op)
+{
+	unsigned int tmp, val;
+
+#ifdef PIC
+	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
+		: "=a" (tmp), "=g" (val) : "a" (op) : "ecx", "edx");
+#else
+	__asm__("cpuid"
+		: "=a" (tmp), "=b" (val) : "a" (op) : "ecx", "edx");
+#endif
+	return val;
+}
+
+static inline unsigned int cpuid_ecx(unsigned int op)
+{
+	unsigned int tmp, val;
+#ifdef PIC
+	__asm__("pushl %%ebx; cpuid; popl %%ebx"
+		: "=a" (tmp), "=c" (val) : "a" (op) : "edx");
+#else
+	__asm__("cpuid"
+		: "=a" (tmp), "=c" (val) : "a" (op) : "ebx", "edx");
+#endif
+	return val;
+
+}
+
+static inline unsigned int cpuid_edx(unsigned int op)
+{
+	unsigned int tmp, val;
+#ifdef PIC
+	__asm__("pushl %%ebx; cpuid; popl %%ebx"
+		: "=a" (tmp), "=d" (val) : "a" (op) : "ecx");
+#else
+	__asm__("cpuid"
+		: "=a" (tmp), "=d" (val) : "a" (op) : "ebx", "ecx");
+#endif
+	return val;
+
+}
+
+static sigjmp_buf jenv;
+
+static inline void model3(int _unused)
+{
+	siglongjmp(jenv, 1);
+}
+
+static inline int RPMClass(void)
+{
+	int cpu;
+	unsigned int tfms, junk, cap;
+	
+	signal(SIGILL, model3);
+	
+	if(sigsetjmp(jenv, 1))
+		return 3;
+		
+	if(cpuid_eax(0x000000000)==0)
+		return 4;
+	cpuid(0x000000001, &tfms, &junk, &junk, &cap);
+	
+	cpu = (tfms>>8)&15;
+	
+	if(cpu < 6)
+		return cpu;
+		
+	if(cap & (1<<15))
+		return 6;
+		
+	return 5;
+}
+
+#endif
+
+static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char ** os)
+{
     static struct utsname un;
     static int gotDefaults = 0;
     char * chptr;
-    struct canonEntry * canon;
+    const struct canonEntry * canon;
 
     if (!gotDefaults) {
 	uname(&un);
@@ -984,6 +1106,15 @@ static void defaultMachine(const char ** arch, const char ** os) {
 	}
 #	endif
 
+#	if defined(__linux__) && defined(__i386__)
+	{
+	    char class = (char) (RPMClass() | '0');
+
+	    if (strchr("3456", un.machine[1]) && un.machine[1] != class)
+		un.machine[1] = class;
+	}
+#	endif
+
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
 				   tables[RPM_MACHTABLE_INSTARCH].canons,
@@ -1003,7 +1134,7 @@ static void defaultMachine(const char ** arch, const char ** os) {
     if (os) *os = un.sysname;
 }
 
-static const char * rpmGetVarArch(int var, char * arch) {
+static const char * rpmGetVarArch(int var, const char * arch) {
     struct rpmvarValue * next;
 
     if (!arch) arch = current[ARCH];
@@ -1034,11 +1165,11 @@ static void freeRpmVar(struct rpmvarValue * orig) {
     while (var) {
 	next = var->next;
 	if (var->arch) {
-	    xfree(var->arch);
+	    free((void *)var->arch);
 	    var->arch = NULL;
 	}
 	if (var->value) {
-	    xfree(var->value);
+	    free((void *)var->value);
 	    var->value = NULL;
 	}
 
@@ -1069,8 +1200,8 @@ static void rpmSetVarArch(int var, const char * val, const char * arch) {
 	}
 
 	if (next->arch && arch && !strcmp(next->arch, arch)) {
-	    if (next->value) xfree(next->value);
-	    if (next->arch) xfree(next->arch);
+	    if (next->value) free((void *)next->value);
+	    if (next->arch) free((void *)next->arch);
 	} else if (next->arch || arch) {
 	    next->next = xmalloc(sizeof(*next->next));
 	    next = next->next;
@@ -1136,14 +1267,14 @@ void rpmSetMachine(const char * arch, const char * os) {
     }
 
     if (!current[ARCH] || strcmp(arch, current[ARCH])) {
-	if (current[ARCH]) free(current[ARCH]);
+	if (current[ARCH]) free((void *)current[ARCH]);
 	current[ARCH] = xstrdup(arch);
 	rebuildCompatTables(ARCH, host_cpu);
     }
 
     if (!current[OS] || strcmp(os, current[OS])) {
-	if (current[OS]) free(current[OS]);
-	current[OS] = xstrdup(os);
+	char * t = xstrdup(os);
+	if (current[OS]) free((void *)current[OS]);
 	/*
 	 * XXX Capitalizing the 'L' is needed to insure that old
 	 * XXX os-from-uname (e.g. "Linux") is compatible with the new
@@ -1152,8 +1283,10 @@ void rpmSetMachine(const char * arch, const char * os) {
 	 * XXX used by rpmInstallPackage->{os,arch}Okay->rpmMachineScore->
 	 * XXX to verify correct arch/os from headers.
 	 */
-	if (!strcmp(current[OS], "linux"))
-	    *current[OS]= 'L';
+	if (!strcmp(t, "linux"))
+	    *t = 'L';
+	current[OS] = t;
+	
 	rebuildCompatTables(OS, host_os);
     }
 }
@@ -1164,10 +1297,10 @@ static void rebuildCompatTables(int type, const char * name) {
 		   name);
 }
 
-static void getMachineInfo(int type, /*@only@*/ /*@out@*/ const char ** name,
+static void getMachineInfo(int type, /*@out@*/ const char ** name,
 			/*@out@*/int * num)
 {
-    struct canonEntry * canon;
+    const struct canonEntry * canon;
     int which = currTables[type];
 
     /* use the normal canon tables, even if we're looking up build stuff */
@@ -1221,7 +1354,7 @@ void rpmRebuildTargetVars(const char **buildtarget, const char ** canontarget)
 	    if ((co = strrchr(c, '-')) == NULL) {
 		co = c;
 	    } else {
-		if (!strcasecmp(co, "-gnu"))
+		if (!xstrcasecmp(co, "-gnu"))
 		    *co = '\0';
 		if ((co = strrchr(c, '-')) == NULL)
 		    co = c;
@@ -1300,9 +1433,9 @@ void rpmFreeRpmrc(void)
 	t = tables + i;
 	if (t->equiv.list) {
 	    for (j = 0; j < t->equiv.count; j++) {
-		if (t->equiv.list[j].name)	xfree(t->equiv.list[j].name);
+		if (t->equiv.list[j].name) free((void *)t->equiv.list[j].name);
 	    }
-	    xfree(t->equiv.list);
+	    free((void *)t->equiv.list);
 	    t->equiv.list = NULL;
 	    t->equiv.count = 0;
 	}
@@ -1311,33 +1444,33 @@ void rpmFreeRpmrc(void)
 		struct machCacheEntry *e;
 		e = t->cache.cache + j;
 		if (e == NULL)	continue;
-		if (e->name)		xfree(e->name);
+		if (e->name)		free((void *)e->name);
 		if (e->equivs) {
 		    for (k = 0; k < e->count; k++) {
-			if (e->equivs[k])	xfree(e->equivs[k]);
+			if (e->equivs[k])	free((void *)e->equivs[k]);
 		    }
-		    xfree(e->equivs);
+		    free((void *)e->equivs);
 		}
 	    }
-	    xfree(t->cache.cache);
+	    free((void *)t->cache.cache);
 	    t->cache.cache = NULL;
 	    t->cache.size = 0;
 	}
 	if (t->defaults) {
 	    for (j = 0; j < t->defaultsLength; j++) {
-		if (t->defaults[j].name)	xfree(t->defaults[j].name);
-		if (t->defaults[j].defName)	xfree(t->defaults[j].defName);
+		if (t->defaults[j].name) free((void *)t->defaults[j].name);
+		if (t->defaults[j].defName) free((void *)t->defaults[j].defName);
 	    }
-	    xfree(t->defaults);
+	    free((void *)t->defaults);
 	    t->defaults = NULL;
 	    t->defaultsLength = 0;
 	}
 	if (t->canons) {
 	    for (j = 0; j < t->canonsLength; j++) {
-		if (t->canons[j].name)		xfree(t->canons[j].name);
-		if (t->canons[j].short_name)	xfree(t->canons[j].short_name);
+		if (t->canons[j].name)	free((void *)t->canons[j].name);
+		if (t->canons[j].short_name)	free((void *)t->canons[j].short_name);
 	    }
-	    xfree(t->canons);
+	    free((void *)t->canons);
 	    t->canons = NULL;
 	    t->canonsLength = 0;
 	}
@@ -1347,22 +1480,22 @@ void rpmFreeRpmrc(void)
 	struct rpmvarValue *this;
 	while ((this = values[i].next) != NULL) {
 	    values[i].next = this->next;
-	    if (this->value)	xfree(this->value);
-	    if (this->arch)	xfree(this->arch);
-	    xfree(this);
+	    if (this->value)	free((void *)this->value);
+	    if (this->arch)	free((void *)this->arch);
+	    free((void *)this);
 	}
 	if (values[i].value)
-	    xfree(values[i].value);
+	    free((void *)values[i].value);
 	values[i].value = NULL;
 	if (values[i].arch)
-	    xfree(values[i].arch);
+	    free((void *)values[i].arch);
 	values[i].arch = NULL;
     }
     if (current[OS])
-	xfree(current[OS]);
+	free((void *)current[OS]);
     current[OS] = NULL;
     if (current[ARCH])
-	xfree(current[ARCH]);
+	free((void *)current[ARCH]);
     current[ARCH] = NULL;
     defaultsInitialized = 0;
     return;
@@ -1413,7 +1546,7 @@ int rpmShowRC(FILE *fp)
     fprintf(fp, "\nRPMRC VALUES:\n");
     for (i = 0, opt = optionTable; i < optionTableSize; i++, opt++) {
 	const char *s = rpmGetVar(opt->var);
-	if (s != NULL || rpmGetVerbosity() < RPMMESS_NORMAL)
+	if (s != NULL || rpmIsVerbose())
 	    fprintf(fp, "%-21s : %s\n", opt->name, s ? s : "(not set)");
     }
     fprintf(fp, "\n");
