@@ -1,3 +1,7 @@
+/** \ingroup signature
+ * \file lib/signature.c
+ */
+
 /* signature.c - RPM signature functions */
 
 /* NOTES
@@ -20,8 +24,13 @@
 #include "misc.h"
 #include "rpmlead.h"
 #include "signature.h"
+#include "debug.h"
 
-typedef int (*md5func)(const char * fn, /*@out@*/unsigned char * digest);
+/*@access Header@*/		/* XXX compared with NULL */
+
+typedef	unsigned char byte;
+
+typedef int (*md5func)(const char * fn, /*@out@*/ byte * digest);
 
 int rpmLookupSignatureType(int action)
 {
@@ -42,17 +51,17 @@ int rpmLookupSignatureType(int action)
       { const char *name = rpmExpand("%{_signature}", NULL);
 	if (!(name && *name != '%'))
 	    rc = 0;
-	else if (!strcasecmp(name, "none"))
+	else if (!xstrcasecmp(name, "none"))
 	    rc = 0;
-	else if (!strcasecmp(name, "pgp"))
+	else if (!xstrcasecmp(name, "pgp"))
 	    rc = RPMSIGTAG_PGP;
-	else if (!strcasecmp(name, "pgp5"))	/* XXX legacy */
+	else if (!xstrcasecmp(name, "pgp5"))	/* XXX legacy */
 	    rc = RPMSIGTAG_PGP;
-	else if (!strcasecmp(name, "gpg"))
+	else if (!xstrcasecmp(name, "gpg"))
 	    rc = RPMSIGTAG_GPG;
 	else
 	    rc = -1;	/* Invalid %_signature spec in macro file */
-	xfree(name);
+	free((void *)name);
       }	break;
     }
     return rc;
@@ -72,18 +81,19 @@ const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
 
     if (saved_pgp_version == PGP_UNKNOWN) {
 	char *pgpvbin;
-	struct stat statbuf;
+	struct stat st;
 	
-	if (!(pgpbin && pgpbin[0] != '%') || ! (pgpvbin = (char *)alloca(strlen(pgpbin) + 2))) {
-	  if (pgpbin) xfree(pgpbin);
+	if (!(pgpbin && pgpbin[0] != '%')) {
+	  if (pgpbin) free((void *)pgpbin);
 	  saved_pgp_version = -1;
 	  return NULL;
 	}
-	sprintf(pgpvbin, "%sv", pgpbin);
+	pgpvbin = (char *)alloca(strlen(pgpbin) + sizeof("v"));
+	(void)stpcpy(stpcpy(pgpvbin, pgpbin), "v");
 
-	if (stat(pgpvbin, &statbuf) == 0)
+	if (stat(pgpvbin, &st) == 0)
 	  saved_pgp_version = PGP_5;
-	else if (stat(pgpbin, &statbuf) == 0)
+	else if (stat(pgpbin, &st) == 0)
 	  saved_pgp_version = PGP_2;
 	else
 	  saved_pgp_version = PGP_NOTDETECTED;
@@ -94,103 +104,98 @@ const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
     return pgpbin;
 }
 
-static int checkSize(FD_t fd, int size, int sigsize)
+static inline int checkSize(FD_t fd, int siglen, int pad, int datalen)
 {
-    int headerArchiveSize;
-    struct stat statbuf;
+    struct stat st;
 
-    fstat(Fileno(fd), &statbuf);
+    fstat(Fileno(fd), &st);
 
-    if (S_ISREG(statbuf.st_mode)) {
-	headerArchiveSize = statbuf.st_size - sizeof(struct rpmlead) - sigsize;
-
-	rpmMessage(RPMMESS_DEBUG, _("sigsize         : %d\n"), sigsize);
-	rpmMessage(RPMMESS_DEBUG, _("Header + Archive: %d\n"), headerArchiveSize);
-	rpmMessage(RPMMESS_DEBUG, _("expected size   : %d\n"), size);
-
-	return size - headerArchiveSize;
-    } else {
-	rpmMessage(RPMMESS_DEBUG, _("file is not regular -- skipping size check\n"));
+    if (!S_ISREG(st.st_mode)) {
+	rpmMessage(RPMMESS_DEBUG,
+	    _("file is not regular -- skipping size check\n"));
 	return 0;
     }
+    rpmMessage(RPMMESS_DEBUG,
+	_("Expected size: %12d = lead(%d)+sigs(%d)+pad(%d)+data(%d)\n"),
+		sizeof(struct rpmlead)+siglen+pad+datalen,
+		sizeof(struct rpmlead), siglen, pad, datalen);
+    rpmMessage(RPMMESS_DEBUG,
+	_("  Actual size: %12d\n"), st.st_size);
+
+    return ((sizeof(struct rpmlead) + siglen + pad + datalen) - st.st_size);
 }
 
-/* rpmReadSignature() emulates the new style signatures if it finds an */
-/* old-style one.  It also immediately verifies the header+archive  */
-/* size and returns an error if it doesn't match.                   */
-
-int rpmReadSignature(FD_t fd, Header *headerp, short sig_type)
+int rpmReadSignature(FD_t fd, Header *headerp, short sigType)
 {
-    unsigned char buf[2048];
+    byte buf[2048];
     int sigSize, pad;
     int_32 type, count;
     int_32 *archSize;
-    Header h;
+    Header h = NULL;
+    int nb;
+    int rc = 1;		/* assume failure */
 
     if (headerp)
 	*headerp = NULL;
-    
-    switch (sig_type) {
+
+    switch (sigType) {
       case RPMSIG_NONE:
 	rpmMessage(RPMMESS_DEBUG, _("No signature\n"));
+	rc = 0;
 	break;
       case RPMSIG_PGP262_1024:
 	rpmMessage(RPMMESS_DEBUG, _("Old PGP signature\n"));
 	/* These are always 256 bytes */
 	if (timedRead(fd, buf, 256) != 256)
-	    return 1;
-	if (headerp) {
-	    *headerp = headerNew();
-	    headerAddEntry(*headerp, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
-	}
+	    break;
+	h = headerNew();
+	headerAddEntry(h, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
+	rc = 0;
 	break;
       case RPMSIG_MD5:
       case RPMSIG_MD5_PGP:
 	rpmError(RPMERR_BADSIGTYPE,
-	      _("Old (internal-only) signature!  How did you get that!?"));
-	return 1;
-	/*@notreached@*/ break;
+	      _("Old (internal-only) signature!  How did you get that!?\n"));
+	break;
       case RPMSIG_HEADERSIG:
-	rpmMessage(RPMMESS_DEBUG, _("New Header signature\n"));
 	/* This is a new style signature */
 	h = headerRead(fd, HEADER_MAGIC_YES);
 	if (h == NULL)
-	    return 1;
+	    break;
 	sigSize = headerSizeof(h, HEADER_MAGIC_YES);
+
+	/* XXX Legacy headers have a HEADER_IMAGE tag added. */
+	if (headerIsEntry(h, RPMTAG_HEADERIMAGE))
+	    sigSize -= (16 + 16);
+
 	pad = (8 - (sigSize % 8)) % 8; /* 8-byte pad */
-	rpmMessage(RPMMESS_DEBUG, _("Signature size: %d\n"), sigSize);
-	rpmMessage(RPMMESS_DEBUG, _("Signature pad : %d\n"), pad);
-	if (! headerGetEntry(h, RPMSIGTAG_SIZE, &type, (void **)&archSize, &count)) {
-	    headerFree(h);
-	    return 1;
-	}
-	if (checkSize(fd, *archSize, sigSize + pad)) {
-	    headerFree(h);
-	    return 1;
-	}
+	if (! headerGetEntry(h, RPMSIGTAG_SIZE, &type, (void **)&archSize, &count))
+	    break;
+	nb = checkSize(fd, sigSize, pad, *archSize);
+	if (!(nb == 0 || nb == -(16+16)))
+	    break;
 	if (pad) {
-	    if (timedRead(fd, buf, pad) != pad) {
-		headerFree(h);
-		return 1;
-	    }
-	}
-	if (headerp) {
-	    *headerp = h;
-	} else {
-	    headerFree(h);
+	    if (timedRead(fd, buf, pad) != pad)
+		break;
 	}
+	rc = 0;
 	break;
       default:
-	return 1;
+	break;
     }
 
-    return 0;
+    if (rc == 0 && headerp)
+	*headerp = h;
+    else if (h)
+	headerFree(h);
+
+    return rc;
 }
 
 int rpmWriteSignature(FD_t fd, Header header)
 {
     int sigSize, pad;
-    unsigned char buf[8];
+    static byte buf[8] = "\000\000\000\000\000\000\000\000";
     int rc = 0;
     
     rc = headerWrite(fd, header, HEADER_MAGIC_YES);
@@ -202,7 +207,6 @@ int rpmWriteSignature(FD_t fd, Header header)
     if (pad) {
 	rpmMessage(RPMMESS_DEBUG, _("Signature size: %d\n"), sigSize);
 	rpmMessage(RPMMESS_DEBUG, _("Signature pad : %d\n"), pad);
-	memset(buf, 0, pad);
 	if (Fwrite(buf, sizeof(buf[0]), pad, fd) != pad)
 	    rc = 1;
     }
@@ -226,7 +230,7 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     char sigfile[1024];
     int pid, status;
     int inpipe[2];
-    struct stat statbuf;
+    struct stat st;
 
     sprintf(sigfile, "%s.sig", file);
 
@@ -264,7 +268,7 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 		break;
 	    }
 	}
-	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)"), path);
+	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)\n"), path);
 	_exit(RPMERR_EXEC);
     }
 
@@ -275,18 +279,18 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 
     (void)waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	rpmError(RPMERR_SIGGEN, _("pgp failed"));
+	rpmError(RPMERR_SIGGEN, _("pgp failed\n"));
 	return 1;
     }
 
-    if (stat(sigfile, &statbuf)) {
+    if (stat(sigfile, &st)) {
 	/* PGP failed to write signature */
 	unlink(sigfile);  /* Just in case */
-	rpmError(RPMERR_SIGGEN, _("pgp failed to write signature"));
+	rpmError(RPMERR_SIGGEN, _("pgp failed to write signature\n"));
 	return 1;
     }
 
-    *size = statbuf.st_size;
+    *size = st.st_size;
     rpmMessage(RPMMESS_DEBUG, _("PGP sig size: %d\n"), *size);
     *sig = xmalloc(*size);
     
@@ -298,7 +302,7 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 	Fclose(fd);
 	if (rc != *size) {
 	    free(*sig);
-	    rpmError(RPMERR_SIGGEN, _("unable to read the signature"));
+	    rpmError(RPMERR_SIGGEN, _("unable to read the signature\n"));
 	    return 1;
 	}
     }
@@ -320,7 +324,7 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     int pid, status;
     int inpipe[2];
     FILE *fpipe;
-    struct stat statbuf;
+    struct stat st;
 
     sprintf(sigfile, "%s.sig", file);
 
@@ -341,7 +345,7 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 	       "--batch", "--no-verbose", "--no-armor", "--passphrase-fd", "3",
 	       "-u", name, "-sbo", sigfile, file,
 	       NULL);
-	rpmError(RPMERR_EXEC, _("Couldn't exec gpg"));
+	rpmError(RPMERR_EXEC, _("Couldn't exec gpg\n"));
 	_exit(RPMERR_EXEC);
     }
 
@@ -352,18 +356,18 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 
     (void)waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	rpmError(RPMERR_SIGGEN, _("gpg failed"));
+	rpmError(RPMERR_SIGGEN, _("gpg failed\n"));
 	return 1;
     }
 
-    if (stat(sigfile, &statbuf)) {
+    if (stat(sigfile, &st)) {
 	/* GPG failed to write signature */
 	unlink(sigfile);  /* Just in case */
-	rpmError(RPMERR_SIGGEN, _("gpg failed to write signature"));
+	rpmError(RPMERR_SIGGEN, _("gpg failed to write signature\n"));
 	return 1;
     }
 
-    *size = statbuf.st_size;
+    *size = st.st_size;
     rpmMessage(RPMMESS_DEBUG, _("GPG sig size: %d\n"), *size);
     *sig = xmalloc(*size);
     
@@ -375,7 +379,7 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 	Fclose(fd);
 	if (rc != *size) {
 	    free(*sig);
-	    rpmError(RPMERR_SIGGEN, _("unable to read the signature"));
+	    rpmError(RPMERR_SIGGEN, _("unable to read the signature\n"));
 	    return 1;
 	}
     }
@@ -387,16 +391,16 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 
 int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *passPhrase)
 {
-    struct stat statbuf;
+    struct stat st;
     int_32 size;
-    unsigned char buf[16];
+    byte buf[16];
     void *sig;
     int ret = -1;
     
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
-	stat(file, &statbuf);
-	size = statbuf.st_size;
+	stat(file, &st);
+	size = st.st_size;
 	ret = 0;
 	headerAddEntry(header, RPMSIGTAG_SIZE, RPM_INT32_TYPE, &size, 1);
 	break;
@@ -423,28 +427,30 @@ int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *
     return ret;
 }
 
-static int verifySizeSignature(const char *datafile, int_32 size, char *result)
+static rpmVerifySignatureReturn
+verifySizeSignature(const char *datafile, int_32 size, char *result)
 {
-    struct stat statbuf;
+    struct stat st;
 
-    stat(datafile, &statbuf);
-    if (size != statbuf.st_size) {
+    stat(datafile, &st);
+    if (size != st.st_size) {
 	sprintf(result, "Header+Archive size mismatch.\n"
 		"Expected %d, saw %d.\n",
-		size, (int)statbuf.st_size);
-	return 1;
+		size, (int)st.st_size);
+	return RPMSIG_BAD;
     }
 
     sprintf(result, "Header+Archive size OK: %d bytes\n", size);
-    return 0;
+    return RPMSIG_OK;
 }
 
 #define	X(_x)	(unsigned)((_x) & 0xff)
 
-static int verifyMD5Signature(const char *datafile, unsigned char *sig, 
+static rpmVerifySignatureReturn
+verifyMD5Signature(const char *datafile, const byte *sig, 
 			      char *result, md5func fn)
 {
-    unsigned char md5sum[16];
+    byte md5sum[16];
 
     fn(datafile, md5sum);
     if (memcmp(md5sum, sig, 16)) {
@@ -461,7 +467,7 @@ static int verifyMD5Signature(const char *datafile, unsigned char *sig,
 		X(md5sum[4]),  X(md5sum[5]),  X(md5sum[6]),  X(md5sum[7]),
 		X(md5sum[8]),  X(md5sum[9]),  X(md5sum[10]), X(md5sum[11]),
 		X(md5sum[12]), X(md5sum[13]), X(md5sum[14]), X(md5sum[15]) );
-	return 1;
+	return RPMSIG_BAD;
     }
 
     sprintf(result, "MD5 sum OK: %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
@@ -471,16 +477,16 @@ static int verifyMD5Signature(const char *datafile, unsigned char *sig,
 	    X(md5sum[8]),  X(md5sum[9]),  X(md5sum[10]), X(md5sum[11]),
 	    X(md5sum[12]), X(md5sum[13]), X(md5sum[14]), X(md5sum[15]) );
 
-    return 0;
+    return RPMSIG_OK;
 }
 
-static int verifyPGPSignature(const char *datafile, void *sig,
-			      int count, char *result)
+static rpmVerifySignatureReturn
+verifyPGPSignature(const char *datafile, const void * sig, int count, char *result)
 {
     int pid, status, outpipe[2];
     FD_t sfd;
     char *sigfile;
-    unsigned char buf[BUFSIZ];
+    byte buf[BUFSIZ];
     FILE *file;
     int res = RPMSIG_OK;
     const char *path;
@@ -490,7 +496,7 @@ static int verifyPGPSignature(const char *datafile, void *sig,
     if ((path = rpmDetectPGPVersion(&pgpVer)) == NULL) {
 	errno = ENOENT;
 	rpmError(RPMERR_EXEC, 
-		 _("Could not run pgp.  Use --nopgp to skip PGP checks."));
+		 _("Could not run pgp.  Use --nopgp to skip PGP checks.\n"));
 	_exit(RPMERR_EXEC);
     }
 
@@ -504,7 +510,7 @@ static int verifyPGPSignature(const char *datafile, void *sig,
     /* Write out the signature */
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
     sigfile = tempnam(tmppath, "rpmsig");
-    xfree(tmppath);
+    free((void *)tmppath);
   }
     sfd = Fopen(sigfile, "w.fdio");
     (void)Fwrite(sig, sizeof(char), count, sfd);
@@ -550,9 +556,8 @@ static int verifyPGPSignature(const char *datafile, void *sig,
 	    break;
 	}
 
-	fprintf(stderr, _("exec failed!\n"));
 	rpmError(RPMERR_EXEC, 
-		 _("Could not run pgp.  Use --nopgp to skip PGP checks."));
+		 _("Could not run pgp.  Use --nopgp to skip PGP checks.\n"));
 	_exit(RPMERR_EXEC);
     }
 
@@ -586,20 +591,20 @@ static int verifyPGPSignature(const char *datafile, void *sig,
     return res;
 }
 
-static int verifyGPGSignature(const char *datafile, void *sig,
-			      int count, char *result)
+static rpmVerifySignatureReturn
+verifyGPGSignature(const char *datafile, const void * sig, int count, char *result)
 {
     int pid, status, outpipe[2];
     FD_t sfd;
     char *sigfile;
-    unsigned char buf[8192];
+    byte buf[BUFSIZ];
     FILE *file;
     int res = RPMSIG_OK;
   
     /* Write out the signature */
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
     sigfile = tempnam(tmppath, "rpmsig");
-    xfree(tmppath);
+    free((void *)tmppath);
   }
     sfd = Fopen(sigfile, "w.fdio");
     (void)Fwrite(sig, sizeof(char), count, sfd);
@@ -623,9 +628,8 @@ static int verifyGPGSignature(const char *datafile, void *sig,
 	       "--batch", "--no-verbose", 
 	       "--verify", sigfile, datafile,
 	       NULL);
-	fprintf(stderr, _("exec failed!\n"));
 	rpmError(RPMERR_EXEC, 
-		 _("Could not run gpg.  Use --nogpg to skip GPG checks."));
+		 _("Could not run gpg.  Use --nogpg to skip GPG checks.\n"));
 	_exit(RPMERR_EXEC);
     }
 
@@ -634,7 +638,7 @@ static int verifyGPGSignature(const char *datafile, void *sig,
     result[0] = '\0';
     while (fgets(buf, 1024, file)) {
 	strcat(result, buf);
-	if (!strncmp("gpg: Can't check signature: Public key not found", buf, 48)) {
+	if (!xstrncasecmp("gpg: Can't check signature: Public key not found", buf, 48)) {
 	    res = RPMSIG_NOKEY;
 	}
     }
@@ -684,7 +688,7 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
 	           "--batch", "--no-verbose", "--passphrase-fd", "3",
 	           "-u", name, "-so", "-",
 	           NULL);
-	    rpmError(RPMERR_EXEC, _("Couldn't exec gpg"));
+	    rpmError(RPMERR_EXEC, _("Couldn't exec gpg\n"));
 	    _exit(RPMERR_EXEC);
 	}   /*@notreached@*/ break;
 	case RPMSIGTAG_PGP5:	/* XXX legacy */
@@ -713,11 +717,11 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
 		    break;
 		}
 	    }
-	    rpmError(RPMERR_EXEC, _("Couldn't exec pgp"));
+	    rpmError(RPMERR_EXEC, _("Couldn't exec pgp\n"));
 	    _exit(RPMERR_EXEC);
 	}   /*@notreached@*/ break;
 	default: /* This case should have been screened out long ago. */
-	    rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file"));
+	    rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file\n"));
 	    _exit(RPMERR_SIGGEN);
 	    /*@notreached@*/ break;
 	}
@@ -746,11 +750,11 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
     case RPMSIGTAG_GPG:
       { const char *name = rpmExpand("%{_gpg_name}", NULL);
 	aok = (name && *name != '%');
-	xfree(name);
+	free((void *)name);
       }
 	if (!aok) {
 	    rpmError(RPMERR_SIGGEN,
-		_("You must set \"%%_gpg_name\" in your macro file"));
+		_("You must set \"%%_gpg_name\" in your macro file\n"));
 	    return NULL;
 	}
 	break;
@@ -758,11 +762,11 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
     case RPMSIGTAG_PGP: 
       { const char *name = rpmExpand("%{_pgp_name}", NULL);
 	aok = (name && *name != '%');
-	xfree(name);
+	free((void *)name);
       }
 	if (!aok) {
 	    rpmError(RPMERR_SIGGEN,
-		_("You must set \"%%_pgp_name\" in your macro file"));
+		_("You must set \"%%_pgp_name\" in your macro file\n"));
 	    return NULL;
 	}
 	break;
@@ -770,7 +774,7 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
 	/* Currently the calling function (rpm.c:main) is checking this and
 	 * doing a better job.  This section should never be accessed.
 	 */
-	rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file"));
+	rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file\n"));
 	return NULL;
 	/*@notreached@*/ break;
     }
@@ -783,26 +787,21 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
     return pass;
 }
 
-int rpmVerifySignature(const char *file, int_32 sigTag, void *sig, int count,
+rpmVerifySignatureReturn
+rpmVerifySignature(const char *file, int_32 sigTag, const void * sig, int count,
 		    char *result)
 {
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
-	if (verifySizeSignature(file, *(int_32 *)sig, result)) {
-	    return RPMSIG_BAD;
-	}
-	break;
+	return verifySizeSignature(file, *(int_32 *)sig, result);
+	/*@notreached@*/ break;
     case RPMSIGTAG_MD5:
-	if (verifyMD5Signature(file, sig, result, mdbinfile)) {
-	    return 1;
-	}
-	break;
+	return verifyMD5Signature(file, sig, result, mdbinfile);
+	/*@notreached@*/ break;
     case RPMSIGTAG_LEMD5_1:
     case RPMSIGTAG_LEMD5_2:
-	if (verifyMD5Signature(file, sig, result, mdbinfileBroken)) {
-	    return 1;
-	}
-	break;
+	return verifyMD5Signature(file, sig, result, mdbinfileBroken);
+	/*@notreached@*/ break;
     case RPMSIGTAG_PGP5:	/* XXX legacy */
     case RPMSIGTAG_PGP:
 	return verifyPGPSignature(file, sig, count, result);
@@ -814,6 +813,6 @@ int rpmVerifySignature(const char *file, int_32 sigTag, void *sig, int count,
 	sprintf(result, "Do not know how to verify sig type %d\n", sigTag);
 	return RPMSIG_UNKNOWN;
     }
-
+    /*@notreached@*/
     return RPMSIG_OK;
 }
