@@ -1,7 +1,10 @@
 #ifndef H_SIGNATURE
 #define	H_SIGNATURE
 
-/* signature.h - generate and verify signatures */
+/** \ingroup signature
+ * \file lib/signature.h
+ * Generate and verify signatures.
+ */
 
 #include <header.h>
 
@@ -29,19 +32,33 @@ extern "C" {
 
 #define RPMSIG_HEADERSIG    5  /* New Header style signature */
 
-/**************************************************/
-/*                                                */
-/* Prototypes                                     */
-/*                                                */
-/**************************************************/
-
+/** \ingroup signature
+ * Return new, empty (signature) header instance.
+ * @return		new
+ */
 Header rpmNewSignature(void);
 
-/* If an old-style signature is found, we emulate a new style one */
+/** \ingroup signature
+ * Read (and verify header+archive size) signature header.
+ * If an old-style signature is found, we emulate a new style one.
+ * @param fd		file handle
+ * @retval header	address of (signature) header
+ * @param sig_type	type of signature header to read (from lead).
+ * @return		0 on success, 1 on error
+ */
 int rpmReadSignature(FD_t fd, /*@out@*/ Header *header, short sig_type);
+
+/** \ingroup signature
+ * Write signature header.
+ * @param fd		file handle
+ * @param header	(signature) header
+ * @return		0 on success, 1 on error
+ */
 int rpmWriteSignature(FD_t fd, Header header);
 
-/* Generate a signature of data in file, insert in header */
+/** \ingroup signature
+ *  Generate a signature of data in file, insert in header.
+ */
 int rpmAddSignature(Header header, const char *file,
 		    int_32 sigTag, const char *passPhrase);
 
@@ -52,18 +69,30 @@ int rpmAddSignature(Header header, const char *file,
 #define RPMLOOKUPSIG_DISABLE	1	/* Disable (--sign was not given) */
 #define RPMLOOKUPSIG_ENABLE	2	/* Re-enable %_signature          */
 
-/* Return type of signature in effect for building */
+/** \ingroup signature
+ * Return type of signature in effect for building.
+ */
 int rpmLookupSignatureType(int action);
 
-/* Utility to read a pass phrase from the user */
+/** \ingroup signature
+ *  Read a pass phrase from the user.
+ */
 char *rpmGetPassPhrase(const char *prompt, const int sigTag);
 
-/* >0 is a valid PGP version */
+/** \ingroup signature
+ * Identify PGP versions.
+ * @note Greater than 0 is a valid PGP version.
+ */
 typedef enum pgpVersion_e {
-	PGP_NOTDETECTED = -1, PGP_UNKNOWN = 0, PGP_2 = 2, PGP_5 = 5
+	PGP_NOTDETECTED = -1,
+	PGP_UNKNOWN = 0,
+	PGP_2 = 2,
+	PGP_5 = 5
 } pgpVersion;
 
-/* Return path to pgp executable of given type, or NULL when not found */
+/** \ingroup signature
+ *  Return path to pgp executable of given type, or NULL when not found.
+ */
 const char *rpmDetectPGPVersion( /*@out@*/ pgpVersion *pgpVersion);
 
 #ifdef __cplusplus
