@@ -1,12 +1,17 @@
+/**
+ * \file lib/stringbuf.c
+ */
+
 #include "system.h"
 
 #include "stringbuf.h"
+#include "debug.h"
 
 #define BUF_CHUNK 1024
 
 struct StringBufRec {
-    char *buf;
-    char *tail;     /* Points to first "free" char (usually '\0') */
+    /*@owned@*/ char *buf;
+    /*@dependent@*/ char *tail;     /* Points to first "free" char */
     int allocated;
     int free;
 };
