@@ -1,20 +1,50 @@
+/** \ingroup rpmtrans payload
+ * \file lib/uninstall.c
+ */
+
 #include "system.h"
 
 #include <rpmlib.h>
 #include <rpmurl.h>
 #include <rpmmacro.h>	/* XXX for rpmExpand */
 
-#include "depends.h"
+#include "depends.h"	/* XXX for headerMatchesDepFlags */
 #include "install.h"
-#include "md5.h"
-#include "misc.h"
-#include "rpmdb.h"
+#include "misc.h"	/* XXX for makeTempFile, doputenv */
+#include "debug.h"
+
+/*@access Header@*/		/* XXX compared with NULL */
 
 static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin";
 
 #define	SUFFIX_RPMSAVE	".rpmsave"
 
-static int removeFile(const char * file, unsigned int flags, short mode, 
+/**
+ * Return scriptlet name from tag.
+ * @param tag		scriptlet tag
+ * @return		name of scriptlet
+ */
+static const char * tag2sln(int tag)
+{
+    switch (tag) {
+    case RPMTAG_PREIN:		return "%pre";
+    case RPMTAG_POSTIN:		return "%post";
+    case RPMTAG_PREUN:		return "%preun";
+    case RPMTAG_POSTUN:		return "%postun";
+    case RPMTAG_VERIFYSCRIPT:	return "%verify";
+    }
+    return "%unknownscript";
+}
+
+/**
+ * Remove (or rename) file according to file disposition.
+ * @param file		file
+ * @param fileAttrs	file attributes (from package header)
+ * @param mode		file type
+ * @param action	file disposition
+ * @return
+ */
+static int removeFile(const char * file, rpmfileAttrs fileAttrs, short mode, 
 		      enum fileActions action)
 {
     int rc = 0;
@@ -24,10 +54,10 @@ static int removeFile(const char * file, unsigned int flags, short mode,
 
       case FA_BACKUP:
 	newfile = alloca(strlen(file) + sizeof(SUFFIX_RPMSAVE));
-	stpcpy(stpcpy(newfile, file), SUFFIX_RPMSAVE);
+	(void)stpcpy(stpcpy(newfile, file), SUFFIX_RPMSAVE);
 
 	if (rename(file, newfile)) {
-	    rpmError(RPMERR_RENAME, _("rename of %s to %s failed: %s"),
+	    rpmError(RPMERR_RENAME, _("rename of %s to %s failed: %s\n"),
 			file, newfile, strerror(errno));
 	    rc = 1;
 	}
@@ -35,21 +65,27 @@ static int removeFile(const char * file, unsigned int flags, short mode,
 
       case FA_REMOVE:
 	if (S_ISDIR(mode)) {
+	    /* XXX should permit %missingok for directories as well */
 	    if (rmdir(file)) {
-		if (errno == ENOTEMPTY)
+		switch (errno) {
+		case ENOENT:	/* XXX rmdir("/") linux 2.2.x kernel hack */
+		case ENOTEMPTY:
 		    rpmError(RPMERR_RMDIR, 
-			_("cannot remove %s - directory not empty"), 
+			_("cannot remove %s - directory not empty\n"), 
 			file);
-		else
-		    rpmError(RPMERR_RMDIR, _("rmdir of %s failed: %s"),
+		    break;
+		default:
+		    rpmError(RPMERR_RMDIR, _("rmdir of %s failed: %s\n"),
 				file, strerror(errno));
+		    break;
+		}
 		rc = 1;
 	    }
 	} else {
 	    if (unlink(file)) {
-		if (errno != ENOENT || !(flags & RPMFILE_MISSINGOK)) {
+		if (errno != ENOENT || !(fileAttrs & RPMFILE_MISSINGOK)) {
 		    rpmError(RPMERR_UNLINK, 
-			      _("removal of %s failed: %s"),
+			      _("removal of %s failed: %s\n"),
 				file, strerror(errno));
 		}
 		rc = 1;
@@ -63,19 +99,17 @@ static int removeFile(const char * file, unsigned int flags, short mode,
       case FA_SKIP:
       case FA_SKIPNSTATE:
       case FA_SKIPNETSHARED:
+      case FA_SKIPMULTILIB:
 	break;
     }
  
     return 0;
 }
 
-/** */
-int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
-			Header h,
-			int flags, rpmCallbackFunction notify,
-			void * notifyData, const void * pkgKey,
-			enum fileActions * actions, FD_t scriptFd)
+int removeBinaryPackage(const rpmTransactionSet ts, unsigned int offset,
+		Header h, const void * pkgKey, enum fileActions * actions)
 {
+    rpmtransFlags transFlags = ts->transFlags;
     const char * name, * version, * release;
     const char ** baseNames;
     int scriptArg;
@@ -83,8 +117,8 @@ int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
     int fileCount;
     int i;
 
-    if (flags & RPMTRANS_FLAG_JUSTDB)
-	flags |= RPMTRANS_FLAG_NOSCRIPTS;
+    if (transFlags & RPMTRANS_FLAG_JUSTDB)
+	transFlags |= RPMTRANS_FLAG_NOSCRIPTS;
 
     headerNVR(h, &name, &version, &release);
 
@@ -92,45 +126,32 @@ int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
      * When we run scripts, we pass an argument which is the number of 
      * versions of this package that will be installed when we are finished.
      */
-    {	dbiIndexSet matches;
-	if (rpmdbFindPackage(db, name, &matches)) {
-	    rpmError(RPMERR_DBCORRUPT, _("cannot read packages named %s for uninstall"),
-		name);
-	    dbiFreeIndexRecord(matches);
-	    rc = 1;
-	    goto exit;
-	}
-	scriptArg = dbiIndexSetCount(matches) - 1;
-	dbiFreeIndexRecord(matches);
-    }
+    if ((scriptArg = rpmdbCountPackages(ts->rpmdb, name)) < 0)
+	return 1;
+    scriptArg -= 1;
 
-    if (!(flags & RPMTRANS_FLAG_NOTRIGGERS)) {
+    if (!(transFlags & RPMTRANS_FLAG_NOTRIGGERS)) {
 	/* run triggers from this package which are keyed on installed 
 	   packages */
-	if (runImmedTriggers(prefix, db, RPMSENSE_TRIGGERUN, h, -1, scriptFd)) {
-	    rc = 2;
-	    goto exit;
-	}
+	if (runImmedTriggers(ts, RPMSENSE_TRIGGERUN, h, -1))
+	    return 2;
 
 	/* run triggers which are set off by the removal of this package */
-	if (runTriggers(prefix, db, RPMSENSE_TRIGGERUN, h, -1, scriptFd)) {
-	    rc = 1;
-	    goto exit;
-	}
+	if (runTriggers(ts, RPMSENSE_TRIGGERUN, h, -1))
+	    return 1;
     }
 
-    if (!(flags & RPMTRANS_FLAG_TEST)) {
-	if (runInstScript(prefix, h, RPMTAG_PREUN, RPMTAG_PREUNPROG, scriptArg, 
-		          flags & RPMTRANS_FLAG_NOSCRIPTS, scriptFd)) {
-	    rc = 1;
-	    goto exit;
-	}
+    if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+	rc = runInstScript(ts, h, RPMTAG_PREUN, RPMTAG_PREUNPROG, scriptArg,
+		          (transFlags & RPMTRANS_FLAG_NOSCRIPTS));
+	if (rc)
+	    return 1;
     }
     
     rpmMessage(RPMMESS_DEBUG, _("will remove files test = %d\n"), 
-		flags & RPMTRANS_FLAG_TEST);
+		transFlags & RPMTRANS_FLAG_TEST);
 
-    if (!(flags & RPMTRANS_FLAG_JUSTDB) &&
+    if (!(transFlags & RPMTRANS_FLAG_JUSTDB) &&
 	headerGetEntry(h, RPMTAG_BASENAMES, NULL, (void **) &baseNames, 
 	               &fileCount)) {
 	const char ** fileMd5List;
@@ -141,15 +162,13 @@ int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
 	int type;
 	char * fileName;
 	int fnmaxlen;
-	int prefixlen = (prefix && !(prefix[0] == '/' && prefix[1] == '\0'))
-			? strlen(prefix) : 0;
+	int rdlen = (ts->rootDir && !(ts->rootDir[0] == '/' && ts->rootDir[1] == '\0'))
+			? strlen(ts->rootDir) : 0;
 
-	headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void **) &dirIndexes,
-	               NULL);
-	headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void **) &dirNames,
-	               NULL);
+	headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void **) &dirIndexes, NULL);
+	headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void **) &dirNames, NULL);
 
-	/* Get buffer for largest possible prefix + dirname + filename. */
+	/* Get buffer for largest possible rootDir + dirname + filename. */
 	fnmaxlen = 0;
 	for (i = 0; i < fileCount; i++) {
 		size_t fnlen;
@@ -158,14 +177,14 @@ int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
 		if (fnlen > fnmaxlen)
 		    fnmaxlen = fnlen;
 	}
-	fnmaxlen += prefixlen + sizeof("/");	/* XXX one byte too many */
+	fnmaxlen += rdlen + sizeof("/");	/* XXX one byte too many */
 
 	fileName = alloca(fnmaxlen);
 
-	if (prefixlen) {
-	    strcpy(fileName, prefix);
-	    rpmCleanPath(fileName);
-	    prefixlen = strlen(fileName);
+	if (rdlen) {
+	    strcpy(fileName, ts->rootDir);
+	    (void)rpmCleanPath(fileName);
+	    rdlen = strlen(fileName);
 	} else
 	    *fileName = '\0';
 
@@ -176,72 +195,75 @@ int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset,
 	headerGetEntry(h, RPMTAG_FILEMODES, &type, (void **) &fileModesList, 
 		 &fileCount);
 
-	if (notify) {
-	    (void)notify(h, RPMCALLBACK_UNINST_START, fileCount, fileCount,
-		pkgKey, notifyData);
+	if (ts->notify) {
+	    (void)ts->notify(h, RPMCALLBACK_UNINST_START, fileCount, fileCount,
+		pkgKey, ts->notifyData);
 	}
 
 	/* Traverse filelist backwards to help insure that rmdir() will work. */
 	for (i = fileCount - 1; i >= 0; i--) {
 
 	    /* XXX this assumes that dirNames always starts/ends with '/' */
-	    stpcpy(stpcpy(fileName+prefixlen, dirNames[dirIndexes[i]]), baseNames[i]);
+	    (void)stpcpy(stpcpy(fileName+rdlen, dirNames[dirIndexes[i]]), baseNames[i]);
 
 	    rpmMessage(RPMMESS_DEBUG, _("   file: %s action: %s\n"),
 			fileName, fileActionString(actions[i]));
 
-	    if (!(flags & RPMTRANS_FLAG_TEST)) {
-		if (notify) {
-		    (void)notify(h, RPMCALLBACK_UNINST_PROGRESS,
-			i, actions[i], fileName, notifyData);
+	    if (!(transFlags & RPMTRANS_FLAG_TEST)) {
+		if (ts->notify) {
+		    (void)ts->notify(h, RPMCALLBACK_UNINST_PROGRESS,
+			i, actions[i], fileName, ts->notifyData);
 		}
 		removeFile(fileName, fileFlagsList[i], fileModesList[i], 
 			   actions[i]);
 	    }
 	}
 
-	if (notify) {
-	    (void)notify(h, RPMCALLBACK_UNINST_STOP, 0, fileCount,
-			pkgKey, notifyData);
-        }
+	if (ts->notify) {
+	    (void)ts->notify(h, RPMCALLBACK_UNINST_STOP, 0, fileCount,
+			pkgKey, ts->notifyData);
+	}
 
 	free(baseNames);
 	free(dirNames);
 	free(fileMd5List);
     }
 
-    if (!(flags & RPMTRANS_FLAG_TEST)) {
+    if (!(transFlags & RPMTRANS_FLAG_TEST)) {
 	rpmMessage(RPMMESS_DEBUG, _("running postuninstall script (if any)\n"));
-	runInstScript(prefix, h, RPMTAG_POSTUN, RPMTAG_POSTUNPROG, scriptArg, 
-		      flags & RPMTRANS_FLAG_NOSCRIPTS, scriptFd);
+	rc = runInstScript(ts, h, RPMTAG_POSTUN, RPMTAG_POSTUNPROG,
+			scriptArg, (transFlags & RPMTRANS_FLAG_NOSCRIPTS));
+	/* XXX postun failures are not cause for erasure failure. */
     }
 
-    if (!(flags & RPMTRANS_FLAG_NOTRIGGERS)) {
+    if (!(transFlags & RPMTRANS_FLAG_NOTRIGGERS)) {
 	/* Run postun triggers which are set off by this package's removal. */
-	if (runTriggers(prefix, db, RPMSENSE_TRIGGERPOSTUN, h, -1, scriptFd)) {
-	    rc = 2;
-	    goto exit;
-	}
+	rc = runTriggers(ts, RPMSENSE_TRIGGERPOSTUN, h, -1);
+	if (rc)
+	    return 2;
     }
 
-    if (!(flags & RPMTRANS_FLAG_TEST)) {
-	rpmMessage(RPMMESS_DEBUG, _("removing database entry\n"));
-	/*
-	 * XXX rpmdbRemove used to have 2nd arg, tolerant = 0. This generates
-	 * XXX an error message because of a db-1.85 hash access bug that
-	 * XXX cannot be easily fixed. So, we turn off the error message.
-	 */
-	rpmdbRemove(db, offset, 0);
-    }
+    if (!(transFlags & RPMTRANS_FLAG_TEST))
+	rpmdbRemove(ts->rpmdb, ts->id, offset);
 
-    rc = 0;
-
- exit:
-    return rc;
+    return 0;
 }
 
-static int runScript(Header h, const char * root, int progArgc, const char ** progArgv, 
-		     const char * script, int arg1, int arg2, FD_t errfd)
+/**
+ * @param ts		transaction set
+ * @param h		header
+ * @param sln		name of scriptlet section
+ * @param progArgc
+ * @param progArgv
+ * @param script
+ * @param arg1
+ * @param arg2
+ * @return
+ */
+static int runScript(const rpmTransactionSet ts, Header h,
+		const char * sln,
+		int progArgc, const char ** progArgv, 
+		const char * script, int arg1, int arg2)
 {
     const char ** argv = NULL;
     int argc = 0;
@@ -252,11 +274,13 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
     int len;
     char * prefixBuf = NULL;
     pid_t child;
-    int status;
+    int status = 0;
     const char * fn = NULL;
     int i;
     int freePrefixes = 0;
     FD_t out;
+    int rc = 0;
+    const char *n, *v, *r;
 
     if (!progArgv && !script)
 	return 0;
@@ -270,6 +294,7 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	argc = progArgc;
     }
 
+    headerNVR(h, &n, &v, &r);
     if (headerGetEntry(h, RPMTAG_INSTPREFIXES, NULL, (void **) &prefixes,
 		       &numPrefixes)) {
 	freePrefixes = 1;
@@ -291,7 +316,7 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 
     if (script) {
 	FD_t fd;
-	if (makeTempFile(root, &fn, &fd)) {
+	if (makeTempFile((!ts->chrootDone ? ts->rootDir : "/"), &fn, &fd)) {
 	    if (freePrefixes) free(prefixes);
 	    return 1;
 	}
@@ -303,7 +328,15 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	(void)Fwrite(script, sizeof(script[0]), strlen(script), fd);
 	Fclose(fd);
 
-	argv[argc++] = fn + strlen(root);
+	{   const char * sn = fn;
+	    if (!ts->chrootDone &&
+		!(ts->rootDir[0] == '/' && ts->rootDir[1] == '\0'))
+	    {
+		sn += strlen(ts->rootDir)-1;
+	    }
+	    argv[argc++] = sn;
+	}
+
 	if (arg1 >= 0) {
 	    char *av = alloca(20);
 	    sprintf(av, "%d", arg1);
@@ -318,13 +351,13 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 
     argv[argc] = NULL;
 
-    if (errfd != NULL) {
+    if (ts->scriptFd != NULL) {
 	if (rpmIsVerbose()) {
-	    out = fdDup(Fileno(errfd));
+	    out = fdDup(Fileno(ts->scriptFd));
 	} else {
 	    out = Fopen("/dev/null", "w.fdio");
 	    if (Ferror(out)) {
-		out = fdDup(Fileno(errfd));
+		out = fdDup(Fileno(ts->scriptFd));
 	    }
 	}
     } else {
@@ -333,6 +366,7 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
     }
     
     if (!(child = fork())) {
+	const char * rootDir;
 	int pipes[2];
 
 	pipes[0] = pipes[1] = 0;
@@ -342,17 +376,17 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	dup2(pipes[0], STDIN_FILENO);
 	close(pipes[0]);
 
-	if (errfd != NULL) {
-	    if (Fileno(errfd) != STDERR_FILENO)
-		dup2(Fileno(errfd), STDERR_FILENO);
+	if (ts->scriptFd != NULL) {
+	    if (Fileno(ts->scriptFd) != STDERR_FILENO)
+		dup2(Fileno(ts->scriptFd), STDERR_FILENO);
 	    if (Fileno(out) != STDOUT_FILENO)
 		dup2(Fileno(out), STDOUT_FILENO);
 	    /* make sure we don't close stdin/stderr/stdout by mistake! */
-	    if (Fileno(out) > STDERR_FILENO && Fileno(out) != Fileno(errfd)) {
+	    if (Fileno(out) > STDERR_FILENO && Fileno(out) != Fileno(ts->scriptFd)) {
 		Fclose (out);
 	    }
-	    if (Fileno(errfd) > STDERR_FILENO) {
-		Fclose (errfd);
+	    if (Fileno(ts->scriptFd) > STDERR_FILENO) {
+		Fclose (ts->scriptFd);
 	    }
 	}
 
@@ -362,7 +396,7 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	    if (ipath && ipath[5] != '%')
 		path = ipath;
 	    doputenv(path);
-	    if (ipath)	xfree(ipath);
+	    if (ipath)	free((void *)ipath);
 	}
 
 	for (i = 0; i < numPrefixes; i++) {
@@ -376,14 +410,16 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	    }
 	}
 
-	switch(urlIsURL(root)) {
+	rootDir = ts->rootDir;
+	switch(urlIsURL(rootDir)) {
 	case URL_IS_PATH:
-	    root += sizeof("file://") - 1;
-	    root = strchr(root, '/');
+	    rootDir += sizeof("file://") - 1;
+	    rootDir = strchr(rootDir, '/');
 	    /*@fallthrough@*/
 	case URL_IS_UNKNOWN:
-	    if (strcmp(root, "/"))
-		/*@-unrecog@*/ chroot(root); /*@=unrecog@*/
+	    if (!ts->chrootDone && !(rootDir[0] == '/' && rootDir[1] == '\0')) {
+		/*@-unrecog@*/ chroot(rootDir); /*@=unrecog@*/
+	    }
 	    chdir("/");
 	    execv(argv[0], (char *const *)argv);
 	    break;
@@ -395,7 +431,20 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	/*@notreached@*/
     }
 
-    (void)waitpid(child, &status, 0);
+    if (waitpid(child, &status, 0) < 0) {
+	rpmError(RPMERR_SCRIPT,
+		 _("retrieval of return code of %s scriptlet from %s-%s-%s failed, waitpid returned %s\n"),
+		 sln, n, v, r, strerror (errno));
+	/* XXX what to do here? */
+	rc = 0;
+    } else {
+	if (!WIFEXITED(status) || WEXITSTATUS(status)) {
+	    rpmError(RPMERR_SCRIPT,
+		     _("execution of %s scriptlet from %s-%s-%s failed, exit status %d\n"),
+		     sln, n, v, r, WEXITSTATUS(status));
+	    rc = 1;
+	}
+    }
 
     if (freePrefixes) free(prefixes);
 
@@ -403,26 +452,14 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
     
     if (script) {
 	if (!rpmIsDebug()) unlink(fn);
-	xfree(fn);
+	free((void *)fn);
     }
 
-    if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	const char *n, *v, *r;
-	/* XXX FIXME: need to identify what script failed as well. */
-	headerNVR(h, &n, &v, &r);
-	rpmError(RPMERR_SCRIPT,
-		_("execution of %s-%s-%s script failed, exit status %d"),
-		n, v, r, WEXITSTATUS(status));
-
-	return 1;
-    }
-
-    return 0;
+    return rc;
 }
 
-/** */
-int runInstScript(const char * root, Header h, int scriptTag, int progTag,
-	          int arg, int norunScripts, FD_t err)
+int runInstScript(const rpmTransactionSet ts, Header h,
+		int scriptTag, int progTag, int arg, int norunScripts)
 {
     void ** programArgv;
     int programArgc;
@@ -446,35 +483,46 @@ int runInstScript(const char * root, Header h, int scriptTag, int progTag,
 	argv = (const char **) programArgv;
     }
 
-    rc = runScript(h, root, programArgc, argv, script, arg, -1, err);
-    if (programArgv && programType == RPM_STRING_ARRAY_TYPE) free(programArgv);
+    rc = runScript(ts, h, tag2sln(scriptTag), programArgc, argv, script,
+		arg, -1);
+    headerFreeData(programArgv, programType);
     return rc;
 }
 
-static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourceH,
-			    Header triggeredH, int arg1correction, int arg2,
-			    char * triggersAlreadyRun, FD_t scriptFd)
+/**
+ * @param ts		transaction set
+ * @param sense
+ * @param sourceH
+ * @param triggeredH
+ * @param arg1correction
+ * @param arg2
+ * @param triggersAlreadyRun
+ * @return
+ */
+static int handleOneTrigger(const rpmTransactionSet ts, int sense,
+			Header sourceH, Header triggeredH,
+			int arg1correction, int arg2,
+			char * triggersAlreadyRun)
 {
     const char ** triggerNames;
     const char ** triggerEVR;
     const char ** triggerScripts;
     const char ** triggerProgs;
-    int_32 * triggerFlags, * triggerIndices;
-    char * triggerPackageName, * sourceName;
+    int_32 * triggerFlags;
+    int_32 * triggerIndices;
+    const char * triggerPackageName;
+    const char * sourceName;
     int numTriggers;
     int rc = 0;
     int i;
-    int index;
-    dbiIndexSet matches;
     int skip;
 
     if (!headerGetEntry(triggeredH, RPMTAG_TRIGGERNAME, NULL, 
 			(void **) &triggerNames, &numTriggers)) {
-	headerFree(triggeredH);
 	return 0;
     }
 
-    headerGetEntry(sourceH, RPMTAG_NAME, NULL, (void **) &sourceName, NULL);
+    headerNVR(sourceH, &sourceName, NULL, NULL);
 
     headerGetEntry(triggeredH, RPMTAG_TRIGGERFLAGS, NULL, 
 		   (void **) &triggerFlags, NULL);
@@ -482,11 +530,11 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
 		   (void **) &triggerEVR, NULL);
 
     for (i = 0; i < numTriggers; i++) {
+
 	if (!(triggerFlags[i] & sense)) continue;
 	if (strcmp(triggerNames[i], sourceName)) continue;
 
-	/* For some reason, the TRIGGERVERSION stuff includes the name
-	   of the package which the trigger is based on. We need to skip
+	/* For some reason, the TRIGGERVERSION stuff includes the name of the package which the trigger is based on. We need to skip
 	   over that here. I suspect that we'll change our minds on this
 	   and remove that, so I'm going to just 'do the right thing'. */
 	skip = strlen(triggerNames[i]);
@@ -507,18 +555,23 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
 	headerGetEntry(triggeredH, RPMTAG_TRIGGERSCRIPTPROG, NULL,
 		       (void **) &triggerProgs, NULL);
 
-	headerGetEntry(triggeredH, RPMTAG_NAME, NULL, 
-		       (void **) &triggerPackageName, NULL);
-	rpmdbFindPackage(db, triggerPackageName, &matches);
-	dbiFreeIndexRecord(matches);
+	headerNVR(triggeredH, &triggerPackageName, NULL, NULL);
 
-	index = triggerIndices[i];
-	if (!triggersAlreadyRun || !triggersAlreadyRun[index]) {
-	    rc = runScript(triggeredH, root, 1, triggerProgs + index,
-			    triggerScripts[index], 
-			    dbiIndexSetCount(matches) + arg1correction, arg2, 
-			    scriptFd);
-	    if (triggersAlreadyRun) triggersAlreadyRun[index] = 1;
+	{   int arg1;
+	    int index;
+
+	    if ((arg1 = rpmdbCountPackages(ts->rpmdb, triggerPackageName)) < 0) {
+		rc = 1;	/* XXX W2DO? same as "execution of script failed" */
+	    } else {
+		arg1 += arg1correction;
+		index = triggerIndices[i];
+		if (!triggersAlreadyRun || !triggersAlreadyRun[index]) {
+		    rc = runScript(ts, triggeredH, "%trigger", 1,
+			    triggerProgs + index, triggerScripts[index], 
+			    arg1, arg2);
+		    if (triggersAlreadyRun) triggersAlreadyRun[index] = 1;
+		}
+	    }
 	}
 
 	free(triggerScripts);
@@ -534,88 +587,71 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
     return rc;
 }
 
-/** */
-int runTriggers(const char * root, rpmdb db, int sense, Header h,
-		int countCorrection, FD_t scriptFd)
+int runTriggers(const rpmTransactionSet ts, int sense, Header h,
+		int countCorrection)
 {
-    char * packageName;
-    dbiIndexSet matches, otherMatches;
-    Header triggeredH;
+    const char * name;
     int numPackage;
-    int rc;
-    int i;
+    int rc = 0;
 
-    headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &packageName, NULL);
+    headerNVR(h, &name, NULL, NULL);
 
-    if ((rc = rpmdbFindByTriggeredBy(db, packageName, &matches)) < 0)
+    numPackage = rpmdbCountPackages(ts->rpmdb, name) + countCorrection;
+    if (numPackage < 0)
 	return 1;
-    else if (rc)
-	return 0;
 
-    rpmdbFindPackage(db, packageName, &otherMatches);
-    numPackage = dbiIndexSetCount(otherMatches) + countCorrection;
-    dbiFreeIndexRecord(otherMatches);
+    {	Header triggeredH;
+	rpmdbMatchIterator mi;
 
-    rc = 0;
-    for (i = 0; i < dbiIndexSetCount(matches); i++) {
-	unsigned int recOffset = dbiIndexRecordOffset(matches, i);
-	if ((triggeredH = rpmdbGetRecord(db, recOffset)) == NULL) 
-	    return 1;
+	mi = rpmdbInitIterator(ts->rpmdb, RPMTAG_TRIGGERNAME, name, 0);
+	while((triggeredH = rpmdbNextIterator(mi)) != NULL) {
+	    rc |= handleOneTrigger(ts, sense, h, triggeredH, 0, numPackage, 
+			       NULL);
+	}
 
-	rc |= handleOneTrigger(root, db, sense, h, triggeredH, 0, numPackage, 
-			       NULL, scriptFd);
-	
-	headerFree(triggeredH);
+	rpmdbFreeIterator(mi);
     }
 
-    dbiFreeIndexRecord(matches);
-
     return rc;
-    
 }
 
-/** */
-int runImmedTriggers(const char * root, rpmdb db, int sense, Header h,
-		     int countCorrection, FD_t scriptFd)
+int runImmedTriggers(const rpmTransactionSet ts, int sense, Header h,
+		     int countCorrection)
 {
-    int rc = 0;
-    dbiIndexSet matches;
-    char ** triggerNames;
+    const char ** triggerNames;
     int numTriggers;
-    int i, j;
     int_32 * triggerIndices;
+    int numTriggerIndices;
     char * triggersRun;
-    Header sourceH;
+    int rc = 0;
 
     if (!headerGetEntry(h, RPMTAG_TRIGGERNAME, NULL, (void **) &triggerNames, 
 			&numTriggers))
 	return 0;
     headerGetEntry(h, RPMTAG_TRIGGERINDEX, NULL, (void **) &triggerIndices, 
-		   &i);
-    triggersRun = alloca(sizeof(*triggersRun) * i);
-    memset(triggersRun, 0, sizeof(*triggersRun) * i);
+		   &numTriggerIndices);
+    triggersRun = alloca(sizeof(*triggersRun) * numTriggerIndices);
+    memset(triggersRun, 0, sizeof(*triggersRun) * numTriggerIndices);
 
-    for (i = 0; i < numTriggers; i++) {
-	if (triggersRun[triggerIndices[i]]) continue;
+    {	Header sourceH = NULL;
+	int i;
+
+	for (i = 0; i < numTriggers; i++) {
+	    rpmdbMatchIterator mi;
+	    const char * name = triggerNames[i];
+
+	    if (triggersRun[triggerIndices[i]]) continue;
 	
-        if ((j = rpmdbFindPackage(db, triggerNames[i], &matches))) {
-	    if (j < 0) rc |= 1;	
-	    continue;
-	} 
+	    mi = rpmdbInitIterator(ts->rpmdb, RPMTAG_NAME, name, 0);
 
-	for (j = 0; j < dbiIndexSetCount(matches); j++) {
-	    unsigned int recOffset = dbiIndexRecordOffset(matches, j);
-	    if ((sourceH = rpmdbGetRecord(db, recOffset)) == NULL) 
-		return 1;
-	    rc |= handleOneTrigger(root, db, sense, sourceH, h, 
-				   countCorrection, dbiIndexSetCount(matches), 
-				   triggersRun, scriptFd);
-	    headerFree(sourceH);
-	    if (triggersRun[triggerIndices[i]]) break;
+	    while((sourceH = rpmdbNextIterator(mi)) != NULL) {
+		rc |= handleOneTrigger(ts, sense, sourceH, h, 
+				   countCorrection, rpmdbGetIteratorCount(mi),
+				   triggersRun);
+	    }
+
+	    rpmdbFreeIterator(mi);
 	}
-
-	dbiFreeIndexRecord(matches);
     }
-
     return rc;
 }
