@@ -1,22 +1,30 @@
+/** \ingroup rpmcli
+ * \file lib/verify.c
+ */
+
 #include "system.h"
 
 #include <rpmlib.h>
-
-#include "md5.h"
-#include "misc.h"
-#include "install.h"
-
-#include "build/rpmbuild.h"
+#include <rpmbuild.h>
 #include <rpmurl.h>
 
+#include "depends.h"
+#include "install.h"
+#include "md5.h"
+#include "misc.h"
+#include "debug.h"
+
 static int _ie = 0x44332211;
-static union _endian { int i; char b[4]; } *_endian = (union _endian *)&_ie;
+static union _vendian { int i; char b[4]; } *_endian = (union _vendian *)&_ie;
 #define	IS_BIG_ENDIAN()		(_endian->b[0] == '\x44')
 #define	IS_LITTLE_ENDIAN()	(_endian->b[0] == '\x11')
 
 #define S_ISDEV(m) (S_ISBLK((m)) || S_ISCHR((m)))
 
-#define	POPT_NOFILES	1000
+#define	POPT_NODEPS	1000
+#define	POPT_NOFILES	1001
+#define	POPT_NOMD5	1002
+#define	POPT_NOSCRIPTS	1003
 
 /* ========== Verify specific popt args */
 static void verifyArgCallback(/*@unused@*/poptContext con,
@@ -24,30 +32,48 @@ static void verifyArgCallback(/*@unused@*/poptContext con,
 	const struct poptOption * opt, /*@unused@*/const char * arg,
 	const void * data)
 {
-    QVA_t *qva = (QVA_t *) data;
+    QVA_t *qva = &rpmQVArgs;
     switch (opt->val) {
-      case POPT_NOFILES: qva->qva_flags |= VERIFY_FILES; break;
+    case POPT_NODEPS: qva->qva_flags |= VERIFY_DEPS; break;
+    case POPT_NOFILES: qva->qva_flags |= VERIFY_FILES; break;
+    case POPT_NOMD5: qva->qva_flags |= VERIFY_MD5; break;
+    case POPT_NOSCRIPTS: qva->qva_flags |= VERIFY_SCRIPT; break;
     }
 }
 
+static int noDeps = 0;
 static int noFiles = 0;
+static int noMd5 = 0;
+static int noScripts = 0;
+
 /** */
 struct poptOption rpmVerifyPoptTable[] = {
-	{ NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
-		verifyArgCallback, 0, NULL, NULL },
- 	{ "nofiles", '\0', 0, &noFiles, POPT_NOFILES,
-		N_("don't verify files in package"), NULL},
-	{ 0, 0, 0, 0, 0,	NULL, NULL }
+ { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
+	verifyArgCallback, 0, NULL, NULL },
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
+	NULL, NULL },
+ { "nodeps", '\0', 0, &noDeps, POPT_NODEPS,
+	N_("do not verify package dependencies"),
+	NULL },
+ { "nofiles", '\0', 0, &noFiles, POPT_NOFILES,
+	N_("don't verify files in package"),
+	NULL},
+ { "nomd5", '\0', 0, &noMd5, POPT_NOMD5,
+	N_("do not verify file md5 checksums"),
+	NULL },
+ { "noscripts", '\0', 0, &noScripts, POPT_NOSCRIPTS,
+        N_("do not execute %verifyscript (if any)"),
+        NULL },
+    POPT_TABLEEND
 };
 
 /* ======================================================================== */
-/* XXX static */
-/** */
-int rpmVerifyFile(const char * prefix, Header h, int filenum, int * result, 
-		  int omitMask)
+int rpmVerifyFile(const char * prefix, Header h, int filenum,
+		int * result, int omitMask)
 {
     char ** baseNames, ** md5List, ** linktoList, ** dirNames;
-    int_32 * verifyFlags, flags;
+    int_32 * verifyFlags;
+    rpmVerifyAttrs flags;
     int_32 * sizeList, * mtimeList, * dirIndexes;
     unsigned short * modeList, * rdevList;
     char * fileStatesList;
@@ -213,7 +239,7 @@ int rpmVerifyFile(const char * prefix, Header h, int filenum, int * result,
 		*result |= RPMVERIFY_GROUP;
 	} else {
 	    rpmError(RPMERR_INTERNAL, _("package lacks both user name and id "
-		  "lists (this should never happen)"));
+		  "lists (this should never happen)\n"));
 	    *result |= RPMVERIFY_GROUP;
 	}
     }
@@ -231,7 +257,7 @@ int rpmVerifyFile(const char * prefix, Header h, int filenum, int * result,
 		*result |= RPMVERIFY_GROUP;
 	} else {
 	    rpmError(RPMERR_INTERNAL, _("package lacks both group name and id "
-		     "lists (this should never happen)"));
+		     "lists (this should never happen)\n"));
 	    *result |= RPMVERIFY_GROUP;
 	}
     }
@@ -239,43 +265,69 @@ int rpmVerifyFile(const char * prefix, Header h, int filenum, int * result,
     return 0;
 }
 
-/* XXX static */
-/** */
-int rpmVerifyScript(const char * root, Header h, FD_t err)
+/**
+ * Return exit code from running verify script in header.
+ * @todo gnorpm/kpackage prevents static, should be using VERIFY_SCRIPT flag.
+ * @param rootDir       path to top of install tree
+ * @param h             header
+ * @param scriptFd      file handle to use for stderr (or NULL)
+ * @return              0 on success
+ */
+int rpmVerifyScript(const char * rootDir, Header h, FD_t scriptFd)
 {
-    return runInstScript(root, h, RPMTAG_VERIFYSCRIPT, RPMTAG_VERIFYSCRIPTPROG,
-		     0, 0, err);
+    rpmdb rpmdb = NULL;
+    rpmTransactionSet ts = rpmtransCreateSet(rpmdb, rootDir);
+    int rc;
+
+    if (scriptFd)
+	ts->scriptFd = fdLink(scriptFd, "rpmVerifyScript");
+    rc = runInstScript(ts, h, RPMTAG_VERIFYSCRIPT, RPMTAG_VERIFYSCRIPTPROG,
+		     0, 0);
+    rpmtransFree(ts);
+    return rc;
 }
 
 /* ======================================================================== */
 static int verifyHeader(QVA_t *qva, Header h)
 {
-    const char ** fileNames;
+    char buf[BUFSIZ];
+    char * t, * te;
+
+    const char ** fileNames = NULL;
     int count;
-    int verifyResult;
-    int i, ec, rc;
-    int_32 * fileFlagsList;
-    int omitMask = 0;
+    int_32 * fileFlagsList = NULL;
+    rpmVerifyAttrs verifyResult = 0;
+    rpmVerifyAttrs omitMask = !(qva->qva_flags & VERIFY_MD5)
+			? RPMVERIFY_MD5 : RPMVERIFY_NONE;
+    int ec = 0;
+    int i;
 
-    ec = 0;
-    if (!(qva->qva_flags & VERIFY_MD5)) omitMask = RPMVERIFY_MD5;
+    te = t = buf;
+    *te = '\0';
 
-    if (headerGetEntry(h, RPMTAG_FILEFLAGS, NULL, (void **) &fileFlagsList, NULL) &&
-	headerIsEntry(h, RPMTAG_BASENAMES)) {
-	rpmBuildFileList(h, &fileNames, &count);
+    if (!headerGetEntry(h, RPMTAG_FILEFLAGS, NULL, (void **) &fileFlagsList, NULL))
+	goto exit;
 
-	for (i = 0; i < count; i++) {
-	    if ((rc = rpmVerifyFile(qva->qva_prefix, h, i, &verifyResult, omitMask)) != 0) {
-		fprintf(stdout, _("missing    %s\n"), fileNames[i]);
-	    } else {
-		const char * size, * md5, * link, * mtime, * mode;
-		const char * group, * user, * rdev;
-		/*@observer@*/ static const char *const aok = ".";
-		/*@observer@*/ static const char *const unknown = "?";
+    if (!headerIsEntry(h, RPMTAG_BASENAMES))
+	goto exit;
 
-		if (!verifyResult) continue;
-	    
-		rc = 1;
+    rpmBuildFileList(h, &fileNames, &count);
+
+    for (i = 0; i < count; i++) {
+	int rc;
+
+	rc = rpmVerifyFile(qva->qva_prefix, h, i, &verifyResult, omitMask);
+	if (rc) {
+	    sprintf(te, _("missing    %s"), fileNames[i]);
+	    te += strlen(te);
+	    ec = rc;
+	} else if (verifyResult) {
+	    const char * size, * md5, * link, * mtime, * mode;
+	    const char * group, * user, * rdev;
+	    /*@observer@*/ static const char *const aok = ".";
+	    /*@observer@*/ static const char *const unknown = "?";
+
+	    ec = 1;
 
 #define	_verify(_RPMVERIFY_F, _C)	\
 	((verifyResult & _RPMVERIFY_F) ? _C : aok)
@@ -286,73 +338,104 @@ static int verifyHeader(QVA_t *qva, Header h)
 	((verifyResult & RPMVERIFY_READFAIL) ? unknown : \
 	 (verifyResult & _RPMVERIFY_F) ? _C : aok)
 	
-		md5 = _verifyfile(RPMVERIFY_MD5, "5");
-		size = _verify(RPMVERIFY_FILESIZE, "S");
-		link = _verifylink(RPMVERIFY_LINKTO, "L");
-		mtime = _verify(RPMVERIFY_MTIME, "T");
-		rdev = _verify(RPMVERIFY_RDEV, "D");
-		user = _verify(RPMVERIFY_USER, "U");
-		group = _verify(RPMVERIFY_GROUP, "G");
-		mode = _verify(RPMVERIFY_MODE, "M");
+	    md5 = _verifyfile(RPMVERIFY_MD5, "5");
+	    size = _verify(RPMVERIFY_FILESIZE, "S");
+	    link = _verifylink(RPMVERIFY_LINKTO, "L");
+	    mtime = _verify(RPMVERIFY_MTIME, "T");
+	    rdev = _verify(RPMVERIFY_RDEV, "D");
+	    user = _verify(RPMVERIFY_USER, "U");
+	    group = _verify(RPMVERIFY_GROUP, "G");
+	    mode = _verify(RPMVERIFY_MODE, "M");
 
 #undef _verify
 #undef _verifylink
 #undef _verifyfile
 
-		fprintf(stdout, "%s%s%s%s%s%s%s%s %c %s\n",
+	    sprintf(te, "%s%s%s%s%s%s%s%s %c %s",
 		       size, mode, md5, rdev, link, user, group, mtime, 
 		       fileFlagsList[i] & RPMFILE_CONFIG ? 'c' : ' ', 
 		       fileNames[i]);
-	    }
-	    if (rc)
-		ec = rc;
+	    te += strlen(te);
+	}
+
+	if (te > t) {
+	    *te++ = '\n';
+	    *te = '\0';
+	    rpmMessage(RPMMESS_NORMAL, "%s", t);
+	    te = t = buf;
+	    *t = '\0';
 	}
-	
-	free(fileNames);
     }
+	
+exit:
+    if (fileNames) free(fileNames);
     return ec;
 }
 
-static int verifyDependencies(/*@only@*/rpmdb db, Header h) {
+static int verifyDependencies(rpmdb rpmdb, Header h)
+{
     rpmTransactionSet rpmdep;
     struct rpmDependencyConflict * conflicts;
     int numConflicts;
-    const char * name, * version, * release;
+    int rc = 0;
     int i;
 
-    rpmdep = rpmtransCreateSet(db, NULL);
+    rpmdep = rpmtransCreateSet(rpmdb, NULL);
     rpmtransAddPackage(rpmdep, h, NULL, NULL, 0, NULL);
 
     rpmdepCheck(rpmdep, &conflicts, &numConflicts);
     rpmtransFree(rpmdep);
 
     if (numConflicts) {
+	const char * name, * version, * release;
+	char * t, * te;
+	int nb = 512;
 	headerNVR(h, &name, &version, &release);
-	fprintf(stdout, _("Unsatisfied dependencies for %s-%s-%s: "),
-		name, version, release);
+
 	for (i = 0; i < numConflicts; i++) {
-	    if (i) fprintf(stdout, ", ");
-	    fprintf(stdout, "%s", conflicts[i].needsName);
+	    nb += strlen(conflicts[i].needsName) + sizeof(", ") - 1;
+	    if (conflicts[i].needsFlags)
+		nb += strlen(conflicts[i].needsVersion) + 5;
+	}
+	te = t = alloca(nb);
+	*te = '\0';
+	sprintf(te, _("Unsatisfied dependencies for %s-%s-%s: "),
+		name, version, release);
+	te += strlen(te);
+	for (i = 0; i < numConflicts; i++) {
+	    if (i) te = stpcpy(te, ", ");
+	    te = stpcpy(te, conflicts[i].needsName);
 	    if (conflicts[i].needsFlags) {
-		printDepFlags(stdout, conflicts[i].needsVersion, 
-			      conflicts[i].needsFlags);
+		int flags = conflicts[i].needsFlags;
+		*te++ = ' ';
+		if (flags & RPMSENSE_LESS)	*te++ = '<';
+		if (flags & RPMSENSE_GREATER)	*te++ = '>';
+		if (flags & RPMSENSE_EQUAL)	*te++ = '=';
+		*te++ = ' ';
+		te = stpcpy(te, conflicts[i].needsVersion);
 	    }
 	}
-	fprintf(stdout, "\n");
 	rpmdepFreeConflicts(conflicts, numConflicts);
-	return 1;
+	if (te > t) {
+	    *te++ = '\n';
+	    *te = '\0';
+	    rpmMessage(RPMMESS_NORMAL, "%s", t);
+	    te = t;
+	    *t = '\0';
+	}
+	rc = 1;
     }
-    return 0;
+    return rc;
 }
 
-/** */
-int showVerifyPackage(QVA_t *qva, rpmdb db, Header h)
+int showVerifyPackage(QVA_t *qva, rpmdb rpmdb, Header h)
 {
-    int ec, rc;
     FD_t fdo;
-    ec = 0;
+    int ec = 0;
+    int rc;
+
     if ((qva->qva_flags & VERIFY_DEPS) &&
-	(rc = verifyDependencies(db, h)) != 0)
+	(rc = verifyDependencies(rpmdb, h)) != 0)
 	    ec = rc;
     if ((qva->qva_flags & VERIFY_FILES) &&
 	(rc = verifyHeader(qva, h)) != 0)
@@ -365,10 +448,9 @@ int showVerifyPackage(QVA_t *qva, rpmdb db, Header h)
     return ec;
 }
 
-/** */
-int rpmVerify(QVA_t *qva, enum rpmQVSources source, const char *arg)
+int rpmVerify(QVA_t *qva, rpmQVSources source, const char *arg)
 {
-    rpmdb db = NULL;
+    rpmdb rpmdb = NULL;
     int rc;
 
     switch (source) {
@@ -377,17 +459,15 @@ int rpmVerify(QVA_t *qva, enum rpmQVSources source, const char *arg)
 	    break;
 	/*@fallthrough@*/
     default:
-	if (rpmdbOpen(qva->qva_prefix, &db, O_RDONLY, 0644)) {
-	    fprintf(stderr, _("rpmVerify: rpmdbOpen() failed\n"));
+	if (rpmdbOpen(qva->qva_prefix, &rpmdb, O_RDONLY, 0644))
 	    return 1;
-	}
 	break;
     }
 
-    rc = rpmQueryVerify(qva, source, arg, db, showVerifyPackage);
+    rc = rpmQueryVerify(qva, source, arg, rpmdb, showVerifyPackage);
 
-    if (db)
-	rpmdbClose(db);
+    if (rpmdb)
+	rpmdbClose(rpmdb);
 
     return rc;
 }
