@@ -39,7 +39,7 @@ int pkgCompare(void * first, void * second) {
     if (!(*a)->name) return 1;
     if (!(*b)->name) return -1;
 
-    return strcasecmp((*a)->name, (*b)->name);
+    return xstrcasecmp((*a)->name, (*b)->name);
 }
 
 
@@ -96,8 +96,6 @@ static void compareFileList(int availFileCount, char **availBaseNames,
 
 static void addLostFiles(rpmdb db, struct pkgSet *psp, struct hash_table *ht)
 {
-    int num;
-    Header h;
     char *name;
     struct packageInfo **pack;
     struct packageInfo key;
@@ -106,16 +104,16 @@ static void addLostFiles(rpmdb db, struct pkgSet *psp, struct hash_table *ht)
     char **installedDirs;
     int_32 * installedDirIndexes;
     int installedFileCount;
+    Header h = NULL;
+    rpmdbMatchIterator mi;
+
+    mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, NULL, 0);
+    while ((h = rpmdbNextIterator(mi)) != NULL) {
 
-    num = rpmdbFirstRecNum(db);
-    while (num) {
-	h = rpmdbGetRecord(db, num);
 	headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
 	if (name && !strcmp(name, "metroess")) {
 	    /* metro was removed from 5.1, but leave it if it's already
 	       installed */
-	    headerFree(h);
-	    num = rpmdbNextRecNum(db, num);
 	    continue;
 	}
 	key.name = name;
@@ -124,11 +122,11 @@ static void addLostFiles(rpmdb db, struct pkgSet *psp, struct hash_table *ht)
 		       sizeof(*psp->packages), (void *)pkgCompare);
 	if (!pack) {
 	    if (headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
-			  (void **) &installedFiles, &installedFileCount)) {
+			  (const void **) &installedFiles, &installedFileCount)) {
 		headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
-			  (void **) &installedDirIndexes, NULL);
+			  (const void **) &installedDirIndexes, NULL);
 		headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
-			  (void **) &installedDirs, NULL);
+			  (const void **) &installedDirs, NULL);
 
 		compareFileList(0, NULL, NULL, NULL, installedFileCount,
 				installedFiles, installedDirs,
@@ -138,16 +136,14 @@ static void addLostFiles(rpmdb db, struct pkgSet *psp, struct hash_table *ht)
 		free(installedDirs);
 	    }
 	}
-	
-	headerFree(h);
-	num = rpmdbNextRecNum(db, num);
     }
+
+    rpmdbFreeIterator(mi);
 }
 
 static int findPackagesWithObsoletes(rpmdb db, struct pkgSet *psp)
 {
-    dbiIndexSet matches;
-    int rc, count, obsoletesCount;
+    int count, obsoletesCount;
     struct packageInfo **pip;
     char **obsoletes;
 
@@ -160,17 +156,11 @@ static int findPackagesWithObsoletes(rpmdb db, struct pkgSet *psp)
 	}
 
 	if (headerGetEntryMinMemory((*pip)->h, RPMTAG_OBSOLETENAME, NULL,
-		       (void **) &obsoletes, &obsoletesCount)) {
+		       (const void **) &obsoletes, &obsoletesCount)) {
 	    while (obsoletesCount--) {
-		rc = rpmdbFindPackage(db, obsoletes[obsoletesCount], &matches);
-		if (!rc) {
-		    if (matches.count) {
-			(*pip)->selected = 1;
-			dbiFreeIndexRecord(matches);
-			break;
-		    }
-
-		    dbiFreeIndexRecord(matches);
+		if (rpmdbCountPackages(db, obsoletes[obsoletesCount]) > 0) {
+		    (*pip)->selected = 1;
+		    break;
 		}
 	    }
 
@@ -193,8 +183,7 @@ static int findUpgradePackages(rpmdb db, struct pkgSet *psp,
     int skipThis;
     Header h, installedHeader;
     char *name;
-    dbiIndexSet matches;
-    int rc, i, count;
+    int count;
     char **installedFiles, **availFiles;
     char **installedDirs, ** availDirs;
     int_32 * installedDirIndexes, * availDirIndexes;
@@ -214,31 +203,26 @@ static int findUpgradePackages(rpmdb db, struct pkgSet *psp,
 	}
 	
 	DEBUG (("Avail: %s\n", name));
-	rc = rpmdbFindPackage(db, name, &matches);
 
-	if (rc == 0) {
-	    skipThis = 0;
-	    rpmErrorSetCallback(errorFunction);
-	    for (i = 0; i < matches.count; i++) {
-		installedHeader =
-		    rpmdbGetRecord(db, matches.recs[i].recOffset);
-		if (rpmVersionCompare(installedHeader, h) >= 0) {
-		    /* already have a newer version installed */
-		    DEBUG (("Already have newer version\n"))
-		    skipThis = 1;
-		    headerFree(installedHeader);
-		    break;
-		}
-		headerFree(installedHeader);
+    {	rpmdbMatchIterator mi;
+
+	mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
+	skipThis = (mi ? 0 : 1);
+	rpmErrorSetCallback(errorFunction);
+	while((installedHeader = rpmdbNextIterator(mi)) != NULL) {
+	    if (rpmVersionCompare(installedHeader, h) >= 0) {
+		/* already have a newer version installed */
+		DEBUG (("Already have newer version\n"))
+		skipThis = 1;
+		break;
 	    }
-	    rpmErrorSetCallback(NULL);
-	    if (! skipThis) {
-		DEBUG (("No newer version installed\n"))
-	    }
-	} else {
-	    skipThis = 1;
-	    DEBUG (("Not installed\n"))
 	}
+	rpmdbFreeIterator(mi);
+	rpmErrorSetCallback(NULL);
+	if (! skipThis) {
+	    DEBUG (("No newer version installed\n"))
+	}
+    }
 	
 	if (skipThis) {
 	    DEBUG (("DO NOT INSTALL\n"))
@@ -247,27 +231,26 @@ static int findUpgradePackages(rpmdb db, struct pkgSet *psp,
 	    (*pip)->selected = 1;
 
 	    if (!headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
-			  (void **) &availFiles, &availFileCount)) {
+			  (const void **) &availFiles, &availFileCount)) {
 		availFiles = NULL;
 		availFileCount = 0;
 	    } else {
 		headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
-			    (void **) &availDirs, NULL);
+			    (const void **) &availDirs, NULL);
 		headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
-			    (void **) &availDirIndexes, NULL);
+			    (const void **) &availDirIndexes, NULL);
 	    }
 
-	    for (i = 0; i < matches.count; i++) {
-		/* Compare the file lists */
-		installedHeader =
-		    rpmdbGetRecord(db, matches.recs[i].recOffset);
+	{   rpmdbMatchIterator mi;
+	    mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
+	    while((installedHeader = rpmdbNextIterator(mi)) != NULL) {
 		if (headerGetEntryMinMemory(installedHeader, RPMTAG_BASENAMES, 
-			      NULL, (void **) &installedFiles,
+			      NULL, (const void **) &installedFiles,
 			      &installedFileCount)) {
 		    headerGetEntryMinMemory(installedHeader, RPMTAG_DIRNAMES, 
-				NULL, (void **) &installedDirs, NULL);
+				NULL, (const void **) &installedDirs, NULL);
 		    headerGetEntryMinMemory(installedHeader, RPMTAG_DIRINDEXES, 
-				NULL, (void **) &installedDirIndexes, NULL);
+				NULL, (const void **) &installedDirIndexes, NULL);
 
 		    compareFileList(availFileCount, availFiles,
 				    availDirs, availDirIndexes,
@@ -278,8 +261,9 @@ static int findUpgradePackages(rpmdb db, struct pkgSet *psp,
 		    free(installedFiles);
 		    free(installedDirs);
 		}
-		headerFree(installedHeader);
 	    }
+	    rpmdbFreeIterator(mi);
+	}
 
 	    if (availFiles) {
 		free(availFiles);
@@ -287,10 +271,6 @@ static int findUpgradePackages(rpmdb db, struct pkgSet *psp,
 	    }
 	}
 
-	if (rc == 0) {
-	    dbiFreeIndexRecord(matches);
-	}
-
 	DEBUG (("\n\n"))
 
 	pip++;
@@ -319,12 +299,12 @@ static int removeMovedFilesAlreadyHandled(struct pkgSet *psp,
 	    headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name, NULL);
 
 	    if (headerGetEntryMinMemory(h, RPMTAG_BASENAMES, NULL,
-			  (void **) &availFiles, &availFileCount)) {
+			  (const void **) &availFiles, &availFileCount)) {
 
 		headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL, 
-			       (void **) &availDirs, NULL);
+			       (const void **) &availDirs, NULL);
 		headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL, 
-			       (void **) &availDirIndexes, NULL);
+			       (const void **) &availDirIndexes, NULL);
 
 		for (i = 0; i < availFileCount; i++) {
 		    if (htInTable(ht, availDirs[availDirIndexes[i]],
@@ -358,6 +338,7 @@ static int findPackagesWithRelocatedFiles(struct pkgSet *psp,
     int_32 * availDirIndexes;
     int availFileCount;
     struct packageInfo **pip;
+    int_16 * availFileModes;
 
     count = psp->numPackages;
     pip = psp->packages;
@@ -370,11 +351,15 @@ static int findPackagesWithRelocatedFiles(struct pkgSet *psp,
 	    if (headerGetEntry(h, RPMTAG_BASENAMES, NULL,
 			 (void **) &availFiles, &availFileCount)) {
 		headerGetEntryMinMemory(h, RPMTAG_DIRNAMES, NULL,
-			    (void **) &availDirs, NULL);
+			    (const void **) &availDirs, NULL);
 		headerGetEntryMinMemory(h, RPMTAG_DIRINDEXES, NULL,
-			    (void **) &availDirIndexes, NULL);
+			    (const void **) &availDirIndexes, NULL);
+		headerGetEntryMinMemory(h, RPMTAG_FILEMODES, NULL,
+			    (const void **) &availFileModes, NULL);
 
 		for (i = 0; i < availFileCount; i++) {
+		    if (S_ISDIR(availFileModes[i])) continue;
+
 		    if (htInTable(ht, availDirs[availDirIndexes[i]], 
 				    availFiles[i])) {
 			htRemoveFromTable(ht, availDirs[availDirIndexes[i]],
@@ -382,7 +367,6 @@ static int findPackagesWithRelocatedFiles(struct pkgSet *psp,
 			DEBUG (("Found file in %s: %s%s\n", name,
 				availDirs[availDirIndexes[i]], availFiles[i]))
 			(*pip)->selected = 1;
-			break;
 		    }
 		}
 		free(availFiles);
@@ -417,11 +401,10 @@ static void printCount(struct pkgSet *psp)
 
 static int unmarkPackagesAlreadyInstalled(rpmdb db, struct pkgSet *psp)
 {
-    dbiIndexSet matches;
     Header h, installedHeader;
     char *name;
     struct packageInfo **pip;
-    int count, rc, i;
+    int count;
 
     count = psp->numPackages;
     pip = psp->packages;
@@ -436,24 +419,21 @@ static int unmarkPackagesAlreadyInstalled(rpmdb db, struct pkgSet *psp)
 		/*logMessage("Failed with bad header");*/
 		return(-1);
 	    }
-	    rc = rpmdbFindPackage(db, name, &matches);
-	    if (rc == 0) {
-		rpmErrorSetCallback(errorFunction);
-		for (i = 0; i < matches.count; i++) {
-		    installedHeader =
-			rpmdbGetRecord(db, matches.recs[i].recOffset);
-		    if (rpmVersionCompare(installedHeader, h) >= 0) {
-			/* already have a newer version installed */
-			DEBUG (("Already have newer version\n"))
-			(*pip)->selected = 0;
-			headerFree(installedHeader);
-			break;
-		    }
-		    headerFree(installedHeader);
+	  { rpmdbMatchIterator mi;
+
+	    mi = rpmdbInitIterator(db, RPMTAG_NAME, name, 0);
+	    rpmErrorSetCallback(errorFunction);
+	    while((installedHeader = rpmdbNextIterator(mi)) != NULL) {
+		if (rpmVersionCompare(installedHeader, h) >= 0) {
+		    /* already have a newer version installed */
+		    DEBUG (("Already have newer version\n"))
+		    (*pip)->selected = 0;
+		    break;
 		}
-		rpmErrorSetCallback(NULL);
-		dbiFreeIndexRecord(matches);
 	    }
+	    rpmdbFreeIterator(mi);
+	    rpmErrorSetCallback(NULL);
+	  }
 	}
 
 	pip++;
@@ -473,12 +453,12 @@ int ugFindUpgradePackages(struct pkgSet *psp, char *installRoot)
 
     /*logDebugMessage(("ugFindUpgradePackages() ..."));*/
 
-    rpmReadConfigFiles(NULL, NULL);
+/*      rpmReadConfigFiles(NULL, NULL); */
 
     rpmSetVerbosity(RPMMESS_FATALERROR);
     old = rpmErrorSetCallback(emptyErrorCallback);
 
-    if (rpmdbOpenForTraversal(installRoot, &db)) {
+    if (rpmdbOpen(installRoot, &db, O_RDONLY, 0644)) {
 	/*logMessage("failed opening %s/var/lib/rpm/packages.rpm",
 		     installRoot);*/
 	return(-1);
