@@ -6,13 +6,11 @@
 #include "build.h"
 #include "install.h"
 #include "signature.h"
+#include "debug.h"
 
-#define GETOPT_REBUILD		1003
-#define GETOPT_RECOMPILE	1004
 #define GETOPT_ADDSIGN		1005
 #define GETOPT_RESIGN		1006
 #define GETOPT_DBPATH		1010
-#define GETOPT_TIMECHECK        1012
 #define GETOPT_REBUILDDB        1013
 #define GETOPT_INSTALL		1014
 #define GETOPT_RELOCATE		1016
@@ -20,6 +18,7 @@
 #define GETOPT_EXCLUDEPATH	1019
 #define	GETOPT_DEFINEMACRO	1020
 #define	GETOPT_EVALMACRO	1021
+#define	GETOPT_RCFILE		1022
 
 enum modes {
     MODE_UNKNOWN	= 0,
@@ -45,7 +44,6 @@ enum modes {
 #define	MODES_K	 (MODE_CHECKSIG | MODES_RESIGN)
 
 #define	MODES_FOR_DBPATH	(MODES_BT | MODES_IE | MODES_QV | MODES_DB)
-#define	MODES_FOR_TIMECHECK	(MODES_BT)
 #define	MODES_FOR_NODEPS	(MODES_BT | MODES_IE | MODE_VERIFY)
 #define	MODES_FOR_TEST		(MODES_BT | MODES_IE)
 #define	MODES_FOR_ROOT		(MODES_BT | MODES_IE | MODES_QV | MODES_DB)
@@ -57,10 +55,6 @@ static int badReloc;
 static int excldocs;
 static int force;
 extern int _ftp_debug;
-static char * ftpPort;
-static char * ftpProxy;
-static char * httpPort;
-static char * httpProxy;
 static int showHash;
 static int help;
 static int ignoreArch;
@@ -81,7 +75,6 @@ static int noUsageMsg;
 static int oldPackage;
 static char * pipeOutput;
 static char * prefix;
-static int queryTags;
 static int quiet;
 static char * rcfile;
 static int replaceFiles;
@@ -94,6 +87,7 @@ static int signIt;
 static int test;
 extern int _url_debug;
 extern int _noDirTokens;
+extern int _useDbiMajor;
 
 static int showVersion;
 extern const char * rpmNAME;
@@ -102,20 +96,15 @@ extern int rpmFLAGS;
 
 extern MacroContext rpmCLIMacroContext;
 
-static struct rpmQVArguments rpmQVArgs;
-static struct rpmBuildArguments rpmBArgs;
+extern struct rpmBuildArguments		rpmBTArgs;
 
 /* the structure describing the options we take and the defaults */
 static struct poptOption optionsTable[] = {
  { "addsign", '\0', 0, 0, GETOPT_ADDSIGN,	NULL, NULL},
-/* all and allmatches both using 'a' is dumb */
- { "all", 'a', 0, 0, 'a',			NULL, NULL},
  { "allfiles", '\0', 0, &allFiles, 0,		NULL, NULL},
  { "allmatches", '\0', 0, &allMatches, 0,	NULL, NULL},
  { "badreloc", '\0', 0, &badReloc, 0,		NULL, NULL},
- { "build", 'b', POPT_ARG_STRING, 0, 'b',	NULL, NULL},
  { "checksig", 'K', 0, 0, 'K',			NULL, NULL},
- { "dbpath", '\0', POPT_ARG_STRING, 0, GETOPT_DBPATH,		NULL, NULL},
  { "define", '\0', POPT_ARG_STRING, 0, GETOPT_DEFINEMACRO,NULL, NULL},
  { "dirtokens", '\0', POPT_ARG_VAL, &_noDirTokens, 0,	NULL, NULL},
  { "erase", 'e', 0, 0, 'e',			NULL, NULL},
@@ -125,12 +114,8 @@ static struct poptOption optionsTable[] = {
  { "force", '\0', 0, &force, 0,			NULL, NULL},
  { "freshen", 'F', 0, 0, 'F',			NULL, NULL},
  { "ftpdebug", '\0', POPT_ARG_VAL, &_ftp_debug, -1,		NULL, NULL},
- { "ftpport", '\0', POPT_ARG_STRING, &ftpPort, 0,	NULL, NULL},
- { "ftpproxy", '\0', POPT_ARG_STRING, &ftpProxy, 0,	NULL, NULL},
  { "hash", 'h', 0, &showHash, 0,		NULL, NULL},
  { "help", '\0', 0, &help, 0,			NULL, NULL},
- { "httpport", '\0', POPT_ARG_STRING, &httpPort, 0,	NULL, NULL},
- { "httpproxy", '\0', POPT_ARG_STRING, &httpProxy, 0,	NULL, NULL},
  {  NULL, 'i', 0, 0, 'i',			NULL, NULL},
  { "ignorearch", '\0', 0, &ignoreArch, 0,	NULL, NULL},
  { "ignoreos", '\0', 0, &ignoreOs, 0,		NULL, NULL},
@@ -140,10 +125,11 @@ static struct poptOption optionsTable[] = {
 /* info and install both using 'i' is dumb */
  { "install", '\0', 0, 0, GETOPT_INSTALL,	NULL, NULL},
  { "justdb", '\0', 0, &justdb, 0,		NULL, NULL},
+ { "macros", '\0', POPT_ARG_STRING, &macrofiles, 0,	NULL, NULL},
  { "nodeps", '\0', 0, &noDeps, 0,		NULL, NULL},
  { "nodirtokens", '\0', POPT_ARG_VAL, &_noDirTokens, 1,	NULL, NULL},
  { "nogpg", '\0', 0, &noGpg, 0,			NULL, NULL},
-#if HAVE_LIBIO_H && defined(_IO_BAD_SEEN)
+#if HAVE_LIBIO_H && defined(_G_IO_IO_FILE_VERSION)
  { "nolibio", '\0', POPT_ARG_VAL, &noLibio, 1,		NULL, NULL},
 #endif
  { "nomd5", '\0', 0, &noMd5, 0,			NULL, NULL},
@@ -155,13 +141,13 @@ static struct poptOption optionsTable[] = {
  { "percent", '\0', 0, &showPercents, 0,	NULL, NULL},
  { "pipe", '\0', POPT_ARG_STRING, &pipeOutput, 0,	NULL, NULL},
  { "prefix", '\0', POPT_ARG_STRING, &prefix, 0,	NULL, NULL},
- { "query", 'q', 0, NULL, 'q',			NULL, NULL},
- { "querytags", '\0', 0, &queryTags, 0,		NULL, NULL},
  { "quiet", '\0', 0, &quiet, 0,			NULL, NULL},
+#ifndef	DYING
  { "rcfile", '\0', POPT_ARG_STRING, &rcfile, 0,	NULL, NULL},
- { "rebuild", '\0', 0, 0, GETOPT_REBUILD,	NULL, NULL},
+#else
+ { "rcfile", '\0', 0, 0, GETOPT_RCFILE,		NULL, NULL},
+#endif
  { "rebuilddb", '\0', 0, 0, GETOPT_REBUILDDB,	NULL, NULL},
- { "recompile", '\0', 0, 0, GETOPT_RECOMPILE,	NULL, NULL},
  { "relocate", '\0', POPT_ARG_STRING, 0, GETOPT_RELOCATE,	NULL, NULL},
  { "replacefiles", '\0', 0, &replaceFiles, 0,	NULL, NULL},
  { "replacepkgs", '\0', 0, &replacePackages, 0,	NULL, NULL},
@@ -170,24 +156,23 @@ static struct poptOption optionsTable[] = {
  { "rpmiodebug", '\0', POPT_ARG_VAL, &_rpmio_debug, -1,		NULL, NULL},
  { "showrc", '\0', 0, &showrc, GETOPT_SHOWRC,	NULL, NULL},
  { "sign", '\0', 0, &signIt, 0,			NULL, NULL},
- { "tarbuild", 't', POPT_ARG_STRING, 0, 't',	NULL, NULL},
  { "test", '\0', 0, &test, 0,			NULL, NULL},
- { "timecheck", '\0', POPT_ARG_STRING, 0, GETOPT_TIMECHECK,	NULL, NULL},
  { "upgrade", 'U', 0, 0, 'U',			NULL, NULL},
  { "urldebug", '\0', POPT_ARG_VAL, &_url_debug, -1,		NULL, NULL},
  { "uninstall", 'u', 0, 0, 'u',			NULL, NULL},
  { "verbose", 'v', 0, 0, 'v',			NULL, NULL},
- { "verify", 'V', 0, 0, 'V',			NULL, NULL},
- {  NULL, 'y', 0, 0, 'V',			NULL, NULL},
  { "version", '\0', 0, &showVersion, 0,		NULL, NULL},
+
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
 		rpmQVSourcePoptTable, 0,	(void *) &rpmQVArgs, NULL },
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
 		rpmQueryPoptTable, 0,		(void *) &rpmQVArgs, NULL },
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
 		rpmVerifyPoptTable, 0,		(void *) &rpmQVArgs, NULL },
- { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
-		rpmBuildPoptTable, 0,		(void *) &rpmBArgs, NULL },
+
+ { NULL, '\0', POPT_ARG_INCLUDE_TABLE,
+		rpmBuildPoptTable, 0,           (void *) &rpmBTArgs, NULL },
+
  { 0, 0, 0, 0, 0,	NULL, NULL }
 };
 
@@ -212,8 +197,8 @@ static void printVersion(void) {
 }
 
 static void printBanner(void) {
-    puts(_("Copyright (C) 1998 - Red Hat Software"));
-    puts(_("This may be freely redistributed under the terms of the GNU GPL"));
+    puts(_("Copyright (C) 1998-2000 - Red Hat, Inc."));
+    puts(_("This program may be freely redistributed under the terms of the GNU GPL"));
 }
 
 static void printUsage(void) {
@@ -229,9 +214,9 @@ static void printUsage(void) {
     puts(_("                        [--excludedocs] [--includedocs] [--noscripts]"));
     puts(_("                        [--rcfile <file>] [--ignorearch] [--dbpath <dir>]"));
     puts(_("                        [--prefix <dir>] [--ignoreos] [--nodeps] [--allfiles]"));
-    puts(_("                        [--ftpproxy <host>] [--ftpport <port>] [--justdb]"));
-    puts(_("                        [--httpproxy <host>] [--httpport <port>] "));
-    puts(_("                        [--noorder] [--relocate oldpath=newpath]"));
+    puts(_("                        [--ftpproxy <host>] [--ftpport <port>]"));
+    puts(_("                        [--httpproxy <host>] [--httpport <port>]"));
+    puts(_("                        [--justdb] [--noorder] [--relocate oldpath=newpath]"));
     puts(_("                        [--badreloc] [--notriggers] [--excludepath <path>]"));
     puts(_("                        [--ignoresize] file1.rpm ... fileN.rpm"));
     puts(_("       rpm {--upgrade -U} [-v] [--hash -h] [--percent] [--force] [--test]"));
@@ -247,9 +232,10 @@ static void printUsage(void) {
     puts(_("       rpm {--query -q} [-afpg] [-i] [-l] [-s] [-d] [-c] [-v] [-R]"));
     puts(_("                        [--scripts] [--root <dir>] [--rcfile <file>]"));
     puts(_("                        [--whatprovides] [--whatrequires] [--requires]"));
-    puts(_("                        [--triggeredby] [--ftpport] [--ftpproxy <host>]"));
-    puts(_("                        [--httpproxy <host>] [--httpport <port>] "));
-    puts(_("                        [--ftpport <port>] [--provides] [--triggers] [--dump]"));
+    puts(_("                        [--triggeredby]"));
+    puts(_("                        [--ftpproxy <host>] [--ftpport <port>]"));
+    puts(_("                        [--httpproxy <host>] [--httpport <port>]"));
+    puts(_("                        [--provides] [--triggers] [--dump]"));
     puts(_("                        [--changelog] [--dbpath <dir>] [targets]"));
     puts(_("       rpm {--verify -V -y} [-afpg] [--root <dir>] [--rcfile <file>]"));
     puts(_("                        [--dbpath <dir>] [--nodeps] [--nofiles] [--noscripts]"));
@@ -259,14 +245,7 @@ static void printUsage(void) {
     puts(_("       rpm {--freshen -F} file1.rpm ... fileN.rpm"));
     puts(_("       rpm {--erase -e} [--root <dir>] [--noscripts] [--rcfile <file>]"));
     puts(_("                        [--dbpath <dir>] [--nodeps] [--allmatches]"));
-    puts(_("                        [--justdb] [--notriggers] rpackage1 ... packageN"));
-    puts(_("       rpm {-b|t}[plciba] [-v] [--short-circuit] [--clean] [--rcfile  <file>]"));
-    puts(_("                        [--sign] [--nobuild] [--timecheck <s>] ]"));
-    puts(_("                        [--target=platform1[,platform2...]]"));
-    puts(_("                        [--rmsource] [--rmspec] specfile"));
-    puts(_("       rpm {--rmsource} [--rcfile <file>] [-v] specfile"));
-    puts(_("       rpm {--rebuild} [--rcfile <file>] [-v] source1.rpm ... sourceN.rpm"));
-    puts(_("       rpm {--recompile} [--rcfile <file>] [-v] source1.rpm ... sourceN.rpm"));
+    puts(_("                        [--justdb] [--notriggers] package1 ... packageN"));
     puts(_("       rpm {--resign} [--rcfile <file>] package1 package2 ... packageN"));
     puts(_("       rpm {--addsign} [--rcfile <file>] package1 package2 ... packageN"));
     puts(_("       rpm {--checksig -K} [--nopgp] [--nogpg] [--nomd5] [--rcfile <file>]"));
@@ -280,7 +259,7 @@ static void printHelpLine(char * prefix, char * help) {
     int lineLength = 79 - indentLength;
     int helpLength = strlen(help);
     char * ch;
-    char format[10];
+    char format[64];
 
     fprintf(stdout, "%s - ", prefix);
 
@@ -311,33 +290,47 @@ static void printHelp(void) {
 		  _("print this message"));
     printHelpLine(  "   --version              ",
 		  _("print the version of rpm being used"));
-    puts(         _("   all modes support the following arguments:"));
-    printHelpLine(_("      --rcfile <file>     "),
+
+    puts("");
+    puts(         _("   All modes support the following arguments:"));
+    printHelpLine(_("    --define '<name> <body>'"),
+		  _("define macro <name> with value <body>"));
+    printHelpLine(_("    --eval '<name>+'      "),
+		  _("print the expansion of macro <name> to stdout"));
+    printHelpLine(_("    --pipe <cmd>          "),
+		  _("send stdout to <cmd>"));
+    printHelpLine(_("    --rcfile <file>       "),
 		  _("use <file> instead of /etc/rpmrc and $HOME/.rpmrc"));
+    printHelpLine(  "    --showrc              ",
+		  _("display final rpmrc and macro configuration"));
     printHelpLine(  "     -v                   ",
 		  _("be a little more verbose"));
     printHelpLine(  "     -vv                  ",
 		  _("be incredibly verbose (for debugging)"));
-    printHelpLine(  "   -q                     ",
-		  _("query mode"));
-    printHelpLine(_("      --root <dir>        "),
-		  _("use <dir> as the top level directory"));
-    printHelpLine(_("      --dbpath <dir>      "),
-		  _("use <dir> as the directory for the database"));
-    printHelpLine(_("      --queryformat <qfmt>"),
-		  _("use <qfmt> as the header format (implies -i)"));
-    puts(         _("   install, upgrade and query (with -p) allow ftp URL's to be used in place"));
+
+    puts("");
+    puts(         _("   Install, upgrade and query (with -p) allow URL's to be used in place"));
     puts(         _("   of file names as well as the following options:"));
     printHelpLine(_("      --ftpproxy <host>   "),
 		  _("hostname or IP of ftp proxy"));
     printHelpLine(_("      --ftpport <port>    "),
 		  _("port number of ftp server (or proxy)"));
-    printHelpLine(_("      --httpproxy <host>   "),
+    printHelpLine(_("      --httpproxy <host>  "),
 		  _("hostname or IP of http proxy"));
-    printHelpLine(_("      --httpport <port>    "),
+    printHelpLine(_("      --httpport <port>   "),
 		  _("port number of http server (or proxy)"));
+
+    puts("");
+    printHelpLine(  "   -q, --query            ",
+		  _("query mode"));
+    printHelpLine(_("      --dbpath <dir>      "),
+		  _("use <dir> as the directory for the database"));
+    printHelpLine(_("      --queryformat <qfmt>"),
+		  _("use <qfmt> as the header format (implies --info)"));
+    printHelpLine(_("      --root <dir>        "),
+		  _("use <dir> as the top level directory"));
     puts(         _("      Package specification options:"));
-    printHelpLine(  "        -a                ",
+    printHelpLine(  "        -a, --all         ",
 		  _("query all packages"));
     printHelpLine(_("        -f <file>+        "),
 		  _("query package owning <file>"));
@@ -350,7 +343,7 @@ static void printHelp(void) {
     printHelpLine(_("        --whatrequires <cap>"),
 		  _("query packages which require <cap> capability"));
     puts(         _("      Information selection options:"));
-    printHelpLine(  "        -i                ",
+    printHelpLine(  "        -i, --info        ",
 		  _("display package information"));
     printHelpLine(  "        --changelog       ",
 		  _("display the package's change log"));
@@ -366,19 +359,15 @@ static void printHelp(void) {
 		  _("show all verifiable information for each file (must be used with -l, -c, or -d)"));
     printHelpLine(  "        --provides        ",
 		  _("list capabilities package provides"));
-    puts(         _("        --requires"));
-    printHelpLine(  "        -R                ",
+    printHelpLine(  "        -R, --requires    ",
 		  _("list package dependencies"));
     printHelpLine(  "        --scripts         ",
 		  _("print the various [un]install scripts"));
-    printHelpLine("          --triggers        ",
+    printHelpLine(  "        --triggers        ",
 		  _("show the trigger scripts contained in the package"));
+
     puts("");
-    puts(           "    -V");
-    puts(           "    -y");
-    printHelpLine(_("    --pipe <cmd>          "),
-		  _("send stdout to <cmd>"));
-    printHelpLine(  "    --verify              ",
+    printHelpLine(  "    -V, -y, --verify      ",
 		  _("verify a package installation using the same same package specification options as -q"));
     printHelpLine(_("      --dbpath <dir>      "),
 		  _("use <dir> as the directory for the database"));
@@ -390,14 +379,9 @@ static void printHelp(void) {
 		  _("do not verify file md5 checksums"));
     printHelpLine(  "      --nofiles           ",
 		  _("do not verify file attributes"));
-    puts("");
-    printHelpLine(  "    --setperms            ",
-		  _("set the file permissions to those in the package database"
-		    " using the same package specification options as -q"));
-    printHelpLine(  "    --setugids            ",
-		  _("set the file owner and group to those in the package "
-		    "database using the same package specification options as "
-		    "-q"));
+    printHelpLine(  "    --querytags           ",
+		  _("list the tags that can be used in a query format"));
+
     puts("");
     puts(         _("    --install <packagefile>"));
     printHelpLine(_("    -i <packagefile>      "),
@@ -406,8 +390,8 @@ static void printHelp(void) {
 		  _("skip files in path <path>"));
     printHelpLine(_("      --relocate <oldpath>=<newpath>"),
 		  _("relocate files from <oldpath> to <newpath>"));
-    printHelpLine(  "      --badreloc",
-		  _("relocate files even though the package doesn't allow it"));
+    printHelpLine(  "      --badreloc          ",
+		  _("relocate files in non-relocateable package"));
     printHelpLine(_("      --prefix <dir>      "),
 		  _("relocate the package to <dir>, if relocatable"));
     printHelpLine(_("      --dbpath <dir>      "),
@@ -416,8 +400,7 @@ static void printHelp(void) {
 		  _("do not install documentation"));
     printHelpLine(  "      --force             ",
 		  _("short hand for --replacepkgs --replacefiles"));
-    puts(           "      -h");
-    printHelpLine(  "      --hash              ",
+    printHelpLine(  "      -h, --hash          ",
 		  _("print hash marks as package installs (good with -v)"));
     printHelpLine(  "      --allfiles          ",
 		  _("install all files, even configurations which might "
@@ -450,6 +433,7 @@ static void printHelp(void) {
 		  _("use <dir> as the top level directory"));
     printHelpLine(  "      --test              ",
 		  _("don't install, but tell if it would work or not"));
+
     puts("");
     puts(         _("    --upgrade <packagefile>"));
     printHelpLine(_("    -U <packagefile>      "),
@@ -497,7 +481,9 @@ static void printHelp(void) {
     printHelpLine(  "      --clean             ",
 		  _("remove build tree when done"));
     printHelpLine(  "      --rmsource          ",
-		  _("remove sources and spec file when done"));
+		  _("remove sources when done"));
+    printHelpLine(  "      --rmspec            ",
+		  _("remove spec file when done"));
     printHelpLine(  "      --sign              ",
 		  _("generate PGP/GPG signature"));
     printHelpLine(_("      --buildroot <dir>   "),
@@ -511,16 +497,16 @@ static void printHelp(void) {
     puts("");
     printHelpLine(_("    --rebuild <src_pkg>   "),
 		  _("install source package, build binary package and remove spec file, sources, patches, and icons."));
-    printHelpLine(_("    --rmsource <spec>     "),
-		  _("remove sources and spec file"));
     printHelpLine(_("    --recompile <src_pkg> "),
 		  _("like --rebuild, but don't build any package"));
+
+    puts("");
     printHelpLine(_("    --resign <pkg>+       "),
 		  _("sign a package (discard current signature)"));
     printHelpLine(_("    --addsign <pkg>+      "),
 		  _("add a signature to a package"));
-    puts(           "    -K");
-    printHelpLine(_("    --checksig <pkg>+     "),
+    puts(         _("    --checksig <pkg>+"));
+    printHelpLine(_("    -K <pkg>+             "),
 		  _("verify package signature"));
     printHelpLine(  "      --nopgp             ",
 		  _("skip any PGP signatures"));
@@ -528,8 +514,8 @@ static void printHelp(void) {
 		  _("skip any GPG signatures"));
     printHelpLine(  "      --nomd5             ",
 		  _("skip any MD5 signatures"));
-    printHelpLine(  "    --querytags           ",
-		  _("list the tags that can be used in a query format"));
+
+    puts("");
     printHelpLine(  "    --initdb              ",
 		  _("make sure a valid database exists"));
     printHelpLine(  "    --rebuilddb           ",
@@ -538,31 +524,32 @@ static void printHelp(void) {
 		  _("use <dir> as the directory for the database"));
     printHelpLine(  "      --root <dir>        ",
 		  _("use <dir> as the top level directory"));
+
+    puts("");
+    printHelpLine(  "    --setperms            ",
+		  _("set the file permissions to those in the package database"
+		    " using the same package specification options as -q"));
+    printHelpLine(  "    --setugids            ",
+		  _("set the file owner and group to those in the package "
+		    "database using the same package specification options as "
+		    "-q"));
 }
 
 int main(int argc, const char ** argv)
 {
     enum modes bigMode = MODE_UNKNOWN;
     QVA_t *qva = &rpmQVArgs;
-    struct rpmBuildArguments *ba = &rpmBArgs;
-    enum rpmQVSources QVSource = RPMQV_PACKAGE;
     int arg;
     int installFlags = 0, uninstallFlags = 0, interfaceFlags = 0;
-    int gotDbpath = 0, verifyFlags;
+    int verifyFlags;
     int checksigFlags = 0;
-    unsigned long int timeCheck = 0L;
-    int addSign = NEW_SIGNATURE;
-    const char * specFile;
-    char * tce;
+    rpmResignFlags addSign = RESIGN_NEW_SIGNATURE;
     char * passPhrase = "";
-    char * cookie = NULL;
     const char * optArg;
     pid_t pipeChild = 0;
     const char * pkg;
     char * errString = NULL;
     poptContext optCon;
-    const char * infoCommand[] = { "--info", NULL };
-    const char * installCommand[] = { "--install", NULL };
     int ec = 0;
     int status;
     int p[2];
@@ -585,10 +572,6 @@ int main(int argc, const char ** argv)
     excldocs = 0;
     force = 0;
     _ftp_debug = 0;
-    ftpProxy = NULL;
-    ftpPort = NULL;
-    httpProxy = NULL;
-    httpPort = NULL;
     showHash = 0;
     help = 0;
     ignoreArch = 0;
@@ -599,7 +582,7 @@ int main(int argc, const char ** argv)
     justdb = 0;
     noDeps = 0;
     noGpg = 0;
-#if HAVE_LIBIO_H && defined(_IO_BAD_SEEN)
+#if HAVE_LIBIO_H && defined(_G_IO_IO_FILE_VERSION)
     noLibio = 0;
 #else
     noLibio = 1;
@@ -614,7 +597,6 @@ int main(int argc, const char ** argv)
     showPercents = 0;
     pipeOutput = NULL;
     prefix = NULL;
-    queryTags = 0;
     quiet = 0;
     _rpmio_debug = 0;
     replaceFiles = 0;
@@ -674,12 +656,11 @@ int main(int argc, const char ** argv)
 
     poptResetContext(optCon);
 
-    if (qva->qva_queryFormat) xfree(qva->qva_queryFormat);
+    if (qva->qva_queryFormat) free((void *)qva->qva_queryFormat);
     memset(qva, 0, sizeof(*qva));
-    if (ba->buildRootOverride) xfree(ba->buildRootOverride);
-    if (ba->targets) free(ba->targets);
-    memset(ba, 0, sizeof(*ba));
-    ba->buildChar = ' ';
+    qva->qva_source = RPMQV_PACKAGE;
+    qva->qva_mode = ' ';
+    qva->qva_char = ' ';
 
     while ((arg = poptGetNextOpt(optCon)) > 0) {
 	optArg = poptGetOptArg(optCon);
@@ -691,19 +672,6 @@ int main(int argc, const char ** argv)
 	    bigMode = MODE_CHECKSIG;
 	    break;
 	    
-	  case 'q':
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_QUERY)
-		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_QUERY;
-	    break;
-
-	  case 'V':
-	  case 'y':
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_VERIFY)
-		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_VERIFY;
-	    break;
-
 	  case 'u':
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_UNINSTALL)
 		argerror(_("only one major mode may be specified"));
@@ -719,52 +687,20 @@ int main(int argc, const char ** argv)
 	    bigMode = MODE_UNINSTALL;
 	    break;
 	
-	  case 'b':
-	  case 't':
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_BUILD)
-		argerror(_("only one major mode may be specified"));
-
-	    if (arg == 'b') {
-		bigMode = MODE_BUILD;
-		errString = _("--build (-b) requires one of a,b,i,c,p,l as "
-				"its sole argument");
-	    } else {
-		bigMode = MODE_TARBUILD;
-		errString = _("--tarbuild (-t) requires one of a,b,i,c,p,l as "
-			      "its sole argument");
-	    }
-
-	    if (strlen(optArg) > 1) 
-		argerror(errString);
-
-	    ba->buildChar = optArg[0];
-	    switch (ba->buildChar) {
-	      case 'a':
-	      case 'b':
-	      case 'i':
-	      case 'c':
-	      case 'p':
-	      case 'l':
-	      case 's':
-		break;
-	      default:
-		argerror(errString);
-		break;
-	    }
-
-	    break;
-	
 	  case 'v':
 	    rpmIncreaseVerbosity();
 	    break;
 
 	  case 'i':
-	    if (bigMode == MODE_QUERY)
+	    if (bigMode == MODE_QUERY) {
+		const char * infoCommand[] = { "--info", NULL };
 		poptStuffArgs(optCon, infoCommand);
-	    else if (bigMode == MODE_INSTALL)
+	    } else if (bigMode == MODE_INSTALL)
 		/*@-ifempty@*/ ;
-	    else if (bigMode == MODE_UNKNOWN)
+	    else if (bigMode == MODE_UNKNOWN) {
+		const char * installCommand[] = { "--install", NULL };
 		poptStuffArgs(optCon, installCommand);
+	    }
 	    break;
 
 	  case GETOPT_INSTALL:
@@ -788,50 +724,11 @@ int main(int argc, const char ** argv)
 	    freshen = 1;
 	    break;
 
-	  case 'p':
-	    if (QVSource != RPMQV_PACKAGE && QVSource != RPMQV_RPM)
-		argerror(_("one type of query/verify may be performed at a " "time"));
-	    QVSource = RPMQV_RPM;
-	    break;
-
-	  case 'g':
-	    if (QVSource != RPMQV_PACKAGE && QVSource != RPMQV_GROUP)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_GROUP;
-	    break;
-
-	  case 'f':
-	    if (QVSource != RPMQV_PACKAGE && QVSource != RPMQV_PATH)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_PATH;
-	    break;
-
-	  case 'a':
-	    if (QVSource != RPMQV_PACKAGE && QVSource != RPMQV_ALL)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_ALL;
-	    break;
-
-	  case GETOPT_REBUILD:
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_REBUILD)
-		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_REBUILD;
-	    break;
-
-	  case GETOPT_RECOMPILE:
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RECOMPILE)
-		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_RECOMPILE;
-	    break;
-
 	  case GETOPT_RESIGN:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RESIGN)
 		argerror(_("only one major mode may be specified"));
 	    bigMode = MODE_RESIGN;
-	    addSign = NEW_SIGNATURE;
+	    addSign = RESIGN_NEW_SIGNATURE;
 	    signIt = 1;
 	    break;
 
@@ -839,24 +736,10 @@ int main(int argc, const char ** argv)
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RESIGN)
 		argerror(_("only one major mode may be specified"));
 	    bigMode = MODE_RESIGN;
-	    addSign = ADD_SIGNATURE;
+	    addSign = RESIGN_ADD_SIGNATURE;
 	    signIt = 1;
 	    break;
 
-	  case GETOPT_DBPATH:
-	    switch (urlIsURL(optArg)) {
-	    case URL_IS_UNKNOWN:
-		if (optArg[0] != '/')
-		    argerror(_("arguments to --dbpath must begin with a /"));
-		break;
-	    default:
-		break;
-	    }
-	    addMacro(NULL, "_dbpath", NULL, optArg, RMIL_CMDLINE);
-	    addMacro(&rpmCLIMacroContext, "_dbpath", NULL, optArg, RMIL_CMDLINE);
-	    gotDbpath = 1;
-	    break;
-
 	  case GETOPT_DEFINEMACRO:
 	    rpmDefineMacro(NULL, optArg, RMIL_CMDLINE);
 	    rpmDefineMacro(&rpmCLIMacroContext, optArg, RMIL_CMDLINE);
@@ -866,21 +749,10 @@ int main(int argc, const char ** argv)
 	  case GETOPT_EVALMACRO:
 	  { const char *val = rpmExpand(optArg, NULL);
 	    fprintf(stdout, "%s\n", val);
-	    xfree(val);
+	    free((void *)val);
 	    noUsageMsg = 1;
 	  } break;
 
-	  case GETOPT_TIMECHECK:
-	    tce = NULL;
-	    timeCheck = strtoul(optArg, &tce, 10);
-	    if ((*tce) || (tce == optArg) || (timeCheck == ULONG_MAX)) {
-		argerror("Argument to --timecheck must be integer");
-	    }
-	    addMacro(NULL, "_timecheck", NULL, optArg, RMIL_CMDLINE);
-	    addMacro(&rpmCLIMacroContext, "_timecheck", NULL, optArg, RMIL_CMDLINE);
-	    timeCheck = 1;
-	    break;
-
 	  case GETOPT_REBUILDDB:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_REBUILDDB)
 		argerror(_("only one major mode may be specified"));
@@ -911,6 +783,12 @@ int main(int argc, const char ** argv)
 	    relocations[numRelocations++].newPath = NULL;
 	    break;
 
+	  case GETOPT_RCFILE:
+	    fprintf(stderr, _("The --rcfile option has been eliminated.\n"));
+	    fprintf(stderr, _("Use --macros with a colon separated list of macro files to read.\n"));
+	    exit(EXIT_FAILURE);
+	    /*@notreached@*/ break;
+
 	  default:
 	    fprintf(stderr, _("Internal error in argument processing (%d) :-(\n"), arg);
 	    exit(EXIT_FAILURE);
@@ -930,12 +808,14 @@ int main(int argc, const char ** argv)
 	exit(EXIT_FAILURE);
     }
 
-    if ((ba->buildAmount & RPMBUILD_RMSOURCE) && bigMode == MODE_UNKNOWN)
-	bigMode = MODE_BUILD;
+    if (bigMode == MODE_UNKNOWN && qva->qva_mode != ' ') {
+	switch (qva->qva_mode) {
+	case 'q':	bigMode = MODE_QUERY;		break;
+	case 'V':	bigMode = MODE_VERIFY;		break;
+	case 'Q':	bigMode = MODE_QUERYTAGS;	break;
+	}
+    }
 
-    if ((ba->buildAmount & RPMBUILD_RMSPEC) && bigMode == MODE_UNKNOWN)
-	bigMode = MODE_BUILD;
-    
     if (initdb) {
 	if (bigMode != MODE_UNKNOWN) 
 	    argerror(_("only one major mode may be specified"));
@@ -943,45 +823,22 @@ int main(int argc, const char ** argv)
 	    bigMode = MODE_INITDB;
     }
 
-    if (queryTags) {
-	if (bigMode != MODE_UNKNOWN) 
-	    argerror(_("only one major mode may be specified"));
-	else
-	    bigMode = MODE_QUERYTAGS;
-    }
-
     if (qva->qva_sourceCount) {
-	if (QVSource != RPMQV_PACKAGE || qva->qva_sourceCount > 1)
+	if (qva->qva_sourceCount > 1)
 	    argerror(_("one type of query/verify may be performed at a "
 			"time"));
-	QVSource = qva->qva_source;
     }
 
-    if (ba->buildRootOverride && bigMode != MODE_BUILD &&
-	bigMode != MODE_REBUILD && bigMode != MODE_TARBUILD) {
-	argerror("--buildroot may only be used during package builds");
-    }
-
-    if (gotDbpath && (bigMode & ~MODES_FOR_DBPATH))
-	argerror(_("--dbpath given for operation that does not use a "
-			"database"));
-
-    if (timeCheck && (bigMode & ~MODES_FOR_TIMECHECK))
-	argerror(_("--timecheck may only be used during package builds"));
-    
     if (qva->qva_flags && (bigMode & ~MODES_QV)) 
 	argerror(_("unexpected query flags"));
 
     if (qva->qva_queryFormat && (bigMode & ~MODES_QV)) 
 	argerror(_("unexpected query format"));
 
-    if (QVSource != RPMQV_PACKAGE && (bigMode & ~MODES_QV)) 
+    if (qva->qva_source != RPMQV_PACKAGE && (bigMode & ~MODES_QV)) 
 	argerror(_("unexpected query source"));
 
-    if (!(bigMode == MODE_INSTALL ||
-	 (bigMode==MODE_BUILD && (ba->buildAmount & RPMBUILD_RMSOURCE))||
-	 (bigMode==MODE_BUILD && (ba->buildAmount & RPMBUILD_RMSPEC))) 
-	&& force)
+    if (!(bigMode == MODE_INSTALL) && force)
 	argerror(_("only installation, upgrading, rmsource and rmspec may be forced"));
 
     if (bigMode != MODE_INSTALL && badReloc)
@@ -1090,18 +947,6 @@ int main(int argc, const char ** argv)
     if (oldPackage && !upgrade)
 	argerror(_("--oldpackage may only be used during upgrades"));
 
-    if ((ftpProxy || ftpPort) && !(bigMode == MODE_INSTALL ||
-	((bigMode == MODE_QUERY && QVSource == RPMQV_RPM)) ||
-	((bigMode == MODE_VERIFY && QVSource == RPMQV_RPM))))
-	argerror(_("ftp options can only be used during package queries, "
-		 "installs, and upgrades"));
-
-    if ((httpProxy || httpPort) && !(bigMode == MODE_INSTALL ||
-	((bigMode == MODE_QUERY && QVSource == RPMQV_RPM)) ||
-	((bigMode == MODE_VERIFY && QVSource == RPMQV_RPM))))
-	argerror(_("http options can only be used during package queries, "
-		 "installs, and upgrades"));
-
     if (noPgp && bigMode != MODE_CHECKSIG)
 	argerror(_("--nopgp may only be used during signature checking"));
 
@@ -1112,23 +957,6 @@ int main(int argc, const char ** argv)
 	argerror(_("--nomd5 may only be used during signature checking and "
 		   "package verification"));
 
-    if (ftpProxy) {
-	addMacro(NULL, "_ftpproxy", NULL, ftpProxy, RMIL_CMDLINE);
-	addMacro(&rpmCLIMacroContext, "_ftpproxy", NULL, ftpProxy, RMIL_CMDLINE);
-    }
-    if (ftpPort) {
-	addMacro(NULL, "_ftpport", NULL, ftpPort, RMIL_CMDLINE);
-	addMacro(&rpmCLIMacroContext, "_ftpport", NULL, ftpPort, RMIL_CMDLINE);
-    }
-    if (httpProxy) {
-	addMacro(NULL, "_httpproxy", NULL, httpProxy, RMIL_CMDLINE);
-	addMacro(&rpmCLIMacroContext, "_httpproxy", NULL, httpProxy, RMIL_CMDLINE);
-    }
-    if (httpPort) {
-	addMacro(NULL, "_httpport", NULL, httpPort, RMIL_CMDLINE);
-	addMacro(&rpmCLIMacroContext, "_httpport", NULL, httpPort, RMIL_CMDLINE);
-    }
-
     if (signIt) {
         if (bigMode == MODE_REBUILD || bigMode == MODE_BUILD ||
 	    bigMode == MODE_RESIGN || bigMode == MODE_TARBUILD) {
@@ -1241,87 +1069,10 @@ int main(int argc, const char ** argv)
 	
       case MODE_REBUILD:
       case MODE_RECOMPILE:
-        if (rpmGetVerbosity() == RPMMESS_NORMAL)
-	    rpmSetVerbosity(RPMMESS_VERBOSE);
-
-	if (!poptPeekArg(optCon))
-	    argerror(_("no packages files given for rebuild"));
-
-	ba->buildAmount = RPMBUILD_PREP | RPMBUILD_BUILD | RPMBUILD_INSTALL;
-	if (bigMode == MODE_REBUILD) {
-	    ba->buildAmount |= RPMBUILD_PACKAGEBINARY;
-	    ba->buildAmount |= RPMBUILD_RMSOURCE;
-	    ba->buildAmount |= RPMBUILD_RMSPEC;
-	    ba->buildAmount |= RPMBUILD_CLEAN;
-	    ba->buildAmount |= RPMBUILD_RMBUILD;
-	}
-
-	while ((pkg = poptGetArg(optCon))) {
-	    ec = rpmInstallSource("", pkg, &specFile, &cookie);
-	    if (ec)
-		break;
-
-	    ba->rootdir = rootdir;
-	    ec = build(specFile, ba, passPhrase, 0, cookie, rcfile, force, noDeps);
-	    if (ec)
-		break;
-	    free(cookie);
-	    xfree(specFile);
-	}
 	break;
 
       case MODE_BUILD:
       case MODE_TARBUILD:
-        if (rpmGetVerbosity() == RPMMESS_NORMAL)
-	    rpmSetVerbosity(RPMMESS_VERBOSE);
-       
-	switch (ba->buildChar) {
-	  /* these fallthroughs are intentional */
-	  case 'a':
-	    ba->buildAmount |= RPMBUILD_PACKAGESOURCE;
-	    /*@fallthrough@*/
-	  case 'b':
-	    ba->buildAmount |= RPMBUILD_PACKAGEBINARY;
-	    ba->buildAmount |= RPMBUILD_CLEAN;
-	    /*@fallthrough@*/
-	  case 'i':
-	    ba->buildAmount |= RPMBUILD_INSTALL;
-	    if ((ba->buildChar == 'i') && ba->shortCircuit)
-		break;
-	    /*@fallthrough@*/
-	  case 'c':
-	    ba->buildAmount |= RPMBUILD_BUILD;
-	    if ((ba->buildChar == 'c') && ba->shortCircuit)
-		break;
-	    /*@fallthrough@*/
-	  case 'p':
-	    ba->buildAmount |= RPMBUILD_PREP;
-	    break;
-	    
-	  case 'l':
-	    ba->buildAmount |= RPMBUILD_FILECHECK;
-	    break;
-	  case 's':
-	    ba->buildAmount |= RPMBUILD_PACKAGESOURCE;
-	    break;
-	}
-
-	if (!poptPeekArg(optCon)) {
-	    if (bigMode == MODE_BUILD)
-		argerror(_("no spec files given for build"));
-	    else
-		argerror(_("no tar files given for build"));
-	}
-
-	while ((pkg = poptGetArg(optCon))) {
-	    ba->rootdir = rootdir;
-	    ec = build(pkg, ba, passPhrase, bigMode == MODE_TARBUILD,
-			NULL, rcfile, force, noDeps);
-	    if (ec)
-		break;
-	    rpmFreeMacros(NULL);
-	    rpmReadConfigFiles(rcfile, NULL);
-	}
 	break;
 
       case MODE_UNINSTALL:
@@ -1398,7 +1149,7 @@ int main(int argc, const char ** argv)
 
       case MODE_QUERY:
 	qva->qva_prefix = rootdir;
-	if (QVSource == RPMQV_ALL) {
+	if (qva->qva_source == RPMQV_ALL) {
 	    if (poptPeekArg(optCon))
 		argerror(_("extra arguments given for query of all packages"));
 
@@ -1407,7 +1158,7 @@ int main(int argc, const char ** argv)
 	    if (!poptPeekArg(optCon))
 		argerror(_("no arguments given for query"));
 	    while ((pkg = poptGetArg(optCon)))
-		ec += rpmQuery(qva, QVSource, pkg);
+		ec += rpmQuery(qva, qva->qva_source, pkg);
 	}
 	break;
 
@@ -1420,7 +1171,7 @@ int main(int argc, const char ** argv)
 
 	qva->qva_prefix = rootdir;
 	qva->qva_flags = verifyFlags;
-	if (QVSource == RPMQV_ALL) {
+	if (qva->qva_source == RPMQV_ALL) {
 	    if (poptPeekArg(optCon))
 		argerror(_("extra arguments given for verify of all packages"));
 	    ec = rpmVerify(qva, RPMQV_ALL, NULL);
@@ -1428,7 +1179,7 @@ int main(int argc, const char ** argv)
 	    if (!poptPeekArg(optCon))
 		argerror(_("no arguments given for verify"));
 	    while ((pkg = poptGetArg(optCon)))
-		ec += rpmVerify(qva, QVSource, pkg);
+		ec += rpmVerify(qva, qva->qva_source, pkg);
 	}
 	break;
     }
@@ -1447,9 +1198,7 @@ int main(int argc, const char ** argv)
     freeNames();
     freeFilesystems();
     urlFreeCache();
-    if (qva->qva_queryFormat) xfree(qva->qva_queryFormat);
-    if (ba->buildRootOverride) xfree(ba->buildRootOverride);
-    if (ba->targets) free(ba->targets);
+    if (qva->qva_queryFormat) free((void *)qva->qva_queryFormat);
 
 #if HAVE_MCHECK_H && HAVE_MTRACE
     muntrace();   /* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
