@@ -1,19 +1,6 @@
-/* system-dependent definitions for fileutils programs.
-   Copyright (C) 89, 91, 92, 93, 94, 1996 Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+/**
+ * \file system.h
+ */
 
 #ifndef	H_SYSTEM
 #define	H_SYSTEM
@@ -112,6 +99,11 @@ extern int errno;
 char *getenv (const char *name);
 #endif /* STDC_HEADERS */
 
+/* XXX solaris2.5.1 has not */
+#if !defined(EXIT_FAILURE)
+#define	EXIT_FAILURE	1
+#endif
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #else
@@ -202,9 +194,9 @@ char *alloca ();
 
 /*@only@*/ void * xmalloc (size_t size);
 /*@only@*/ void * xcalloc (size_t nmemb, size_t size);
-/*@only@*/ void * xrealloc (void *ptr, size_t size);
+/*@only@*/ void * xrealloc (/*@only@*/ void *ptr, size_t size);
 /*@only@*/ char * xstrdup (const char *str);
-void *vmefail(void);
+/*@only@*/ void *vmefail(size_t size);
 
 #if HAVE_MCHECK_H
 #include <mcheck.h>
@@ -213,14 +205,17 @@ void *vmefail(void);
 
 /* Memory allocation via macro defs to get meaningful locations from mtrace() */
 #if HAVE_MCHECK_H && defined(__GNUC__)
-#define	xmalloc(_size) 		(malloc(_size) ? : vmefail())
-#define	xcalloc(_nmemb, _size)	(calloc((_nmemb), (_size)) ? : vmefail())
-#define	xrealloc(_ptr, _size)	(realloc((_ptr), (_size)) ? : vmefail())
-#define	xstrdup(_str)	(strcpy((malloc(strlen(_str)+1) ? : vmefail()), (_str)))
+#define	xmalloc(_size) 		(malloc(_size) ? : vmefail(_size))
+#define	xcalloc(_nmemb, _size)	(calloc((_nmemb), (_size)) ? : vmefail(_size))
+#define	xrealloc(_ptr, _size)	(realloc((_ptr), (_size)) ? : vmefail(_size))
+#define	xstrdup(_str)	(strcpy((malloc(strlen(_str)+1) ? : vmefail(strlen(_str)+1)), (_str)))
 #endif	/* HAVE_MCHECK_H && defined(__GNUC__) */
 
 /* Retrofit glibc __progname */
 #if defined __GLIBC__ && __GLIBC__ >= 2
+#if __GLIBC_MINOR__ >= 1
+#define	__progname	__assert_program_name
+#endif
 #define	setprogname(pn)
 #else
 #define	__progname	program_name
@@ -229,7 +224,7 @@ void *vmefail(void);
     else __progname = pn;		\
   }
 #endif
-char *__progname;
+const char *__progname;
 
 #if HAVE_NETDB_H
 #ifndef __LCLINT__
