@@ -1,13 +1,11 @@
 #include "system.h"
 
 #include "rpmlib.h"
+#include "debug.h"
 
 int main(int argc, char ** argv)
 {
-    Header h;
-    int offset;
-    int dspBlockNum = 0;			/* default to all */
-    int blockNum = 0;
+    unsigned int dspBlockNum = 0;		/* default to all */
     rpmdb db;
 
     setprogname(argv[0]);	/* Retrofit glibc __progname */
@@ -25,25 +23,28 @@ int main(int argc, char ** argv)
 	exit(1);
     }
 
-    offset = rpmdbFirstRecNum(db);
-    while (offset) {
-	blockNum++;
+    {	Header h = NULL;
+	unsigned int blockNum = 0;
+	rpmdbMatchIterator mi;
+#define	_RECNUM	rpmdbGetIteratorOffset(mi)
+
+	mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, NULL, 0);
+
+	while ((h = rpmdbNextIterator(mi)) != NULL) {
+
+	    blockNum++;
+	    if (!(dspBlockNum != 0 && dspBlockNum != blockNum))
+		continue;
 
-	if (!dspBlockNum || dspBlockNum == blockNum) {
-	    h = rpmdbGetRecord(db, offset);
-	    if (!h) {
-		fprintf(stderr, _("headerRead failed\n"));
-		exit(1);
-	    }
-	  
 	    headerDump(h, stdout, 1, rpmTagTable);
-	    fprintf(stdout, "Offset: %d\n", offset);
-	    headerFree(h);
-	}
+	    fprintf(stdout, "Offset: %d\n", _RECNUM);
     
-	if (dspBlockNum && blockNum > dspBlockNum) exit(0);
+	    if (dspBlockNum && blockNum > dspBlockNum)
+		exit(0);
+	}
+
+	rpmdbFreeIterator(mi);
 
-	offset = rpmdbNextRecNum(db, offset);
     }
 
     rpmdbClose(db);
