@@ -64,6 +64,7 @@ marc@redhat.com and ewt@redhat.com.
 */
 
 #include <stdarg.h>
+#include "debug.h"
 
 /*---------typedefs---------*/
 
@@ -162,7 +163,7 @@ usage (void)
   printf("\n\n");
   printf("DESCRIPTION:\n\n");
   printf("List the dependencies or the fully qualified class names, of the \n"
-	 "classfiles listed on the command line. \n");
+	 "classfiles listed on the command line. \n\n");
   printf("OPTIONS:\n\n");
   printf("--requires  For each class files listed in the arguments,\n"
 	 " -r         print the list of class files that would be\n"
@@ -243,7 +244,7 @@ usage (void)
 	 "assumed to be zero. \n\n"
 	 "");
   printf("EXAMPLES (Java Keywords): \n\n"
-	 "\t public static final String REVISION = \"$Revision: 2.6 $\";\n"
+	 "\t public static final String REVISION = \"$Revision: 2.6.8.3 $\";\n"
 	 "\t public static final String EPOCH = \"4\";\n"
 	 "\t public static final String REQUIRES = \"RPM_Requires: "
 	 "java(gnu.regexp.RE) java(com.ibm.site.util.Options)>=1.5\";\n"
